/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Simple plugin which provide a little info box along the left side of the Gmail
 * page. The box can be extended by other plugins to provide various information
 * to the user.
 *
 * @class ? @extends giMainPageUIPlugin
 */
var boxMain = giMainPageUIPlugin.extend({
    // Properties {
    /**
     * True if fully loaded
     * @property {Private Boolean}
     */
    _loaded: false,
    
    /**
     * The load error box
     * @property {Private boxSection}
     */
    _loadWarning: null,
    
    //}

    /**
     * The constructor.
     * @constructor ?
     */
    constructor: function() {
        giLogger.log('boxMain object created', 'boxMain');
        this.base();
    },

    /**
     * @function {} ?
     * @inheritdesc
     */
    load: function(accountData, pageBase) {
        giLogger.log('Loading boxMain for gmail', 'boxMain');
        this.base(accountData, pageBase, 'box');

        // Load the international properties for the entire gtd plugin
        giI18N.addStringBundle("box");


        // Initiate Gmail events:
        this.gmail.page.addEventListener(glPageWrapper.UI_READY, this._baseUIReady, this);
    },

    /**
     * Called on page unload, it cleans up everything
     * @function ?
     */
    unload: function() {
		
        this.base();
    },


    
    /**
     * Returns a list of the required glGmail components
     * @function {Array} ?
     * @return The required glGmail components
     */
    getRequiredComponents: function() {
        return [];
    },
    



    /**
     * Finishes all the loading that box needs. A callback to the
     * {@link glPageWrapper.UI_READY} event.
     * @function ?
     * @param {Event} evt - The UI_READY event
     */
    _baseUIReady: function(evt) {
        giLogger.log('baseUIReady for boxMain', 'boxMain');
        
        // Add the CSS // TODO make part of plugin structure
        this.gmail.page.addCSS("chrome://gtdinbox/skin/box/c/main.css"); // TODO add to manifest.xml
        
        // Initiate the box:
        var box = this.getUIObject('box');
        
        // Add listener to check for load problems (e.g. third-party cookies disabled)
        this.gmail.environment.addEventListener(glEnvironment.ENVIRONMENT_DATA_RECEIVED, this._environmentChanged, this);
                
        // Notify anyone waiting:
        this._loaded = true;
        this.dispatchEvent( {name:boxMain.LOADED} );
        
        // Move it above chat:
        if( this.prefs.getPref('force_above_chat') ) { // TODO Decide about this. 
            var doc = this.gmail.canvasDocument;
            var elChatRoot = doc.evaluate(".//div[contains(@class,'ps')]", this.gmail.getUIObject('navBar').navBar, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
            //elChatRoot = elChatRoot.parentNode.parentNode;
            elChatRoot.parentNode.insertBefore(box.el, elChatRoot);
        }
    },
    
    /**
     * Query if object fully loaded
     * @function {Boolean} ?
     * @return True if loaded
     */
    isLoaded: function(){
        return this._loaded;
    },
    
    /**
     * Notification environment changed, used for problem-detection.
     * If cannot retrieve any labels, it suggests that a load fail occured.
     * @function {} ?
     */
    _environmentChanged: function(evt) {
        // Remove the event, only needed once:
        this.gmail.environment.removeEventListener(glEnvironment.ENVIRONMENT_DATA_RECEIVED, this._environmentChanged, this);
        
        // Test for internal labels:
        var labels = this.gmail.environment.getInternalLabels();
        if( labels.length==0 ) {
            
            this.showLoadError();
            
        }
    },

    
    /**
     * A helper for giPageManager to visibly report errors in load
     * @function {} ?
     */
    showLoadError: function(e) {
        if( !this._loadWarning ) {
            this._loadWarning = this.getUIObject('loadWarning');
            this._loadWarning.setError(e);
            this.getUIObject('box').addSection(this._loadWarning, true);
        }
    },
    
    
    //}
}, {    
    /**
     * Notification the class has fully loaded
     * @event
     */
    LOADED: 'box_loaded',
    
});
boxMain.implement(giEventDispatcher); // Add event dispatching support