/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Links section
 *
 * @class ? @extends boxSection
 */
var boxLinks = boxSection.extend({
    // Properties {
    
    /**
     * The News Box UI object
     * @property {boxNews}
     */
    boxNews: null,
    
    /**
     * The 'new' icon for new posts, on the link
     * @property {Element}
     */
    _elBlogNew: null,
	
    //}

    // Methods {
    /**
     * Connects the element to the given boxMain object's page. Called
     * automatically by {@link boxMain.getUIObject}.
     * @function ?
     * @param {boxMain} main - The boxMain object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting boxLinks object to view', 'boxLinks');
        this.id = 'boxlinks',
        this.base(main);
        
        // Add Box News:
        this.boxNews = this._main.getUIObject('news');
        this.boxNews.addEventListener(boxNews.BLOG_UPDATED, this._blogUpdated, this);
        
        
        
        // Create the standard links
        
        var links = [['blog','http://blog.gtdinbox.com'], ['support','http://gtdinbox.com/support.htm'], ['workflow','http://gtdinbox.com/better_inbox.htm'], ['preferences', 'chrome://gtdinbox/content/overlay_options.xul']];
        for( var i = 0; i < links.length; i++ ) {
            var elRow = this._main.gmail.doc.createElement("DIV");
            elRow.innerHTML = "<a href='"+links[i][1]+"' target='_blank'>" + giI18N.getString('Box.links.'+links[i][0]) + "</a>";
            var data = null;
            if( links[i][0]=='blog' ) {
                data = this._addRow(links[i][0], elRow, this.boxNews.el);
                data.el.setAttribute('gtdi-link-id', links[i][0]);
            
                this._elBlogNew = this._main.gmail.doc.createElement('IMG');
                this._elBlogNew.className = 'gtdi-news-new';
                this._elBlogNew.style.display = "none";
                this._elBlogNew.src = "chrome://gtdinbox/skin/box/newitem.png";
                data.el.appendChild( this._elBlogNew );
            } else {
                data = this._addRow(links[i][0], elRow);
            }
            data.el.setAttribute('gtdi-link-id', links[i][0]);
        }
        
        
        
        
        // Set up an event listener for clicks
        this._eventsContainer.observe( this.el, 'mousedown', this._click, false );
        
        this._isExpandable(true);
        
        var elForceChatContainer = this._main.gmail.doc.createElement("DIV");
        var elForceChat = this._main.gmail.doc.createElement("INPUT");
        elForceChat.setAttribute('type', 'checkbox');
        elForceChat.checked = this._main.prefs.getPref('force_above_chat');
        elForceChat.id = 'gtdi-boxpref-force';
        elForceChatContainer.appendChild(elForceChat);
        elForceChatContainer.appendChild(this._main.gmail.doc.createTextNode('Force above Chat'));
        this._addRow('force_chat', elForceChatContainer);
        
    },
    
    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting boxLinks from view', 'boxLinks');
        
        this._eventsContainer.reset();
        this._eventsContainer = null;
        this.boxNews.disconnect();
        this._elBlogNew = null;
        
        this.base();
    },
    
    
    /**
     * Notification that a row (or child element) was clicked:
     * @function
     */
    _click: function(evt) {
        try {
            if( evt.button==2 ) return;
            var el = evt.target;
            
            // TODO remove the preference code:
            if( el.id=="gtdi-boxpref-force" ) {
                this._main.prefs.setPref('force_above_chat', el.checked, true);
                alert('Please refresh Gmail for changes to take affect.');
                return;
            }
            
            var elLink = el;
            if( elLink.nodeName!="A" ) elLink = elLink.getElementsByTagName("A")[0];
            var elRow = null;
            while( el.parentNode && !elRow ) {
                if( el==this.el ) break;
                if( el.getAttribute('gtdi-link-id') ) elRow = el;
                el = el.parentNode;
            }
            
                            
            if( elRow ) {
                // Clicked link
                var link = elLink.href;
                if( link ) {
                    if( elRow.getAttribute('gtdi-link-id')=='preferences' ) {
                        giUtil.stopEvent(evt);
                        var a = window.open(link, null, "chrome,centerscreen");
                        a.close();
                        var a = window.open(link, null, "chrome,centerscreen");
                    } else {
                        if( elRow.getAttribute('gtdi-link-id')=='blog' ) {
                            this.boxNews.unmarkNew();
                        }
                        //if( evt.button==0 ) gBrowser.selectedTab = gBrowser.addTab(link); // TODO Reduce dependency upon firefox
                    }
                }
                return false;
            }
        } catch(e) {
            giLogger.warn(e, 'boxLinks._click');
        }
    },

    
    /**
     * Notification the blog was updated. Update new symbol.
     * @function {} ?
     */
    _blogUpdated: function(evt) {
        if( evt.newItems>0 ) {
            giUIObject.addClassName(this._elBlogNew.parentNode, 'gtdi-news-bloglink-isnew');
        } else {
            giUIObject.removeClassName(this._elBlogNew.parentNode, 'gtdi-news-bloglink-isnew');
        }
        if( evt.newItems>0 ) {
            this._elBlogNew.style.display = "";
            //this._elBlogNewExpander.style.display = "";
        } else {
            this._elBlogNew.style.display = "none";
            //this._elBlogNewExpander.style.display = "none";
        }
        this._elBlogNew.title = giI18N.getString('News.blog.new.title').replace('%1', evt.newItems);
        //this._elBlogNewExpander.title = giI18N.getString('News.blog.new.title').replace('%1', evt.newItems);
        
    },

    /**
     * Expand or hide the expanded view of the section.
     * This is specific to each implementation of a section; and so should be handled by the subclass.
     * @function {Boolean} ?
     * @param {Boolean} v - If true, show as expanded. If false, hide. If undefined, return current status.
     * @return If no params, return true/false for expanded status.
     */
    expanded: function(v) {
        if( v ) {
            // Show everything
            for( var i = 0; i < this._rows.length; i++ ) {
                this._rows[i].el.style.display = "";
            }
        } else if( typeof v!="undefined" ) {
            // Hide everything
            for( var i = 0; i < this._rows.length; i++ ) {
                //if( this._rows[i].id=='blog' ) {
                //    this._rows[i].el.style.display = "";
                //} else {
                    this._rows[i].el.style.display = "none";
                //}
            }
        }
        
        return this.base(v);
    },

    /**
     * Override _showPopup, detect a call for blog, and quick start it if required
     * @function {} ?
     */
    _showPopout: function(popout, elRow, popoutData) {
        
        if( elRow.getAttribute('gtdi-box-row-id')=='blog' ) {
            if( !this.boxNews.isInitiated() ) this.boxNews.refresh();
        }
        
        this.base(popout, elRow, popoutData);
    },
    
    //}
});
