/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * A load warning
 *
 * @class ? @extends boxSection
 */
var boxLoadWarning = boxSection.extend({
    // Properties {
    
    //}

    // Methods {
    /**
     * Connects the element to the given boxMain object's page. Called
     * automatically by {@link boxMain.getUIObject}.
     * @function ?
     * @param {boxMain} main - The boxMain object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting boxLinks object to view', 'boxLinks');
        this.base(main);

        // Create the el:
        var doc = this._main.gmail.canvasDocument;
        var elMsg = doc.createElement("DIV");
        elMsg.id = "gtdi-box-load-warning";
        elMsg.innerHTML = giI18N.getString('Box.loadwarning') + " ";
        
        
        var elFAQ = doc.createElement("A");
        elFAQ.id = "gtdi-box-load-warning-faq";
        elFAQ.href = "http://www.gtdinbox.com/faq.htm#troubleshooting";
        elFAQ.setAttribute('target', 'about:blank');
        elFAQ.innerHTML = giI18N.getString('Box.loadwarning.faq');
        elMsg.appendChild( elFAQ );
        
        this._elPopout = doc.createElement("DIV");
        this._elPopout.innerHTML = giI18N.getString('Box.loadwarning.popoutdefault');
        
        this._addRow('loadwarning', elMsg, this._elPopout );
        
        // Set up an event listener for clicks
        this._eventsContainer.observe( this.el, 'click', this._click, false );
    },
    
    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting boxLinks from view', 'boxLinks');
               
        this.base();
    },
    
    
    /**
     * Notification that a row (or child element) was clicked:
     * @function
     */
    _click: function(evt) {
        var el = evt.target;
        
    },
    
    /**
     * Optionally add an error message
     * @function {}
     * @param {Error} e
     */
    setError: function(e) {
        this._elPopout.innerHTML = e.toString();
    },
    
    //}
});
