/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Links section
 *
 * @class ? @extends boxSection
 */
var boxOfflineWarning = boxSection.extend({
    // Properties {

	
    //}

    // Methods {
    /**
     * Connects the element to the given boxMain object's page. Called
     * automatically by {@link boxMain.getUIObject}.
     * @function ?
     * @param {boxMain} main - The boxMain object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting boxOfflineWarning object to view', 'boxOfflineWarning');
        this.id = 'boxofflinewarning',
        this.base(main);
        
        this._main.gmail.offline.addEventListener(glOffline.GO_ONLINE, this._offlineStateChange, this);
        this._main.gmail.offline.addEventListener(glOffline.GO_UNSTABLE, this._offlineStateChange, this);
        this._offlineStateChange();
    },
    
    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting boxOfflineWarning from view', 'boxOfflineWarning');
        
        this._main.gmail.offline.removeEventListener(glOffline.GO_ONLINE, this._offlineStateChange, this);
        this._main.gmail.offline.removeEventListener(glOffline.GO_UNSTABLE, this._offlineStateChange, this);
        
        this.base();
    },
    
    /**
     * If unstable, show our warning
     * @function {} ?
     */
    _offlineStateChange: function() {
        if( this._main.gmail.offline.isUnstable() ) {
            if( !this._elWarning ) {
                var doc = this._main.gmail.canvasDocument;
                this._elWarning = doc.createElement("DIV");
                this._elWarning.id = "gtdi-offline-warning";
                this._elWarning.innerHTML = giI18N.getString("Box.OfflineWarning");
                this.el.appendChild( this._elWarning );
            }
            this.el.style.display = "";
        } else {
            // Hide it
            this.el.style.display = "none";
        }
    },
    
    //}
});
