/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


/**
 * Base class for all plugins
 *
 * @class ?
 */
var giPlugin = giBase.extend({
    // Properties {
    /**
     * File URI for the plugin manifest file. Used for resolving paths against
     * @property {URI} ?
     */
    manifestURI: null,
    //}

    // Methods {
    /**
     * Called by the plugin loader after the page base class has been succesfully
     * instantiated and plugins are ready for loading.
     * @function ?
     * @param {giPluginAccountData} accountData - Helper object for managing access to preferences and other account data
     * @param {Object} pageBase - The page base class, so {glGmail} for full pages for example
     */
    load: function(accountData, pageBase) {
        throw new Error('Must implement load method');
    },

    /**
     * Called when the page is closed (or the plugin is shutdown for some other
     * reason).
     * @function ?
     */
    unload: function() {
        throw new Error('Must implement unload method');
    },

    /**
     * Returns the absolute path to the given filename
     * @function ?
     * @param {String} path - The path to the file you'd like resolved
     * @return The absolute path resolved against the plugin base
     */
    getPath: function(path) {
        return this.manifestURI.resolve(path);
    },
    //}
});