/**
 * Provides a simplified interface to managing preferences for plugins
 *
 * @class ?
 */
var giPluginPrefManager = giBase.extend({
    // Properties {
    /**
     * The account the pref manager is scoped to
     * @property {String} ?
     */
    _account: null,

    /**
     * The plugin the pref manager is scoped to
     * @property {String} ?
     */
    _plugin: null,

    /**
     * Reference to the pref manager
     * @property {giPrefManager} ?
     */
    _prefs: null,
    //}

    /**
     * Returns an initialized giPluginPrefManager object scoped to the given
     * plugin and account;
     * @constructor ?
     * @param {String} account - The name of the active account
     * @param {String} plugin - The name of the plugin
     */
    constructor: function(account, plugin) {
        this._account = account;
        this._plugin = plugin;
        this._prefs = Components.classes['@gtdinbox.com/pref-manager;1'].getService().wrappedJSObject;
    },

    // Methods {
    /**
     * Returns the value of the given preference
     * @function {Object} ?
     * @param {String} pref - The preference string
     * @return The value of the preference
     */
    getPref: function(pref) {
        return this._prefs.getPluginPref(this._account, this._plugin, pref);
    },

    /**
     * Returns the value of the given preference without the plugin scoping
     * @function {Object} ?
     * @param {String} pref - The preference string
     * @return The value of the preference
     */
    getGlobal: function(pref) {
        return this._prefs.getPref(this._account, pref);
    },

    /**
     * Sets the preference to the given value
     * @function ?
     * @param {String} pref - The preference string
     * @param {Object} value - The value to set the preference to
     * @param {Boolean} instantSave - If true, save immediately. 
     */
    setPref: function(pref, value, instantSave) {
        this._prefs.setPluginPref(this._account, this._plugin, pref, value, instantSave);
    },
    
    /**
     * Ensure the current prefs have all major key/values found in the defaults.
     * Used to repair potentially corrupted current settings.
     * @function {} ?
     */
    extendPluginPrefsWithDefaults: function() {
        this._prefs.extendPluginPrefsWithDefaults(this._account, this._plugin);
    },
    
    //}
});