/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The header section of the page on both the conversation and table list views
 * that contains action buttons, labeling dropdowns, and the more actions dropdown.
 * Provides a reference to the container div, methods for getting the button
 * elements, and methods for getting the dropdown glUIWidget objects.
 *
 * @class ? @extends glUIWidget
 */
var glActionsHeader = glUIWidget.extend({
    /**
     * Whether to auto flush the action buffer or not, it defaults to true
     * @property {Boolean} ?
     */
    
    // Properties {
    
    /** @property {Element} ? A reference to the div that is the root of the action header */
    root: null,
    
    /** @property {Element} ? A reference to the right-hand pagination div */
    pagination: null,

    /** @property {Element} ? The "Move To" dropdown activator button */
    moveToDropdownButton: null,

    /** @property {Element} moveToDropdownMenu The move to dropdown menu element. */
    _moveToDropdownMenu: null,

    /** @property {Element} ? A reference to the "Labels" dropdown activator button */
    labelsDropdownButton: null,

    /** @property {Element} labelsDropdownMenu The labels dropdown menu element. */
    _labelsDropdownMenu: null,


    /** @property {Element} ? A reference to the "More Actions" dropdown activator button */
    moreActionsDropdownButton: null,

    /** @property {Element} moreActionsDropdownMenu The more actions dropdown menu element. */
    _moreActionsDropdownMenu: null,


    /** @property {String} ? The current view type */
    _viewType: null,
    //}

    // Methods {
    /**
     * Connects the element to the given glGmail object's page. Called automatically
     * by {@link glGmail.getUIObject}.
     * @function ?
     * @param {glGmail} gmail - The glGmail object for the page
     */
    connect: function(gmail) {
        giLogger.log('Connecting glActionsHeader object to view', 'glActionsHeader');

        this.base(gmail);
        this._gmail = gmail;

        this._viewType = gmail.page.getActiveViewType();

        // Get view root node to do searches from
        var viewRoot = gmail.page.getViewRoot();
        if(!viewRoot) giLogger.error('Could not find view root', 'glActionsHeader');

        // Root
        var doc = gmail.canvasDocument;
        
        var result = doc.evaluate('.//div[@class="D A1"]/div[@class="nH"][1] | .//div[@class="nH"]/div[contains(@class,"iI D")]', viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        if( !result.singleNodeValue ) result = doc.evaluate('.//div[contains(@class,"A1 D")]/div[@class="nH"][1]', viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        this.root = result.singleNodeValue;

        if(!this.root) giLogger.error('Could not find root node', 'glActionsHeader');

        // Pagination
        this.pagination = this.root.childNodes[1];
    
        // moveToDropdownButton
        var result = doc.evaluate(".//div[starts-with(@class,'eeCDlc')][2]/div[starts-with(@class,'goog-imageless-button')][1]", viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        if( !result.singleNodeValue ) result = doc.evaluate(".//div[@class='sGcZQb']//div[starts-with(@class,'eeCDlc')][2]/div[starts-with(@class,'goog-imageless-button')][1] | .//div[@class='Wtoymb']//div[starts-with(@class,'eeCDlc')][2]/div[starts-with(@class,'goog-imageless-button')][1]", viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        this.moveToDropdownButton = result.singleNodeValue;
            
        // labelsDropdownButton
        var result = doc.evaluate(".//div[starts-with(@class,'eeCDlc')][2]/div[starts-with(@class,'goog-imageless-button')][2]", viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        if( !result.singleNodeValue ) result = doc.evaluate(".//div[@class='sGcZQb']//div[starts-with(@class,'eeCDlc')][2]/div[starts-with(@class,'goog-imageless-button')][2] | .//div[@class='Wtoymb']//div[starts-with(@class,'eeCDlc')][2]/div[starts-with(@class,'goog-imageless-button')][2]", viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        this.labelsDropdownButton = result.singleNodeValue;
	
        if(!this.labelsDropdownButton) giLogger.error('Could not find labels dropdown button node', 'glActionsHeader');

        // moreActionsDropdownButton
        var result = doc.evaluate(".//div[starts-with(@class,'eeCDlc')][3]/div[starts-with(@class,'goog-imageless-button')][1]", viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        if( !result.singleNodeValue ) result = doc.evaluate(".//div[@class='sGcZQb']//div[starts-with(@class,'eeCDlc')][3]/div[starts-with(@class,'goog-imageless-button')][1] | .//div[@class='Wtoymb']//div[starts-with(@class,'eeCDlc')][3]/div[starts-with(@class,'goog-imageless-button')][1]", viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        if( !result.singleNodeValue ) result = doc.evaluate(".//div[@class='QOD9Ec']", viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        this.moreActionsDropdownButton = result.singleNodeValue;
        
        if(!this.moreActionsDropdownButton) giLogger.error('Could not find more actions dropdown button node', 'glActionsHeader');


    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting glActionsHeader from view', 'glActionsHeader');
        this.root = null;

        this.base();
    },

    /**
     * Whether the UI object can be cached by {@link glGmail.getUIObject}
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return false;
    },

    /**
     * Returns the node for the button if it is in the view
     * @function {Element} ?
     * @param {int} buttonCode - The button code, which is the act attribute of the button
     * @return The button node - null if not found
     */
    getButton: function(buttonCode) {
        var result = this.root.ownerDocument.evaluate(".//div[@act='"+buttonCode+"']", this.root, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        return (result.singleNodeValue) ? result.singleNodeValue : null;
    },

    /**
     * Clicks the item in the "More Actions" dropdown with the given task
     * @function ?
     * @param {int} actionIndex - The index of the target item (use the static constants to make this easier)
     */
    clickMoreActionsItem: function(actionIndex) {
        // Open dropdown
       // giUIObject.simulateMouseEvent(this.moreActionsDropdownButton, 'mousedown');

        // Find the dropdown menu element
        var dropdownMenu = this._getDropdownMenu('moreActionsDropdown');
        //dropdownMenu.style.display = 'none'; // Prevent from flashing


        // Find the item
        var result = this.root.ownerDocument.evaluate('.//div[@role="menuitem"]['+actionIndex+']', dropdownMenu, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        var menuItem = result.singleNodeValue;


        // Click the dropdown item if found
        if(menuItem && menuItem.style.display!="none") {	// Ignore hidden items, as Gmail has deemed them unclickable in current context
            giUIObject.simulateMouseEvent(menuItem, 'mousedown');
            giUIObject.simulateMouseEvent(menuItem, 'mouseup');
        } else {
            // Hide the dropdown
            //giUIObject.simulateMouseEvent(this.moreActionsDropdownButton, 'mousedown');
        }
    },

    /**
     * Returns the dropdown menu element by triggering the menu show. Returns
     * from cache if possible.
     * @function {Element} ?
     * @param {String} dropdown - The dropdown type
     * @return The dropdown menu element
     */
    _getDropdownMenu: function(dropdown) {
        // TODO if decide to permanently remove caching, clean up
        if(this['_'+dropdown+'Menu']) {
            // Ensure it's refreshed
            var button = this[dropdown+'Button'];
            if(button.getAttribute('aria-expanded') != 'true') {
                giUIObject.simulateMouseEvent(button, 'mousedown');
                giUIObject.simulateMouseEvent(button, 'mousedown');
            }
        } else {
            // Not yet in cache, so populate the cache and then return it

            var button = this[dropdown+'Button'];
            var hideAgain = false;
            if(button.getAttribute('aria-expanded') != 'true') {
                giLogger.log('Dropdown not showing so show it', 'glActionsHeader');
                hideAgain = true;

                // Get GMail to generate the dropdown menu
                giUIObject.simulateMouseEvent(button, 'mousedown');
            }

            // Get dropdown menu
            var doc = button.ownerDocument;
            var xpathIterator = doc.evaluate(".//div[contains(@class,'goog-menu AW')]", this.root, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var elem = null;
            var dropdownMenu = null;
            while(elem = xpathIterator.iterateNext()) {
                if( (elem.style.display != 'none') && (elem.childNodes.length > 0) ) {
                    dropdownMenu = elem;
                    break;
                }
            }


            if(hideAgain) {
                // Hide the dropdown menu again
                giUIObject.simulateMouseEvent(button, 'mousedown');
                //if( dropdownMenu ) dropdownMenu.style.display = 'none'; // Needed to prevent flash, because the hide event listeners don't activate for a while
                //setTimeout(function() { giUIObject.simulateMouseEvent(button.ownerDocument.body, 'mousedown'); }, 500);
            }

            if(!dropdownMenu) {
                giLogger.error('Could not find dropdown menu node', 'glActionsHeader');
            }

            this['_'+dropdown+'Menu'] = dropdownMenu;
        }

        return this['_'+dropdown+'Menu'];
    },

    _getDropdownMenuLabels: function(dropdown) {
        var elDropdownMenu = this._getDropdownMenu(dropdown);
        var doc = elDropdownMenu.ownerDocument;
        var elDropdownMenuLabels = doc.evaluate(".//td//div[starts-with(@class,'goog-menu-content')]", elDropdownMenu, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        return elDropdownMenuLabels;
    },

    /**
     * If present, return the input box spawned by the Labels menu's Create Label function
     * @function {Element} ?
     * @return The INPUT element for Create Label
     */
    getCreateLabelInput: function() {
        var doc = this._gmail.canvasDocument;
        return doc.evaluate("//div[@class='az']/input[@class='qc']", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
    },
    

    //}
},{
    // Static Properties {
    /** @property {static int} ? Archive button action code */
    ARCHIVE_BUTTON: 7,
    /** @property {static int} ? Report spam button action code */
    REPORT_SPAM_BUTTON: 9,
    /** @property {static int} ? Delete button action code */
    DELETE_BUTTON: 10,
    /** @property {static int} ? Move to Inbox button action code */
    MOVE_TO_INBOX_BUTTON: 8,
    /** @property {static int} ? Discard drafts button action code */
    DISCARD_DRAFTS_BUTTON: 16,
    /** @property {static int} ? Delete forever button action code */
    DELETE_FOREVER_BUTTON: 17,
    /** @property {static int} ? Not spam button action code */
    NOT_SPAM_BUTTON: 18,
    /** @property {static int} ? Refresh link action code */
    REFRESH_BUTTON: 20,

    /** @property {static int} ? Mark as read menu action index */
    MARK_AS_READ: 1,
    /** @property {static int} ? Mark as unread menu action index */
    MARK_AS_UNREAD: 2,
    /** @property {static int} ? Add star menu action index */
    ADD_STAR: 3,
    /** @property {static int} ? Remove star menu action index */
    REMOVE_STAR: 4,
    /** @property {static int} ? Archive menu action index */
    ARCHIVE: 5,
    /** @property {static int} ? Not spam menu action index */
    NOT_SPAM: 6,
    /** @property {static int} ? Create event menu action index */
    CREATE_EVENT: 7,
    /** @property {static int} ? Filter messages like these menu action index */
    FILTER_MESSAGES_LIKE_THESE: 8,
    /** @property {static int} ? Mute menu action index */
    MUTE: 9,
    /** @property {static int} ? Move to drafts menu action index */
    MOVE_TO_DRAFTS: 10,
    //}
});

glActionsHeader.prototype.__defineGetter__('moveToDropdownMenu', function() { return this.moveToDropdownButton ? this._getDropdownMenu('moveToDropdown') : null; }); // Add dropdownMenu getter
glActionsHeader.prototype.__defineGetter__('labelsDropdownMenu', function() { return this._getDropdownMenu('labelsDropdown'); }); // Add dropdownMenu getter
glActionsHeader.prototype.__defineGetter__('moreActionsDropdownMenu', function() { return this._getDropdownMenu('moreActionsDropdown'); }); // Add dropdownMenu getter

glActionsHeader.prototype.__defineGetter__('labelsDropdownMenuLabels', function() { return this._getDropdownMenuLabels('labelsDropdown'); }); // Add dropdownMenu getter
glActionsHeader.prototype.__defineGetter__('moveToDropdownMenuLabels', function() { return this._getDropdownMenuLabels('moveToDropdown'); }); // Add dropdownMenu getter
