/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Abstraction layer for the navigation bar along the left side containing links
 * to the inbox, trash, and things, as well as the compose link at the top.
 *
 * @class ? @extends glUIWidget
 */
var glNavBar = glUIWidget.extend({
    // Properties {
    /** @property {Element} ? The container div for the nav bar */
    navBar: null,

    /** @property {Element} ? The compose button */
    composeLink: null,

    /** @property {Element} ? The div containing all of the major buckets (inbox, sent, trash, etc.) */
    mainBucketsContainer: null,

    //}

    // Methods {
    /**
     * Connects the element to the given glGmail object's page. Called automatically
     * by {@link glGmail.getUIObject}.
     * @function ?
     * @param {glGmail} gmail - The glGmail object for the page
     */
    connect: function(gmail) {
        giLogger.log('Connecting glNavBar object to view', 'glNavBar');

        this.base(gmail);

        var doc = gmail.canvasDocument;

        this.composeLink = doc.evaluate("//div[@class='qk']/span[@class='p9']", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        if(!this.composeLink) giLogger.error('Could not find composeLink node', 'glNavBar');
        
        this.navBar = gmail.page.getNavPaneElement();
        if(!this.navBar) giLogger.error('Could not find navBar node', 'glNavBar');

        this.mainBucketsContainer = doc.evaluate(".//div[contains(@class,'nH pp')]", this.navBar, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        if(!this.mainBucketsContainer) giLogger.error('Could not find mainBucketsContainer node', 'glNavBar');

        giLogger.log('Connected glNavBar object to view', 'glNavBar');
        
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting glNavBar from view', 'glNavBar');
        this.composeLink = null;
        this.mainBucketsContainer = null;
        this.navBar = null;

        this.base();
    },

    /**
    * Retrieve the A link for the given item (e.g. glNavBar.INBOX)
    * @function {Element} ?
    * @param {String} item - Static value to describe desired item
    * @return - The A link
    */
    getNavLink: function(item) {
        return this.navBar.ownerDocument.evaluate(".//a[contains(@href, '" + item + "')]", this.navBar, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
    },
    
    /**
    * Retrieve the container for the given item (e.g. glNavBar.INBOX)
    * @function {Element} ?
    * @param {String} item - Static value to describe desired item
    * @return - The container element
    */
    getNavItem: function(item) {
        var el = this.getNavLink(item);
        while( el && (el.className!='qi pX' && el.className.indexOf('Alfa2e')==-1) ) el = el.parentNode;
        return el;
    },

    //}
},{
    
    INBOX: '#inbox',
    STARRED: '#starred',
    CHATS: '#chats',
    SENT_MAIL: '#sent',
    DRAFTS: '#drafts',
    ALL_MAIL: '#all',
    SPAM: '#spam',
    TRASH: '#trash',
	
});
