/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The parent class for all "view" UI objects. It includes code for recording that
 * it has been connected and is automatically connected to the view when loaded
 * by {@link glGmail.getUIObject}.
 *
 * @class {abstract} ?
 */
var glUIView = giBase.extend({
    // Properties {
    /**
     * glGmail object
     * @property {glGmail} ?
     */
    _glGmail: null,

    /**
     * Whether or not it is connected. Useful for only running connect once or
     * re-connecting when it gets disconnected.
     * @property {Boolean} ?
     */
    _isConnected: null,
    //}

    /**
     * The constructor.
     * @constructor ?
     */
    constructor: function() {
        if(this.constructor === glUIView) throw new Error('Cannot be instantiated');
    },

    // Methods {
    /**
     * Connects the element to the given glGmail object's page. Called automatically
     * by {@link glGmail.getUIObject}.
     * @function ?
     * @param {glGmail} gmail - The glGmail object for the page
     */
    connect: function(gmail) {
        this._isConnected = true;
        this._glGmail = gmail;
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        this._isConnected = false;
        this._glGmail = null;
    },

    /**
     * Whether the UI object can be cached by {@link glGmail.getUIObject}
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return false;
    },

    /**
     * Returns whether or not connected
     * @function {Boolean} ?
     * @return Connection status
     */
    isConnected: function() {
        return this._isConnected;
    },
    //}
});