/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


var gtdRecent = giBase.extend({
	// Properties {
	
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	// }
	
	/**
	* @constructor ?
	* @param {gtdMainPagePlugin} gtd - The main gtd plugin for this page
	*/
	constructor: function(gtd) {
		this.base(gtd.gmail);
		this._gtd = gtd;
		
	    this._gtd.gmail.actions.addEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);
        this._gtd.gmail.environment.addEventListener(glEnvironment.RENAME_LABEL, this._labelsChanged, this);
        this._gtd.gmail.environment.addEventListener(glEnvironment.DELETE_LABEL, this._labelsChanged, this);
	},
	
	/**
	* Callback for the {@link glActionManager.THREAD_CHANGED} event. Updates all
	* stored threads from the event and removes any that no longer fit the search
	* criteria.
	* @function ?
	* @param {Event} evt - The thread changed event
	*/
	_threadChanged: function(evt) {
		if(evt.action === glActionManager.ADD_LABEL) {
			var labelname = evt.label;
			
			// Extract type:
			var foundType = "misc";
			var typeSettings = this._gtd.prefs.getPref('labels.types');
			for( t in typeSettings ) {
				if( t!="misc" && gtdLabelsData.testPrefix(labelname, typeSettings[t].prefix) ) {
					foundType = t;
					break;
				}
			}
			
			// Update Recent list:
			var recent = this._gtd.prefs.getPref('recent', true) || {};
			if( !recent[foundType] ) recent[foundType] = [];
			
			// Remove previous occurences:
			for( var i = 0; i < recent[foundType].length; i++ ) {
				if( recent[foundType][i][0]==labelname ) {
					recent[foundType].splice(i,1);
					break;
				}
			}
			// Add new label at front:
			recent[foundType].splice(0, 0, [labelname, (new Date()).valueOf()]);
			// Cap length
			if( recent[foundType].length > 5 ) recent[foundType].length = 5;
			
			this._gtd.prefs.setPref('recent', recent);
		}
	},
	
	/**
	* Get array of recent labels for a given type
	* @function {String[][]} ?
	* @param {String} type - The type of label to retrieve
	* @return Array of recent labels. Format: [[<labelname>, <dateadded>],]
	*/
	getLabels: function(type) {
		var recent = this._gtd.prefs.getPref('recent', true) || {};
		if( recent[type] ) {
			return recent[type];
		} else {
			return [];
		}
	},
	
    /**
     * Notification a label was renamed or removed - in either case, update the Recent cache
     * @function {} ?
     */
    _labelsChanged: function(evt) {
        var recent = this._gtd.prefs.getPref('recent', true) || {};
        var change = false;
        for( type in recent ) {
            for( var i = recent[type].length - 1; i>=0; i-- ) {
                if( evt.label==recent[type][i][0] ) {
                    if( evt.name==glEnvironment.RENAME_LABEL ) {
                        // Update label name:
                        recent[type][i][0] = evt.newLabel;
                        change = true;
                    } else if( evt.name==glEnvironment.DELETE_LABEL ) {
                        // Remove it
                        recent[type].splice(i,1);
                        change = true;
                    }
                }
            }
        }
        if( change ) {
            // Update prefs
            this._gtd.prefs.setPref('recent', recent);
        }
    },
	
});