/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * @class ? @extends giUIObject
 */
var gtdBrowser = giUIObject.extend({
    // Properties {
    
	/**
	* The main container
	* @property {Element} ?
	*/
	el: null,

    /**
	* The header bar
	* @property {Element} ?
	*/
	elHeader: null,
    
    /**
	* The main content
	* @property {Element} ?
	*/
	elBody: null,

    //}


	// Methods {
	/**
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdBrowser object to view', 'gtdBrowser');
		
		this.base(main);
		
		
		var doc = this._main.gmail.canvasDocument;
        
        this.el = doc.createElement("DIV");
		this.el.id = "gtdi-browser";
        this.el.className = 'gtdi-has-header';
        this.el.setAttribute("gtdi-resizable", "1");
        
        // Header
		this.elHeader = doc.createElement("DIV");
		this.elHeader.id = "gtdi-header";
		this.el.appendChild( this.elHeader );
		
        // Body
        this.elBody = doc.createElement("DIV");
		this.elBody.id = "gtdi-body";
		this.el.appendChild( this.elBody );
        
	},

	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		giLogger.log('Disconnecting gtdBrowser from view', 'gtdBrowser');
		
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		this.elHeader = null;
        this.elBody = null;
        
		this.base();
	},
	
    /**
     * Return the title
     * @function {String} ?
     * @return The title of this browser and its content
     */
    getTitle: function() {
        return "";
    },

    //}
},{
    // Static Properties {
    
    /**
     * Notification that something within the gtdBrowser requested something else to be loaded.
     * Includes parameters of 'type' (e.g. gtdBrowser.TYPES.CONTEXT), 'item' (e.g. C/Bob) and openInGmail (e.g. true).
     * Will be caught by containing/managing object to decide how to process.
     * @event
     */
    LOAD_TYPE: 'load_type',
    
    /**
     * Notification that an element _could_ have changed within the DOM, that is gtdi-resizable
     * @event
     */
    RESIZABLE_ELEMENT_CHANGE: 'resizable_element_change',
    
    TYPES: {
        STATUSES: 'statuses',
        PROJECTS: 'projects',
        CONTEXTS: 'contexts',
        REFERENCES: 'references',
        MISC: 'misc',
        SEARCH: 'search',
        VIEW: 'view',
        OVERVIEW: 'overview',
        CONTACT: 'contact',
        THREAD: 'thread',
    },
    
    
    // }
    
    // Static Methods {
    
    /**
     * Instantiate the appropriate gtdBrowser derived class for the item and type.
     * @function {gtdBrowser} ?
     * @param {gtdMainPagePlugin} gtd
     * @param {String} item - The item. Will try to parse (e.g. test prefix of a label, look for email format, or else assume search) if no type provided.
     * @param {optional String} type - If present, force the type. e.g. gtdBrowser.TYPES.CONTEXT
     * @return The gtdBrowser object for this type, loaded
     */
    createBrowser: function(gtd, item, type) {
        if( !type ) {
            // Try to interpret the type
            if( !item ) {
                // Blank, must be overview
                type = gtdBrowser.TYPES.OVERVIEW;
            } else if ( item instanceof glThread ) {
                type = gtdBrowser.TYPES.THREAD;
            } else if ( giUtil.testEmailAddress(item) ) {
                // It's an email address
                type = gtdBrowser.TYPES.CONTACT;
            } else if ( gtd.labelsData.hasLabel(item) ) {
                // It's a label, find its type
                type = gtd.labelsData.getLabelType(item);
            } else if ( gtd.views.hasView(item) ) {
                // It's a view:
                type = gtdBrowser.TYPES.VIEW;
            } else {
                // Must be search (could be a contact name, but we cannot be certain)
                type = gtdBrowser.TYPES.SEARCH;
            }
        }
        
        var b = null;
        switch(type) {
            case gtdBrowser.TYPES.STATUSES:
            case gtdBrowser.TYPES.PROJECTS:
            case gtdBrowser.TYPES.CONTEXTS:
            case gtdBrowser.TYPES.REFERENCES:
            case gtdBrowser.TYPES.MISC:
                // Load label browser
                b = gtd.getUIObject('browserThreadsLabel');
                break;
            case gtdBrowser.TYPES.SEARCH:
                b = gtd.getUIObject('browserThreadsSearch');
                break;
            case gtdBrowser.TYPES.VIEW:
                b = gtd.getUIObject('browserThreadsView');
                break;
            case gtdBrowser.TYPES.OVERVIEW:
                b = gtd.getUIObject('browserThreadsOverview');
                break;
            case gtdBrowser.TYPES.CONTACT:
                b = gtd.getUIObject('browserThreadsContact');
                break;
            case gtdBrowser.TYPES.THREAD:
                // Load thread
                b = gtd.getUIObject('browserThread');
                break;
        }
        b.load(type, item);
        return b;
    },
    
    // }
});
gtdBrowser.implement(giEventDispatcher); // Add event dispatching support
