/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * The resizable popupable container.
 *
 * @class ? @extends giUIObject
 */



var gtdBrowserPopup = giUIObject.extend({
    // Properties {
    
	/** @property {private String} ? The method used to update the server */
	_serverActionType: null,
    	    	
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
	/**
	* The popup 
	* @property {Element} ?
	*/
	el: null,
    
    /**
     * The back button and branding
     * @property {Element}
     */
    elNav: null,
    
    /**
     * The back button
     * @property {Element}
     */
    _elBack: null,
	
	/** @property {private Element} - The el that contains the gtdBrowser el */
	_elContainer: null,
	
    /**
     * The current browser in view
     * @property {gtdBrowser} -
     */
    _currentBrowser: null,
    
    /**
     * Stack of browser objects previously loaded
     * @property {gtdBrowser[]}
     */
    _browserHistory: [],
    
    /**
     * @property {giResizer}
     */
    _resizer: null,
    
    
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdBrowserPopup object to view', 'gtdBrowserPopup');
		
		this.base(main);
		this._eventsContainer = new giEventsContainer(this);
		
        this._browserHistory = [];
        
		// Create container
		var doc = main.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.className = "gtdi gtdi-popup";
		this.el.style.position = "absolute";
		
		this._elContainer = doc.createElement("DIV");
		this._elContainer.id = "gtdi-browser-container";
        this._elContainer.setAttribute("gtdi-resizable", "1");
		this.el.appendChild( this._elContainer );
		
		// Add the close:
		var elClose = doc.createElement("A");
		elClose.className = "gtdi-close";
		elClose.title = giI18N.getString("Browser.Popup.Close");
		elClose.innerHTML = giI18N.getString("Browser.Popup.Close");
		this.el.appendChild( elClose );
		
        // Add navigation and branding:
        this.elNav = doc.createElement("DIV");
        this.elNav.id = "gtdi-backhome";
        
        var elNavList = doc.createElement("UL");
        this.elNav.appendChild( elNavList );
                
        this._elBack = doc.createElement("LI");
		this._elBack.id = "gtdi-backbutton";
		this._elBack.title = giI18N.getString("ThreadViewer.Back");
		elNavList.appendChild( this._elBack );
        
        var elBrand = doc.createElement("LI");
        elBrand.id = "gtdi-homebutton";
        elBrand.title = giI18N.getString('Browser.Popup.Overview');
        elNavList.appendChild( elBrand );
        
        // Default serverActionType:
        this._serverActionType = glActionManager.USE_AJAX;
        
        // Set it hidden/off:
		this.hide();
		
		// Attach to view:
		this._main.gmail.canvasDocument.body.appendChild(this.el);
		
		// Add Event
		this._eventsContainer.observe(this.el, "mousedown", this._click, false);
        this._eventsContainer.observe(elClose, "click", this._clickClose, false);
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		giLogger.log('Disconnecting gtdBrowserPopup from view', 'gtdBrowserPopup');
		
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ) {
			if( this[p] && this[p].nodeName && this[p].nodeType ) {
				this[p] = null;
			}
		}
        
        if( this._resizer ) {
            this._resizer.disconnect();
            this._resizer = null;
        }
        
        if( this._currentBrowser ) this._currentBrowser.disconnect();
        this._currentBrowser = null;
        for( var i = 0; i < this._browserHistory.length; i++ ) {
            this._browserHistory[i].disconnect();
        }
        this._browserHistory.length = 0;
		
        		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		
		this.base();
	},
    
        
        
        
        
    /**
     * Load a given gtdBrowser into the viewport
     * @function {} ?
     * @param {gtdBrowser} browser
     */
    load: function(browser) {
                
        if( this._currentBrowser ) {
            // Remove listeners:
            this._currentBrowser.removeEventListener(gtdBrowser.LOAD_TYPE, this._loadType, this);
            this._currentBrowser.removeEventListener(gtdBrowser.RESIZABLE_ELEMENT_CHANGE, this._resizableElementChange, this);
            // Push it onto the history stack
            this._browserHistory.push(this._currentBrowser);
            // Store scroll positions:
            this._rememberScrollPositions(this._currentBrowser.el);
        }
        this._refreshBackButton();
        
        // Set the new current browser:
        this._currentBrowser = browser;
        
        // Update it to our environment:
        if( this._currentBrowser.setServerActionType ) this._currentBrowser.setServerActionType(this._serverActionType);
        
        // Inject our navigation
        this._currentBrowser.elHeader.insertBefore(this.elNav, this._currentBrowser.elHeader.firstChild);
        this._eventsContainer.observe(this._currentBrowser.elHeader, 'mousedown', this._clickHeader, false );
        
        // Clear out the current view:
        this._elContainer.innerHTML = "";
        
        // Attach the new browser:
        this._elContainer.appendChild( this._currentBrowser.el );
        
        // Attach listeners:
        this._currentBrowser.addEventListener(gtdBrowser.LOAD_TYPE, this._loadType, this);
        this._currentBrowser.addEventListener(gtdBrowser.RESIZABLE_ELEMENT_CHANGE, this._resizableElementChange, this);
        
        // Show the popup
        if( this.isActive() ) {
            // Attach resizer (wrt new layout):
            this._resizableElementChange();
        } else {
            this._show();
        }
        
        // Restore current scroll positions (if it has them):
        this._rememberScrollPositions(this._currentBrowser.el, true);
                
        // Notify:
        this.dispatchEvent({name: gtdBrowserPopup.LOADED_BROWSER, browser: this._currentBrowser});
    },
      
    /**
     * Update back button - set enabled/disabled, and set tooltip
     * @function {} ?
     */
    _refreshBackButton: function() {
        if( this._browserHistory.length > 0 ) {
            if( !giUIObject.hasClassName(this._elBack, 'gtdi-active') ) giUIObject.addClassName(this._elBack, 'gtdi-active');
            this._elBack.title = this._browserHistory[this._browserHistory.length-1].getTitle();
        } else {
            giUIObject.removeClassName(this._elBack, 'gtdi-active');
            this._elBack.title = "";
        }
    },
	
    /**
     * Either store, or restore, scroll positions on elements with the gtdi-remember-scroll attribute (within el).
     * Store on the el, in the format: el.gtdiScrollData = {elId:<int>}
     * @function {} ?
     * @param {Element} el - The element to look within for gtdi-remember-scroll elements
     * @param {Boolean} restore - If true, restore
     */
    _rememberScrollPositions: function(el, restore) {
        
        // Get the elements:
        var result = el.ownerDocument.evaluate( ".//*[@gtdi-remember-scroll]", el, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
                
        if( restore ) {
            if( el.gtdiScrollData ) {
                for( var i = 0; i < result.snapshotLength; i++ ) {
                    var elScroll = result.snapshotItem(i);
                    if( elScroll.id && el.gtdiScrollData[elScroll.id] ) {
                        elScroll.scrollTop = el.gtdiScrollData[elScroll.id];
                    }
                }
            }
        } else {
            for( var i = 0; i < result.snapshotLength; i++ ) {
                var elScroll = result.snapshotItem(i);
                if( elScroll.id ) {
                    if( !el.gtdiScrollData ) {
                        el.gtdiScrollData = {};
                    }
                    el.gtdiScrollData[elScroll.id] = elScroll.scrollTop;
                }
            }
        }
        
    },
    
    /**
     * Notification that some part of the popup was clicked
     * @function {} ?
     */
    _click: function(evt) {
        try {
            var el = evt.target;
            if( el==this._currentBrowser.elHeader ) {
                // Clicked header
                // Record start point:
                this._drag = {pointerX:evt.pageX, pointerY: evt.pageY, left:parseInt(this.el.style.left), top:parseInt(this.el.style.top)};
                // Listen to mouse move:
                this._eventsContainer.observe(this._main.gmail.canvasDocument.body, 'mousemove', this._dragMouseMove, false);
                this._eventsContainer.observe(this._main.gmail.canvasDocument.body, 'mouseup', this._dragMouseUp, false);
            } else {
                while( el.parentNode && el!=this.el ) {
                    if( el.id=='gtdi-homebutton' ) {
                        // Clicked cube - open overview
                        this.load( gtdBrowser.createBrowser(this._main) );
                        break;
                    } else if( el.id=="gtdi-backbutton" ) {
                        // Clicked back
                        if( this._browserHistory.length>0 ) {
                            // Pop end of stack:
                            var browser = this._browserHistory[this._browserHistory.length-1];
                            this._browserHistory.length = this._browserHistory.length - 1;
                            
                            // Clean up current:
                            if( this._currentBrowser ) {
                                // Ensure it's disconnected, and not stacked into history on next load
                                this._currentBrowser.disconnect();
                                this._currentBrowser = null;
                            }
                            
                            // Load previous browser:
                            this.load(browser);
                            
                            // Ensure button is accurate:
                            this._refreshBackButton();
                        }
                        break;
                    }
                    el = el.parentNode;
                }
            }
        } catch( e ) {
            giLogger.warn(e, 'gtdBrowserPopup._click');
        }
    },
    
    /**
     * Notification the close button was clicked hide it.
     * Has to be 'click' not 'mousedown' or it over selects
     * @function {} ?
     */
    _clickClose: function(evt) {
        this.hide();
    },
	
    /**
     * Notification that we need to reinit the resizer (as internal elements have changed)
     * @function {} ?
     */
    _resizableElementChange: function() {
        if( this._resizer ) {
            this._resizer.initResizableEls();   // Adjust around added/removed elements
        } else {
            this._resizer = new giResizer();
            this._resizer.connect(this.el, this._main.prefs, 'resizer', this._main.gmail); // Important that this is called after all internal elements correctly set up
        }
        
    },
    
	/**
	* Calculate the offset/delta from the start point as the mouse move
	* @function ?
	*/
	_dragMouseMove: function(event) {
		var delta = [event.pageX - this._drag.pointerX, event.pageY - this._drag.pointerY];
		this.el.style.left = (this._drag.left + delta[0]) + "px";
		this.el.style.top = (this._drag.top + delta[1]) + "px";
	},
	
	/**
	* Stop monitoring mouse move 
	* @function ?
	*/
	_dragMouseUp: function(event) {
		this._eventsContainer.stopObserving(this._main.gmail.canvasDocument.body, 'mousemove', this._dragMouseMove, false);
		this._eventsContainer.stopObserving(this._main.gmail.canvasDocument.body, 'mouseup', this._dragMouseUp, false);
	},
	
	
    
    
    
    /**
     * Notification that a sibling requested to load an item (gtdBrowser.LOAD_TYPE event)
     * @function {} ?
     */
    _loadType: function(evt) {
        if( evt.openInGmail ) {
            // Load in Gmail
            if( evt.type==gtdBrowser.TYPES.SEARCH ) {
                var search = this._main.gmail.getUIObject('searchBox');
                search.setSearch(evt.item);
                search.submit();
                this.hide();
            }
            if( evt.type==gtdBrowser.TYPES.THREAD ) {
                // Load thread in Gmail:
                var win = this.el.ownerDocument.defaultView.top;
                var path = win.location.href;
        
                if( evt.newWindow==1 ) {
                    // Open in a new window
                    var url = win.location.href.substr(0, win.location.href.indexOf("#"));
                    url += "#inbox/" + evt.item.threadId;
                    gBrowser.selectedTab = gBrowser.addTab(url);
                } else {
            
                    if( path.indexOf("#search")>-1 || path.indexOf("#label")>-1 ) {	// It's in the double format, #search/searchstr
                        var m = path.match( /#(.+?)\/(.+?)(\/[^\/]+$|$)/ );
                        if( m ) path = "#"+m[1]+"/"+m[2];
                        else path = "";
                    } else {
                        var m = path.match( /#(.+?)(\/[^\/]+$|$)/ );
                        if( m ) path = "#"+m[1];
                        else path = "";
                    }
                    if( !path ) path = "#inbox";
            
                    var hash = path + "/" + evt.item.threadId;
                    giLogger.log('Opening Thread In Gmail: ' + hash, 'gtdBrowserPopup');
                    win.location.hash = hash;
                    this.hide();
                }
            }
        } else {
            // Create a new browser:
            var browser = gtdBrowser.createBrowser(this._main, evt.item, evt.type);
            if( browser ) {
                this.load( browser );
                if( evt.threads ) browser.setThreads(evt.threads);
            }
        }
    },
    
    
    
	
    /**
     * Detect Esc key to close
     * @function {}
     */
    _keyPress: function(evt) {
        if( evt.keyCode==27 ) {
            this.hide();
        }
    },
    
	/**
     * Hide the popup
     * @function {} ?
     */
    hide: function() {
        this.el.style.display = "none";
        
        this._eventsContainer.stopObserving(this._main.gmail.canvasDocument, 'keypress', this._keyPress, false);
        
        
        // Wipe out history:
        for( var i = 0; i < this._browserHistory.length; i++ ) {
            this._browserHistory[i].disconnect();
        }
        this._browserHistory.length = 0;
        // Destroy current:
        if( this._currentBrowser ) {
            this._currentBrowser.disconnect();
            this._currentBrowser = null;
        }
        
        this._main.gmail.lightbox.hide();
        this.dispatchEvent({name: gtdBrowserPopup.HIDE});
    },

    /**
     * Return true if showing
     * @function {Boolean} ?
     * @return True if visible
     */
    isActive: function() {
        return (this.el.style.display!="none");
    },
    
    /**
     * Show the popup if not visible. Called by .load
     * @function ?
     */
    _show: function() {
        if( this.isActive() ) return; // Already visible
        this.el.style.display = "";
        
        // Resize if necessary:
        this._resizableElementChange();
        
        // Centre popup:
        var popupDims = giUIObject.getDimensions(this._elContainer);
        var viewport = giUIObject.viewport(this.el.ownerDocument);
                        
        x = (viewport[0]/2)-(popupDims.width/2);
        y = (viewport[1]/2)-(popupDims.height/2) + this.el.ownerDocument.defaultView.pageYOffset;
        
        var xy = giUIObject.boundsCheckXY(this._elContainer, x,y,true,true);
        x = xy[0];
        y = xy[1];
        
        if( typeof x!="undefined" ) this.el.style.left = x + "px";
        if( typeof y!="undefined" ) this.el.style.top = y + "px";
       
        this._main.gmail.lightbox.show();
        this._eventsContainer.observe(this._main.gmail.canvasDocument, 'keypress', this._keyPress, false);
    },
	
	
	/**
	* Set method for updating server. Ensure future created objects can use this info; and existing objects are updated.
	* @function ?
	* @param {String} type - The type. A static variable of glActionManager
	*/
	setServerActionType: function(type) {
		this._serverActionType = type;
		// TODO propogate to children
	},
	
    
	
	
	
	
	
	/**
	* Whether the UI object can be cached 
	* @function {Boolean} ?
	* @return Whether the UI can be cached
	*/
	isCacheable: function() {
		return true;
	},
	

},{
	// Static Properties:
	HIDE: 'hide',
    
    LOADED_BROWSER: 'loaded',
    
});
gtdBrowserPopup.implement(giEventDispatcher); // Add event dispatching support