/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * The container for the thread UI, which can move through multiple threads in a set
 *
 * @class ? @extends gtdBrowser
 */
var gtdBrowserThread = gtdBrowser.extend({
    // Properties {

	
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
    
	
	_elTitle: null, 
	
	_elCurrent: null,
	
	
	_threadUI: null, 
	
	_threads: null,
	
	_thread: null,
	
	
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdBrowserThread object to view', 'gtdBrowserThread');
		
		
		this.base(main);
		this._eventsContainer = new giEventsContainer(this);
		this._threads = [];
		
		// Create container
		var doc = this._main.gmail.canvasDocument;
        

		// Create header / breadcrumb
		
		var elBreadcrumb = doc.createElement("DIV");
		elBreadcrumb.className = 'gtdi-breadcrumb';
		this.elHeader.appendChild( elBreadcrumb );
						
		// Create header/paging
		var elPaging = doc.createElement("DIV");
		elPaging.id = 'gtdi-paging';
		elPaging.className = 'gtdi-btngrp';
		this.elHeader.appendChild( elPaging );
		
		elBtnGrp = doc.createElement("DIV");
		elBtnGrp.className = "gtdi-btn-grp";
		elPaging.appendChild( elBtnGrp );
		
		elBtn = doc.createElement("DIV");
		elBtn.className = "gtdi-btn";
		elBtn.innerHTML = '<div class="gtdi-btn"><b class="gtdi-btn-t"></b><b class="gtdi-btn-t2"></b><b class="gtdi-btn-m"></b><b class="gtdi-btn-b2"></b><b class="gtdi-btn-b"></b><span class="gtdi-btn-label"><span class="gtdi-paging-prev">' + giI18N.getString('Browser.Thread.Paging.prev') + '</span></span></div>';
		elBtnGrp.appendChild( elBtn );
		
		elBtn = doc.createElement("DIV");
		elBtn.className = "gtdi-btn";
		elBtn.innerHTML = '<div class="gtdi-btn"><b class="gtdi-btn-t"></b><b class="gtdi-btn-t2"></b><b class="gtdi-btn-m"></b><b class="gtdi-btn-b2"></b><b class="gtdi-btn-b"></b><span class="gtdi-btn-label"><span class="gtdi-paging-current"></span></span></div>';
		elBtnGrp.appendChild( elBtn );
		this._elCurrent = doc.evaluate(".//span[@class='gtdi-paging-current']", elBtn, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
		
		elBtn = doc.createElement("DIV");
		elBtn.className = "gtdi-btn";
		elBtn.innerHTML = '<div class="gtdi-btn"><b class="gtdi-btn-t"></b><b class="gtdi-btn-t2"></b><b class="gtdi-btn-m"></b><b class="gtdi-btn-b2"></b><b class="gtdi-btn-b"></b><span class="gtdi-btn-label"><span class="gtdi-paging-next">' + giI18N.getString('Browser.Thread.Paging.next') + '</span></span></div>';
		elBtnGrp.appendChild( elBtn );
		
		
		// Create body
		this._threadUI = this._main.getUIObject('thread');
		this.elBody.appendChild( this._threadUI.el );
		
		
		// Add click handlers:
		this._eventsContainer.observe(this.el, 'mousedown', this._click, false);
		this._threadUI.addEventListener(gtdThread.CLICK_OPEN, this._clickOpen, this);
                
        // Add Labelling Box handlers:
        this._threadUI.getLabellingBox().addEventListener(gtdLabellingBox.CLICK_ARCHIVE, this._requestArchive, this);
        
        
        this._main.gmail.actions.addEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);

	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		giLogger.log('Disconnecting gtdBrowserThread from view', 'gtdBrowserThread');
		
	       
        this._main.gmail.actions.removeEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);
        
        this._threadUI.getLabellingBox().removeEventListener(gtdLabellingBox.CLICK_ARCHIVE, this._requestArchive, this);
        this._threadUI.disconnect();
        this._threadUI = null;

        this._elTitle = null;
        this._elCurrent = null;
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		this._threads = null;
		this._thread = null;
		
		this.base();
	},
	
	/**
	* Specify the thread set to show
	* @function ?
	* @param {String} type - The type
	* @param {glThread} item - The thread to start on
	*/
	load: function(type, item) {
		this._threads.length = 0;
		this._loadThread(item);
	},
	
	/**
	* Load an individual thread within a thread set
	* @function {glThread}?
	* @param {glThread} thread
	*/
	_loadThread: function(thread) {
        if( !thread ) return null;
        
        this._thread = thread;
        this._threadUI.load(thread);
        this._setPaging();
        
        
        return thread;
	},
	
    /**
     * Load a collection of threads that we can page through.
     * @function {} ?
     * @param {optional glThread[]} threads - The set of threads
     */
    setThreads: function(threads) {
        this._threads = threads || [];
        var elPaging = this.elHeader.ownerDocument.evaluate(".//*[@id='gtdi-paging']", this.elHeader, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
		elPaging.style.display = (this._threads.length==0)? 'none' : '';
        this._setPaging();
    },
    
	/**
	* Update position (X of Y) on the paging UI
	* @function {} ?
	*/
	_setPaging: function() {
		for( var i = 0; i < this._threads.length; i++ ) {
			if( this._threads[i].threadId==this._thread.threadId ) {
				this._elCurrent.innerHTML = (i+1) + ' ' + giI18N.getString('Browser.Thread.Paging.of') + ' ' + this._threads.length;
				break;
			}
		}
	},
	
	/**
	* Move to the next available thread in the thread set
	* @function {} ?
	*/
	_pagingNext: function() {
        var oldThreadId = this._thread.threadId
		for( var i = 0; i < this._threads.length; i++ ) {
			if( this._threads[i].threadId==oldThreadId ) {
				if( this._threads[i+1] ) {
					this._threads[i+1] = this._loadThread(this._threads[i+1]);
                    this.dispatchEvent({name: gtdBrowserThread.MOVE_NEXT, thread:this._threads[i+1], index:(i+1), oldThreadId: oldThreadId});
					break;
				}
			}
		}
	},
	
	/**
	* Move to the previous available thread in the thread set
	* @function {} ?
	*/
	_pagingPrev: function() {
        var oldThreadId = this._thread.threadId;
		for( var i = 0; i < this._threads.length; i++ ) {
			if( this._threads[i].threadId==oldThreadId ) {
				if( this._threads[i-1] ) {
					this._threads[i-1] = this._loadThread(this._threads[i-1]);
                    this.dispatchEvent({name: gtdBrowserThread.MOVE_PREVIOUS, thread:this._threads[i-1], index:(i-1), oldThreadId: oldThreadId});
					break;
				}
			}
		}
	},
	
	_click: function(event) {
		try {
			var el = event.target;
		
			while( el.parentNode ) {
				if( el.className=='gtdi-btn' ) {
                    giUtil.stopEvent(event);
					// Clicked a button, see if it contains prev or next
					var elBtn = el.ownerDocument.evaluate(".//*[contains(@class,'gtdi-paging-prev') or contains(@class, 'gtdi-paging-next')]", el, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
					if( elBtn ) {
						if( elBtn.className=='gtdi-paging-prev' ) {
							giUtil.stopEvent(event);
							this._pagingPrev();
							break;
						}
						if( elBtn.className=='gtdi-paging-next' ) {
							giUtil.stopEvent(event);
							this._pagingNext();
							break;
						}
					}
				}
				el = el.parentNode;
			}
		} catch(e) {
			giLogger.warn(e, 'gtdBrowserThread._click', this._main.gmail);
		}
	},
	
	/**
	* Set method for updating server. Ensure future created objects can use this info; and existing objects are updated.
	* @function ?
	* @param {String} type - The type. A static variable of glActionManager
	*/
	setServerActionType: function(type) {
		// Propage to the thread
		this._threadUI.setServerActionType(type);
	},
	
	
	
	/**
	* Respond to user clicking 'Open Mail', propogate
	* @function {} ?
	*/
	_clickOpen: function(evt) {
		this.dispatchEvent({name: gtdBrowser.LOAD_TYPE, type: gtdBrowser.TYPES.THREAD, item: evt.thread, openInGmail: true, newWindow: evt.newWindow});
	},
        
    /**
     * Notification that the user clicked Archive
     * @function {} ?
     */
    _requestArchive: function(evt) {
        
        var threadId = this._thread.getRecentId();
        if( evt.moveNext ) {
            this._pagingNext();
        }
        this._main.gmail.actions.performAction(glActionManager.ARCHIVE, threadId, glActionManager.USE_AJAX);
        
        /*
        switch(evt.name) {
            case gtdLabellingBox.USER_ARCHIVE_COMPLETE:
                this.dispatchEvent({name: gtdBrowserThread.USER_ARCHIVE_COMPLETE, threadId: evt.threadId});
                break;
            case gtdLabellingBox.USER_ARCHIVE_MOVENEXT_COMPLETE:
                // Move to the next item automatically. Beware this will fire a MOVE_NEXT event.
                this._pagingNext();
                this.dispatchEvent({name: gtdBrowserThread.USER_ARCHIVE_MOVENEXT_COMPLETE, threadId: evt.threadId});
                break;
        }
        */
    },
    
    /**
     * Notification from action manager that a thread changed
     * 
     * As it is likely our threads have come from ThreadListCache we need to exercise caution.
     * ThreadListCache cannot be expected to maintain each thread it distributes via getThread/getThreads
     * In particular, if the tl refreshes while we have this popup open; the ThreadListCache will refresh; and no longer store reference to the threads we're using.
     * Therefore, we need to call updateFromEvent for each thread we hold.
     * 
     * @function {} ?
     */
    _threadChanged: function(event) {
        
        // TODO This is a really slow way to find a thread; and is used a lot.
        // Really need a way to find a thread with an ID in a list. SimpleThreadContainer? updateFromEvent
        // Would be used by gtdBrowserThread, ThreadContainer.
        // Not that easy, as we have multiple IDs for a thread.
        // Could have a single thread object for each thread (managed statically)
        for( var i = 0; i < this._threads.length; i++ ) {
            if( this._threads[i].hasId( event.thread ) ) {
                this._threads[i].updateFromEvent(event);
                break;
            }
        }
        
        giLogger.log('thread changed', 'gtdConversationView._threadChanged');
        if( event.action===glActionManager.ADD_LABEL ) {
            var statusData = this._main.labelsData.getStatusData();
            if( event.label==statusData.finished ) {
                // Finished label, is it our thread?
                if( this._thread.hasId(event.thread) ) {
                    giLogger.log('thread finished: ' + this._thread.getRecentId(), 'gtdConversationView._threadChanged');
                    
                    // Archive it, and move next
                    var threadId = this._thread.getRecentId();
                    this._pagingNext(); // Move next first, or else Previewer will remove the Popup (because current thread ID becomes invalid)
                    this._main.gmail.actions.performAction(glActionManager.MARK_AS_READ, threadId, glActionManager.USE_AJAX);
                    this._main.gmail.actions.performAction(glActionManager.ARCHIVE, threadId, glActionManager.USE_AJAX);
                    
                }
            }
        }
        
    },
	
	
    /**
     * Return the current threads
     * @function {glThread[]} ?
     * @return The current threads
     */
    getThreads: function() {
        return this._threads;
    },
    
    /**
     * Return the current thread
     * @function {glThread} ?
     * @return The current thread
     */
    getThread: function() {
        return this._thread;
    },
    
    /**
     * @function {String} ?
     */
    getTitle: function() {
        return this._thread? this._thread.subject : "";
    },

},{
    
    MOVE_NEXT: 'threadNext',
    MOVE_PREVIOUS: 'threadPrevious',
    
    /**
     * Move to the next thread
     * @event ?
     */
    //USER_ARCHIVE_MOVENEXT_COMPLETE: 'archive_move_next',
    
    /**
     * User clicks Archive/Finish (but not next)
     * @event ?
     */
    //USER_ARCHIVE_COMPLETE: 'archive_thread',
    
});
gtdBrowserThread.implement(giEventDispatcher); // Add event dispatching support