/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Base class for threads-containing-entities (e.g. a label)
 *
 * TODO If build up a big Back history in Popup, there will be a lot of listeners. If notice slowdown, find a way to enable/disable.
 *
 * @class ? @extends gtdBrowser
 */
var gtdBrowserThreads = gtdBrowser.extend({
    // Properties {
    
        
	/** @property {private String} ? The method for updating the server */
	_serverActionType: null,
    	
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
    
    
    /** @property {private Element} ?  */
	_elHeaderType: null,
	
	/** @property {private Element} ?  */
	_elHeaderItem: null,
	
    /** @property {private Element} ?  */
    _elHeaderLink: null, 
	
    /** @property {private Element} ?  */
    _elGroupByList: null,
    
	/** @property {private Element} ? */
	_elTabs: null,
	
	/** @property {private Element} ? */
	_elTabsList: null,
	
	/** @property {private Element} ?  */
	_elWarning: null,
	
    _elContent: null,
    
    /**
     * Optional cache of glThread objects from statusThreadsContainer, using our searchConditions
     * @property {glThread[]}
     */
    _filteredStatusThreads: null,
    
    /**
     * The main search for the item of this type
     * @property {glSearchConditions}
     */
    searchConditions: null,
    

    /**
     * A hash of tab data
     * Points to a data object. For Views, this would be a gtdBrowserThreads_ThreadListTab object.
     * @property {Object{}}
     */
    tabs: null,
	
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdBrowserThreads object to view', 'gtdBrowserThreads');
		
		this._eventsContainer = new giEventsContainer(this);
		this.base(main);
        
		
		var doc = main.gmail.canvasDocument;
        
        
		// Header / Items
        var elHeaderItem = doc.createElement("DIV");
		elHeaderItem.id = "gtdi-context";
        this.elHeader.appendChild( elHeaderItem );
		
		this._elHeaderType = doc.createElement("H1");
		elHeaderItem.appendChild( this._elHeaderType );
		
		this._elHeaderItem = doc.createElement("SPAN");
		elHeaderItem.appendChild( this._elHeaderItem );
        
        this._elHeaderLink = doc.createElement("IMG");
        this._elHeaderLink.id = 'gtdi-open-in-gmail';
        this._elHeaderLink.src = 'chrome://gtdinbox/skin/gtd/goto.png';
        this._elHeaderLink.title = giI18N.getString('Browser.Popup.openInGmail');

		
		// Header / GroupBy
        var elGroupBy = doc.createElement("DIV");
		elGroupBy.id = "gtdi-groupby";
        this.elHeader.appendChild( elGroupBy );
		
				
    		
		// Body / Tabs
		this._elTabs = doc.createElement("DIV");
		this._elTabs.id = "gtdi-menu";
		this.elBody.appendChild( this._elTabs );
		
		this._elTabsList = doc.createElement("UL");
		this._elTabs.appendChild( this._elTabsList );
		
		
		// Body / Thread
		this._elContent = doc.createElement("DIV");
        this._elContent.id = "gtdi-content";
		this.elBody.appendChild( this._elContent );
	
    
		// Add event listeners
		this._eventsContainer.observe( this.el, 'click', this._click, false );
        
        // Listen to statusThreadsContainer, which will invalidate our cache:
        var listeners = [gtdThreadContainer.NEW_SEARCH, gtdThreadContainer.LOADED, gtdThreadContainer.LOADING_UPDATE, gtdThreadContainer.CHANGED];
        for( var i = 0; i < listeners.length; i++ ) this._main.statusThreadsContainer.addEventListener(listeners[i], this._threadContainerChanged, this);
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		giLogger.log('Disconnecting gtdBrowserThreads from view', 'gtdBrowserThreads');
	    
        		
		// Wipe references to elements
		for( p in this ) {
			if( this[p] && this[p].nodeName && this[p].nodeType && this[p]!=this.el ) {
				this[p] = null;
			}
		}
               
        var listeners = [gtdThreadContainer.NEW_SEARCH, gtdThreadContainer.LOADED, gtdThreadContainer.LOADING_UPDATE, gtdThreadContainer.CHANGED];
        for( var i = 0; i < listeners.length; i++ ) this._main.statusThreadsContainer.removeEventListener(listeners[i], this._threadContainerChanged, this);
        
        for( i in this.tabs ) {
            if( this.tabs[i] instanceof gtdBrowserThreads_ThreadListTab ) {
                this.tabs[i].unload();
            }
        }
        this.tabs = null;
		
        
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
        this.searchConditions = null;
        this._filteredStatusThreads = null;
		
		this.base();
	},
	
    
    
	
	/**
	* Initiate the loading of this viewer.
	* Give tabs their initial counts and pass in an item filter for this item; rely on tabs to pass back a SELECT_TAB for the first item, which finishes the loading in ._tabChange
	* @function ?
	* @param {String} type - The type. Based on gtdBrowserThreads static properties
	* @param {String} item - The item. Depends on type (e.g. for gtdBrowser.TYPES.STATUSES it will be a status label. For gtdBrowser.TYPES.CONTACT it will be a name/email)
	* @param {optional glSearchConditions} searchConditions - The search for this item/type (created by a subclass)
	*/
	load: function(type, item, searchConditions) {
		
		if( this._elWarning ) {
			this._elWarning.parentNode.removeChild( this._elWarning );
			this._elWarning = null;
		}
		
		
		// Clear the tabs:
		this._elTabsList.innerHTML = "";
		
        // Store variables:
		this._type = type || this._type;
		this._item = item || this._item;
        this.searchConditions = new glSearchAndConditions();   // Wrap in AND if we want to append more items dynamically
        if( searchConditions ) this.searchConditions.addCondition(searchConditions);
        
		// Update the title
		this.setHeaderType(type, item);
		
		
        this.tabs = {};
        
        // Construct the basic search:
        // Handled by children
		
	},
	
    
	/**
	* Set the titles
	* @function ?
	* @param {String} type - The type of item, as defined in the static properties of gtdBrowserThreads
	* @param {String} item - The name of the item (e.g. a label or a contact email)
	*/
	setHeaderType: function(type, item) {
        
        // Set the main type:
        this._elHeaderType.innerHTML = giI18N.getString("Browser.Types." + type, type);
        
        // Set the item text:
        
        switch( type ) {
            case gtdBrowser.TYPES.STATUSES:
            case gtdBrowser.TYPES.PROJECTS:
        	case gtdBrowser.TYPES.CONTEXTS:
            case gtdBrowser.TYPES.REFERENCES:
            case gtdBrowser.TYPES.MISC:
            case gtdBrowser.TYPES.CONTACT:
            case gtdBrowser.TYPES.SEARCH:
                this._elHeaderItem.innerHTML = item;
                break;
            case gtdBrowser.TYPES.VIEW:
                this._elHeaderItem.innerHTML = this._main.views.getViewName(item);
                break;
            case gtdBrowser.TYPES.OVERVIEW:
                this._elHeaderItem.innerHTML = "";
                break;
        }
        
        if( this._elHeaderItem.innerHTML=="" ) {
            this._elHeaderItem.style.display = "none";
            this._elHeaderLink.style.display = "none";
        } else {
            this._elHeaderType.innerHTML += ":";
            this._elHeaderItem.appendChild( this._elHeaderLink );
            this._elHeaderLink.style.display = "";
            this._elHeaderLink.setAttribute('gtdSearch', this.searchConditions.toString());
        }
              
        
	},
    
	
	/**
	* Set method for updating server. Ensure future created objects can use this info; and existing objects are updated.
	* @function ?
	* @param {String} type - The type. A static variable of glActionManager
	*/
	setServerActionType: function(type) {
		this._serverActionType = type;
        
        for( i in this.tabs ) {
            if( this.tabs[i].threadList ) {
                this.tabs[i].threadList.setServerActionType( type );
            }
        }
	},
	
	/**
     * Notification of a click event
     * @function {} ?
     */
    _click: function(evt) {
        try {
            var el = evt.target;
            
            while( el.parentNode && el!=this.el ) {
                // Test for a tab:
                if( el.getAttribute("gtdi-tabid") ) {
                    this.selectTab( el.getAttribute("gtdi-tabid") );
                    break;
                }
                if( el.id=="gtdi-open-in-gmail") {
                    this.dispatchEvent({name:gtdBrowser.LOAD_TYPE, type: gtdBrowser.TYPES.SEARCH, item: el.getAttribute('gtdSearch'), openInGmail:true});
                    break;
                }
                
                // Go up tree:
                el = el.parentNode;
            }
        } catch(e) {
            giLogger.warn(e, 'gtdBrowserThreads._click');
        }
    },
    
    
    /**
     * Notification that the thread container was updated
     * Could be a LOADING_UPDATE, LOADED or CHANGED event
     * @function {} ?
     */
    _threadContainerChanged: function(evt) {
        // Invalidate our cache
        this._filteredStatusThreads = null;
    },
    
    
    /**
     * Return a subset array of the gtd.statusThreadsContainer threads, filtered on our searchConditions
     * Throws an error if the query cannot run matchThread
     * Need to handle edge case where there are no status threads, which would fail to throw an error if matchThread fails.
     * @function {glThread[]} ?
     * @return Array of glThreads filtered from statusThreadContainer
     */
    getFilteredStatusThreads: function() {
        
        if( !this._filteredStatusThreads ) {
            var filteredStatusThreads = [];
        
            var threads = this._main.statusThreadsContainer.threads;
            if( threads.length==0 ) {
                // Edge case - try and make it crash if matchThread does not work
                if( !this.searchConditions.canMatchThreads() ) {
                    throw new Error("Impossible match in search conditions: " + this.searchConditions.toString());
                }
            } else {
                for( var i = 0; i < threads.length; i++ ) {
                    if( this.searchConditions.matchThread(threads[i]) ) {
                        filteredStatusThreads.push(threads[i]);
                    }
                }
            }
            
            // Set only if no errors:
            this._filteredStatusThreads = filteredStatusThreads;
        }
        
        return this._filteredStatusThreads || [];
    },
        
    
    
    /**
     * Add a row to the tab list
     * @function {} ?
     * @paramset Generic
     * @param {String} id - The unique identifier for the row
     * @param {String} displayName - The name to show on the tab
     * @param {Boolean} incCount - If true, include a count box
     * @param {optional Object} tabData - Tab data to store
     * @paramset ThreadListTab
     * @param {String} id - The unique identifier for the row
     * @param {String} displayName - The name to show on the tab
     * @param {Boolean} incCount - If true, include a count box
     * @param {optional gtdBrowserThreads_ThreadListTab} tabData - Tab data to store
     */
    addTabRow: function(id, displayName, incCount, tabData) {
        // Create the element:
        var doc = this._main.gmail.canvasDocument;
        var elTab = doc.createElement("LI");
		elTab.className = 'gtdi-menu-item';
		elTab.setAttribute('gtdi-tabid', id);
		
		elTab.innerHTML = "<div><span class='gtdi-menuitem'>" + displayName + "</span></div>";
        if( incCount ) elTab.innerHTML += "<span class='gtdi-count></span>";
        
        this._elTabsList.appendChild( elTab );
        
        // Store the data:
        this.tabs[id] = tabData || {};
        this.tabs[id].el = elTab;
        
        if( tabData instanceof gtdBrowserThreads_ThreadListTab ) {
            // Set initial count:
            tabData.refreshCount();
        }
    },
    
    /**`
     * Mark the tab as selected and fire notification event.
     * It is expected that child classes will override this function to provide specific action.
     * @function {} ?
     * @param {String} id - The tab id
     */
    selectTab: function(id) {
        
        if( !id ) {
            // Attempt to get first non-zero item. If that fails, use first.
            for( t in this.tabs ) {
                if( this.tabs[t] instanceof gtdBrowserThreads_ThreadListTab ) {
                    if( this.tabs[t].getThreads(true).length>0 ) {
                        id = t;
                        break;
                    }
                }
            }
            if( !id ) {
                // Get first:
                if( !this._elTabsList.firstChild ) return;
                id = this._elTabsList.firstChild.getAttribute("gtdi-tabid");
            }
        }
                
        // Clear the content:
        this._elContent.innerHTML = "";
        
        // Update the selected item:
        var doc = this._main.gmail.canvasDocument;
        // Get the current tab:
        var elTab = doc.evaluate(".//li[contains(@class, 'gtdi-active')]", this._elTabsList, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        if( elTab ) {
            // Strip it of current:
            giUIObject.removeClassName( elTab, 'gtdi-active' );
        }
        
        // Get the newly selected tab:
        var elTab = doc.evaluate(".//li[@gtdi-tabid='"+id+"']", this._elTabsList, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        if( elTab ) {
            giUIObject.addClassName( elTab, 'gtdi-active');
        }
        
        
        // See if we can do some standard logic for a threadlist:
        if( this.tabs[id] instanceof gtdBrowserThreads_ThreadListTab ) {
            if( !this.tabs[id].threadList ) {
                // Instantiate
                this.tabs[id].threadList = this._main.getUIObject('threadList');
                
                // Add listener
                this.tabs[id].threadList.addEventListener(gtdBrowser.LOAD_TYPE, this._loadType, this);
                
                this.tabs[id].threadList.setServerActionType(this._serverActionType);
                
                // Fire notification:
                this.dispatchEvent({name: gtdBrowserThreads.CREATE_THREADLIST, threadList: this.tabs[id].threadList});
            }
            
            // Set threads:
            this.tabs[id].refreshThreadList();
            
            // Set to current view:
            this._elContent.appendChild( this.tabs[id].threadList.el );
        }
        
        
        // Dispatch notification:
        this.dispatchEvent({name: gtdBrowser.RESIZABLE_ELEMENT_CHANGE});
        this.dispatchEvent({name: gtdBrowserThreads.SELECT_TAB, tabId: id});
    },

    
    /**
     * A helper function to create a set of tabs from views in gtd.views
     * @function {} ?
     */
    _createTabsFromViews: function() {
        var views = this._main.views.getViews();    // Array of view ids
        
        for( var i = 0; i < views.length; i++ ) {
            
            // Add data to mark it as a threads item:
            var id = views[i];
            var tabData = new gtdBrowserThreads_ThreadListTab(this._main, this, id, this._main.views.getViewSearch(id));
            // Create tab element:
            this.addTabRow( id, this._main.views.getViewName(id), true, tabData ); // TODO need to localise
        }
        
        
    },
    
    /**
     * A helper function to create a threadlist tab
     * @function {gtdBrowserThreads_ThreadListTab} ?
     * @param {String} id - The unique id of the tab. Must correspond to a locale string.
     * @param {String} search - The search string for the tab.
     * @return The threadlist tab object created
     */
    _createThreadListTab: function(id, search) {
        var tabData = new gtdBrowserThreads_ThreadListTab(this._main, this, id, glSearchConditions.createFromSearchString(search));
        this.addTabRow( id, giI18N.getString('Browser.Threads.Tabs.'+id, id), true, tabData );
        return tabData;
    },
    
    
    
    /**
     * Update the tab UI to a new count
     * @function {} ?
     * @param {String} id - The tab id
     * @param {String} count - The count (numeric or ?)
     */
    setTabCount: function(id, count) {
        
        // Get the tab element:
        var elTabCount = this._main.gmail.canvasDocument.evaluate(".//li[@gtdi-tabid='"+id+"']/span[@class='gtdi-count']", this._elTabsList, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        if( elTabCount ) {
            elTabCount.innerHTML = count;
        }
    },
    
    /**
     * Return the id of the active tab
     * @function {String} ?
     * @return The ID of the active tab
     */
    getActiveTab: function() {
        var elTab = this._main.gmail.canvasDocument.evaluate(".//li[contains(@class,'gtdi-active')]", this._elTabsList, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        return elTab.getAttribute("gtdi-tabid");
    },
    
    /**
     * Notification that something was requested to be loaded (e.g. openingmail)
     * Propogate
     * @function {}
     */
    _loadType: function(evt) {
        this.dispatchEvent(evt);
    },
    

}, {
	
	// Events {
	
    
    /**
     * Notification a tab was selected
     * @event
     */
    SELECT_TAB: 'select_tab',
    
    /**
     * Notification that thread list was created for a tab
     * @event
     */
    CREATE_THREADLIST: 'create_threadlist',
    
	//}


    // Static Properties {

	DEFAULT_NUMBER_THREADS: 200,
	
	
	// }
	
});
gtdBrowserThreads.implement(giEventDispatcher); // Add event dispatching support


/**
 * A helper class for a tab dedicated to ThreadLists (which would be most Views).
 * As a helper class, there are quite close ties between gtdBrowserThreads and this.
 * 
 * @class
 */
var gtdBrowserThreads_ThreadListTab = giBase.extend({
    
    /**
     * The ID
     * @property {String}
     */
    id: null,
    
    /**
     * The tab element
     * @property {Element}
     */
    el: null,
    
    /**
     * The browser
     * @property {gtdBrowserThreads}
     */
    _browser: null,
    
    /**
     * The thread container created for this tab.
     * Could be null. 
     * @property {gtdThreadContainer}
     */
    _threadContainer: null,
    
    /**
     * The thread container in use. Could be this tabs, statusThreadsContainer, or the browser
     * @property {gtdThreadContainer}
     */
    _currentThreadContainer: null,
    
    /**
     * The search filter for the tab.
     * @property {glSearchConditions}
     */
    searchConditions: null,
    
    /**
     * Our filtered threads
     * @property {glThread[]}
     */
    _threads: null,
    
    /**
     * The ThreadList UI. Lazy loaded.
     * @property {gtdThreadList}
     */
    threadList: null,
    
    /**
     * Cache whether our searchConditions are critically dependent upon a Status label
     * @property {Boolean}
     */
    __isStatusConstrained: null,
    
    _gtd: null,
    
    /**
     * The constructor
     * @function {} ?
     * @param {gtdMainPagePlugin} gtd
     * @param {gtdBrowserThreads} browser
     * @param {String} id
     * @param {glSearchConditions} searchConditions
     */
    constructor: function(gtd, browser, id, searchConditions) {
        this._gtd = gtd;
        this._browser = browser;
        this.id = id;
        this.searchConditions = searchConditions;
        
        this._gtd.views.addEventListener(gtdViews.VIEW_EDITED, this._viewEdited, this);
        
    },
    
    /**
     * Called to clean up when no longer in use.
     * @function {} ?
     */
    unload: function() {
        if( this._currentThreadContainer ) {
            // Remove listeners
            var listeners = [gtdThreadContainer.NEW_SEARCH, gtdThreadContainer.LOADED, gtdThreadContainer.LOADING_UPDATE, gtdThreadContainer.CHANGED];
            for( var i = 0; i < listeners.length; i++ ) this._currentThreadContainer.removeEventListener(listeners[i], this._threadContainerChanged, this);
            this._currentThreadContainer = null;
        }
        if( this._threadContainer ) {
            this._threadContainer.unload();
            this._threadContainer = null;
        }
        if( this.threadList ) {
            this.threadList.disconnect();
            this.threadList = null;
        }
        
        this._threads = null;
        
        this._gtd.views.removeEventListener(gtdViews.VIEW_EDITED, this._viewEdited, this);
        
        
        this.searchConditions = null;
        this._browser = null;
        this._gtd = null;
    },
    
    /**
     * Return an array of threads, filtered both on the browser's searchConditions and this tab
     * Attempt to utilise statusThreadsContainer wherever possible for optimal performance.
     *
     * There are two main search definitions that a tab must include:
     *  The browser's main search, and filter search on the tab.
     *  These searches act over a gtdThreadContainer.
     *  
     * If possible, we try to use gtdThreadContainer as this does not require anything to be downloaded.
     *  This can only happen if the tab is a statusConstrained search, and the main search does not throw an Impossible Match.
     *  I.e. if it can be confidently processed by matchThread over the statusThreadsContainer data.
     *
     * (There is an exception we have not yet catered for - where browser.searchConditions is a statusConstrained search; and tab.searchConditions does not throw an Impossible Match error)
     *
     * If this fails, then we create a new gtdThreadContainer for the tab, combining browser+tab search.
     *  E.g. for 'Recent' items, or 'Files'.
     *  It raises the question of whether it would be better to create a browser.threadContainer, and then filter that with tab.searchConditions; so that tabs can download and share a single threadContainer.
     *      Not only does this add code complexity, it is not guarenteed to offer efficiency improvements.
     *      E.g. if the tab is 'has:attachment', and the browser search is 'P/Automation', the browser.threadContainer would just be 'P/Automation', and thus would need to download a lot of unwanted threads just to get the ones with attachments.
     *
     * If the browser searchConditions change, then we must invalidate tab.threadContainer.
     *  There may be room to temporarily filter it (using matchThread) until the user wants to download more (at which point it will reset); but this may prove too complex; and an alternate UI for rapid refinement may be needed.
     * 
     * @function {glThread[]} ?
     * @param {optional Boolean} forCount - If true, do not download anything - use only what we have
     * @return An array of threads. 
     */
    getThreads: function(forCount) {
        
        var oldThreadContainer = this._currentThreadContainer;
        
        if( !this._threads ) {
            
            
            var gotThreads = false;
            if( this._isStatusConstrained()  ) {
                // Try to use the browser.searchConditions to filter the statusThreadsContainer:
                try {
                    threads = this._browser.getFilteredStatusThreads(); // This will always be accurate, even if statusThreadsContainer changes
                    this._threads = [];
                    for( var i = 0; i < threads.length; i++ ) {
                        if( this.searchConditions.matchThread(threads[i]) ) {
                            this._threads.push(threads[i]);
                        }
                    }
                    this._currentThreadContainer = this._gtd.statusThreadsContainer;
                    gotThreads = true;
                } catch(e) {
                    giLogger.warn(e, 'gtdBrowserThreads_ThreadListTab.getThreads', this._gtd.gmail);
                    this._threads = null;
                }
                
            }
            
            if( !gotThreads ) {
                // Either not status constrained, or the searchConditions through an Impossible Match.
                // So, download the messages
                
                
                if( !this._threadContainer && !forCount ) {
                    var search = new glSearch(this._gtd.gmail);
                    search.addCondition(this.searchConditions);
                    search.addCondition(this._browser.searchConditions);
                    // Create a new thread container:
                    this._threadContainer = new gtdThreadContainer(this._gtd, search);
                    this._threadContainer.search(25);
                }
                
                if( this._threadContainer ) {
                    this._threads = [];
                    for( var i = 0; i < this._threadContainer.threads.length; i++ ) {
                        this._threads.push( this._threadContainer.threads[i] );
                    }
                }
                this._currentThreadContainer = this._threadContainer; // Might be null
                
            }
            
        }
        
        
        if( oldThreadContainer!=this._currentThreadContainer ) {
            // Changed listeners
            var listeners = [gtdThreadContainer.NEW_SEARCH, gtdThreadContainer.LOADED, gtdThreadContainer.LOADING_UPDATE, gtdThreadContainer.CHANGED];
            
            if( oldThreadContainer ) {
                // Remove listeners
                for( var i = 0; i < listeners.length; i++ ) oldThreadContainer.removeEventListener(listeners[i], this._threadContainerChanged, this);
            }
            if( this._currentThreadContainer ) {
                // Add listeners:
                for( var i = 0; i < listeners.length; i++ ) this._currentThreadContainer.addEventListener(listeners[i], this._threadContainerChanged, this);
            }
        }
        
        
        
        return this._threads || [];
    },
    
    /**
     * Calculate the count and update it on the el
     * @function {} ?
     * @param {Boolean} force - If true, ensure we download a threadContainer if needed (expensive)
     */
    refreshCount: function(force) {
        var forCount = force ? false : true;
        var threads = this.getThreads(forCount);
        if( this._currentThreadContainer ) {
            if( this._currentThreadContainer.isComplete() ) {
                this._browser.setTabCount(this.id, threads.length);
            } else {
                this._browser.setTabCount(this.id, threads.length + "+");
            }
        } else {
            // Cannot download threads. Show ?
            this._browser.setTabCount(this.id, "?");
        }
    },
    
    /**
     * If threadList loaded, refresh it with our threads
     * @function {} ?
     */
    refreshThreadList: function() {
        if( this.threadList ) {
            var threads = this.getThreads();
            this.threadList.setThreads(threads, this._currentThreadContainer);
        }
    },
    
    /**
     * Notification our thread list changed.
     * Clear out the cache and update the count.
     * @function {} ?
     */
    _threadContainerChanged: function(evt) {
        // Reset cache of threads, so as on next getThreads they are reconstructed:
        this._threads = null;
        
        // Update thread list:
        if( this._browser.getActiveTab()==this.id ) {
            this.refreshThreadList();
        }
        
        // Update count:
        this.refreshCount();
    },
    
    /**
     * Notification that the browser search changed
     * TODO plumb this in
     * @function {} ?
     */
    _browserSearchChanged: function(evt) {
        if( this._threadContainer ) {
            // Reset it - will force it to download afresh
            this._threadContainer = null;
        }
        
        // Reset thread cache - will force either browser or statusThreadContainer to be re-evaluated
        this._threads = null;
        
        // Update thread list:
        if( this._browser.getActiveTab()==this.id ) {
            this.refreshThreadList();
        }
        
        // Update count:
        this.refreshCount();
    },
    
    /**
     * Test to see if the searchConditions are dependent (by AND) upon a Status label.
     * To be dependent, even all OR conditions must use a label:<status>, or one or more AND conditions must use label:<status>
     * The point is to decide if we can confidently use statusThreadsContainer.
     * @function {Boolean} ?
     * @param {optional glSearchConditions} searchConditions - Current searchConditions
     * @return True, if searchConditions require a status label
     */
    _isStatusConstrained: function(searchConditions, statusData) {
        if( this.__isStatusConstrained!==null ) return this.__isStatusConstrained;
        
        // Initiate:
        var rootItem = false;
        if( !searchConditions ) {
            rootItem = true;
            searchConditions = this.searchConditions;
        }
        if( !statusData ) {
            statusData = {labels: this._gtd.labelsData.getLabelsArray('statuses'), hash: {}};
            for( var i = 0; i < statusData.labels.length; i++ ) {
                // Create a "label:s-action" style lookup of valid status labels
                statusData.hash[ glSearch.l(statusData.labels[i]) ] = true;
            }
        }
        
        
        var pass = false;
        if( searchConditions instanceof glSearchOrConditions ) {
            // If OR, all branches must contain Status to be valid.
            if( searchConditions.conditions.length > 0 ) {
                pass = true;
                for(var i = 0; i < searchConditions.conditions.length; i++) {
                    var condition = searchConditions.conditions[i];
                    if(condition instanceof glSearchConditions) {
                        pass = this._isStatusConstrained(condition, statusData);
                    } else {
                        if( statusData.hash[condition] ) {
                            pass = true;
                        }
                    }
                    if( !pass ) break;  // Only one fail accepted
                }
            }
        } else {
            // As an AND, we just need to find a single Status to match
            pass = false;
            for(var i = 0; i < searchConditions.conditions.length; i++) {
                var condition = searchConditions.conditions[i];
                if(condition instanceof glSearchConditions) {
                    pass = this._isStatusConstrained(condition, statusData);
                } else {
                    if( statusData.hash[condition] ) {
                        pass = true;
                    }
                }
                if( pass ) break;   // Only need one to be accepted.
            }
        }
        
        if( rootItem ) {
            // Cache result
            this.__isStatusConstrained = pass;
        }
        return pass;
    },
    
    /**
     * Notification that a view was edited
     * Update affected queries and refilter
     * @function {} ?
     */
    _viewEdited: function(evt) {
        for( i in this.tabs ) {
            if( this.tabs[i].viewId==evt.viewId ) {
                this.tabs[i].filter = this._main.views.getViewSearch(evt.viewId);
            }
        }
    },
    
});