/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Label Types section
 *
 * TODO This is broken. It extends boxSection but it's really a giUIObject.
 *  It should probably reconsider how it is invoked.
 * 
 * @class ? @extends boxSection
 */
var gtdBoxTypes = boxSection.extend({
    // Properties {
    
    /**
     * The gtdMainPagePlugin for the page
     * @property {gtdMainPagePlugin}
     */
    _gtd: null,
    
    
    /**
     * Hash of gtdLabelsTree objects, for each popout type
     * @property {Object}
     */
    _labelsTrees: null,
	
    
    //}

    // Methods {
    /**
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        giLogger.log('Connecting gtdBoxTypes object to view', 'gtdBoxTypes');
        this._gtd = gtd;
        
        // Now get the box plugin: TODO This is extremely ugly. Need to build into plugin framework.
        var box = giPageManager.getPluginInstance(boxMain, this._gtd.gmail);

        this._labelsTrees = {};
        
        this.id = 'gtdtypes';
        this.base(box);
        
        // Create a row for each known type
        this._refresh();
               
        // Listen for popup fires:
        this.addEventListener( boxSection.POPOUT, this._popout, this);
        
        // Listen for label changes
        this._gtd.gmail.environment.addEventListener(glEnvironment.CREATE_LABEL, this._createLabelEvent, this);
        this._gtd.statusThreadsContainer.addEventListener(gtdThreadContainer.LOADED, this._refreshTypeCounts, this);
        this._gtd.statusThreadsContainer.addEventListener(gtdThreadContainer.CHANGED, this._refreshTypeCounts, this);
        this._gtd.labelsData.addEventListener(gtdLabelsData.LABEL_CHANGED, this._labelChanged, this);
        
        this._isExpandable(true);
    },
    
    
    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdBoxTypes from view', 'gtdBoxTypes');
        
        
        this._gtd.statusThreadsContainer.removeEventListener(gtdThreadContainer.LOADED, this._refreshTypeCounts, this);
        this._gtd.statusThreadsContainer.removeEventListener(gtdThreadContainer.CHANGED, this._refreshTypeCounts, this);
        this._gtd.gmail.environment.removeEventListener(glEnvironment.CREATE_LABEL, this._createLabelEvent, this);
        this._gtd.labelsData.removeEventListener(gtdLabelsData.LABEL_CHANGED, this._labelChanged, this);
        
        for( t in this._labelsTrees ) {
            this._labelsTrees[t].disconnect();
            delete this._labelsTrees[t];
        }
        
        
        this.base();
    },
    
    
    
    /**
     * Create a row for each known type
     * @function
     */
    _refresh: function() {
        
        // Clear out any previous entries:
        for( var i = this.el.childNodes.length - 1; i >= 0; i-- ) {
            if( !this._elExpand || this.el.childNodes[i]!=this._elExpand ) {
                this.el.removeChild( this.el.childNodes[i] );
            }
        }
        
        // See which types exist:
        var foundOne = false;
		var typeSettings = this._gtd.prefs.getPref('labels.types');
		for( t in typeSettings ) {
            var elRow = this._main.gmail.doc.createElement("DIV");
            elRow.setAttribute("gtdi-box-row-type", t);
            elRow.innerHTML = giI18N.getString('Labels.Types.'+t, t);
            giUIObject.addClassName(elRow, 'gtdi-box-row-gtdtypes');
            var elPopup = this._main.gmail.doc.createElement("DIV");
            this._addRow(t, elRow, elPopup, true);
            
            foundOne = true;
        }
        
        // Set display status:
        this.el.style.display = foundOne? '' : 'none';
        
        // Refresh type counts
        this._refreshTypeCounts();
        
        // Refresh expanded status:
        if( this._elExpand ) this.expanded(this.expanded());
        
    },
    
    /**
     * Refresh the counts for each label type, shown on the rows
     * @function {} ?
     * @param {optional Event} evt - If present, comes from statusThreadContainer change
     */
    _refreshTypeCounts: function(evt) {
        // Process each label in statusThreadContainer
        // Increase the count for each label type
        // Render the total count for each type
        
        
        
        // Now render:
        for( var i = 0; i < this._rows.length; i++ ) {
            var el = this._rows[i].el.firstChild;
            if( this._gtd.statusThreadsContainer.isTypeActive(this._rows[i].id) ) {
                giUIObject.addClassName(el, "gtdi-box-row-gtdtypes-hasitems");
            } else {
                giUIObject.removeClassName(el, "gtdi-box-row-gtdtypes-hasitems");
            }
        }
    },
    
    /**
     * Notification an environment label changed.
     * Clear out the popout and refresh type counts
     * @function {} ?
     */
    _labelChanged: function(evt) {
        this._refreshTypeCounts();
        
        // Clear out the rows:
        for( r in this._rows ) {
            if( this._rows[r].elPopout ) {
                this._rows[r].elPopout.innerHTML = "";
            }
        }
    },
    
    /**
     * Showing popout, populate it with information
     * @function {}
     */
    _popout: function(evt) {
        var el = evt.elPopout;
        
        var doc = this._main.gmail.canvasDocument;
        var type = evt.elRow.getAttribute('gtdi-box-row-id');
        if( el.innerHTML=="" ) {
            // Need to load the element with labels
            
            
            if( type!="statuses" ) {
                // Create recent container
                var elRecent = doc.createElement("DIV");
                elRecent.className = 'gtdi-box-popout-label-recent';
                elRecent.style.display = "none"
                el.appendChild( elRecent );
            }
            
            
            // Create labels:
            var typeLabelDOM = this._gtd.labelsData.getLabelsEl(type);    // TODO preserve typeLabelDOM for manipulation
            this._labelsTrees[type] = typeLabelDOM;
            if( this._gtd.labelsData.getLabelsArray(type).length==0 ) {
                // No labels
                var info = giI18N.getString('LabelsInfo.desc.'+type);
                if( info ) {
                    var elInfo = doc.createElement("DIV");
                    elInfo.className = 'gtdi-box-popout-nolabels-info';
                    elInfo.innerHTML = info;
                    el.appendChild( elInfo );
                }
                
                var typeSettings = this._gtd.prefs.getPref('labels.types');
                
                var elNotice = doc.createElement("DIV");
                elNotice.className = "gtdi-box-popout-nolabels-createinfo";
                elNotice.innerHTML = giI18N.getString("Box.Labels.NoLabels").replace("%1", typeSettings[type].prefix);
                el.appendChild( elNotice );
                
                if( type!='old' ) {
                    var elButton = doc.createElement("BUTTON");
                    elButton.innerHTML = giI18N.getString('Box.Labels.NoLabels.Create');
                    elButton.className = 'gtdi-box-popup-nolabels-create';
                    elButton.setAttribute('gtdi-newlabel-typeName', giI18N.getString('Labels.Types.' + type, type));
                    elButton.setAttribute('gtdi-newlabel-prefix', typeSettings[type].prefix)
                    el.appendChild( elButton );
                    this._eventsContainer.observe(elButton, 'click', this._clickCreateLabel, false);
                }
                
            } else {
                // Attach labels:
                el.appendChild( typeLabelDOM.el );
                
                // Now add extenders
                var result = doc.evaluate(".//DIV[contains(@class,'gtdi-labeldom-label')]", doc.body, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
                // Extend each label
                for( var i = 0; i < result.snapshotLength; i++ ) {
                    var elLabel = result.snapshotItem(i);
                    if( !giUIObject.hasClassName(elLabel, 'gtdi-labeldom-placeholder') ) {
                        elLabel.title = giI18N.getString('Box.label').replace('%1', elLabel.getAttribute('gtdLabelName'));
                        
                        elAll = doc.createElement("IMG");
                        elAll.className = "gtdi-box-popout-label-showall";
                        elAll.src = "chrome://gtdinbox/skin/gtd/show-threads.png";
                        elAll.title = giI18N.getString('Box.showall').replace('%1', elLabel.getAttribute('gtdLabelName'));
                        elLabel.appendChild( elAll );
                        
                        var elImg = doc.createElement("IMG");
                        elImg.className = "gtdi-box-row-extend";
                        elImg.src = "chrome://gtdinbox/skin/gtd/popout.png";
                        elImg.title = giI18N.getString('Box.filterextend').replace('%1', elLabel.getAttribute('gtdLabelName'));
                        elLabel.appendChild( elImg );
                    }   
                }    
            }
            
            
            // Listen for clicks
            this._eventsContainer.stopObserving( el, 'click', this._clickPopout, false );
            this._eventsContainer.observe( el, 'click', this._clickPopout, false );
        }
        
        
        if( type!="statuses" ) {
            // Get the recent box:
            var elRecent = doc.evaluate(".//*[@class='gtdi-box-popout-label-recent']", el, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
            var recent = this._gtd.recent.getLabels(type);
            if( recent.length==0 ) {
                elRecent.style.display = "none";
            } else {
                if( recent.length>3 ) recent.length = 3; // Cap it at 3 items - more readable.
                recent.sort();  // Ensure it is sorted for better scanability
                elRecent.style.display = "";
                elRecent.innerHTML = "";
                var elRecentTitle = doc.createElement("DIV");
                elRecentTitle.innerHTML = giI18N.getString("Labelling.dropdown.recent");
                elRecentTitle.className = 'gtdi-box-popout-label-recent-title';
                elRecent.appendChild( elRecentTitle );
                for( var i = 0; i < recent.length; i++ ) {
                    var elRecentItem = doc.createElement("DIV");
                    elRecentItem.className = "gtdi-box-popout-label-recent-item";
                    elRecentItem.innerHTML = recent[i][0];
                    elRecentItem.innerHTML += "<img src='chrome://gtdinbox/skin/gtd/show-threads.png' class='gtdi-box-popout-label-showall' title='" + giI18N.getString('Box.showall').replace('%1', recent[i][0]) + "' />";
                    elRecentItem.innerHTML += "<img src='chrome://gtdinbox/skin/gtd/popout.png' class='gtdi-box-row-extend' title='" + giI18N.getString('Box.filterextend').replace('%1', recent[i][0]) + "' />";
                    elRecentItem.title = giI18N.getString('Box.label').replace('%1', recent[i][0]);
                    elRecentItem.setAttribute("gtdLabelName", recent[i][0]);
                    elRecent.appendChild( elRecentItem );
                }
            }
        }
        
        

        // Check each label and update according to count:
        var result = doc.evaluate( ".//div[@gtdLabelName]", evt.elPopout, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
        for( var i = 0; i < result.snapshotLength; i++ ) {
            var elLabel = result.snapshotItem(i);
            var count = this._gtd.statusThreadsContainer.getLabelCount(elLabel.getAttribute('gtdLabelName'));
            if( count ) {
                elLabel.innerHTML = elLabel.innerHTML.replace(/(.+?)(\s\(\d+\))?$/, "$1"+" ("+count+")");
                // Now set the parents bold as they contain children:
                var hierarchy = elLabel.getAttribute('gtdLabelName').split("/");
                var parents = {};
                var path = "";
                for( var j = 0; j < hierarchy.length; j++ ) {
                    if( j>0 ) path += "/";
                    path += hierarchy[j];
                    parents[path] = true;
                }
                var j = i;
                while( j>=0 && elLabel ) {
                    if( giUIObject.hasClassName(elLabel, 'gtdi-box-popout-label-has_statuses') || !parents[elLabel.getAttribute('gtdLabelName')] ) {
                        // We're either within rows that have already been bolded; or no longer in the parent hierarchy, so stop
                        break;
                    }
                    giUIObject.addClassName(elLabel, 'gtdi-box-popout-label-has_statuses');
                    elLabel = elLabel.previousSibling;
                    j = j-1;
                }
            } else {
                // No status items found, strip any trace:
                elLabel.innerHTML = elLabel.innerHTML.replace(/(.+?)(\s\(\d+\))?$/, "$1");
                giUIObject.removeClassName(elLabel, 'gtdi-box-popout-label-has_statuses');
            }
            
        }
        
    },
    
        
    /**
     * Notification the user clicked something on a popup
     * @function {} ?
     */
    _clickPopout: function(evt) {
        try {
            var el = evt.target;
            
            if( giUIObject.hasClassName(el, 'gtdi-labeldom-placeholder') || giUIObject.hasClassName(el.parentNode, 'gtdi-labeldom-placeholder') ) {
                // Ignore place holders
                return;
            }
            
            // Label Popout:
            if( el.className.indexOf('gtdi-labeldom-expander')>-1 ) {
                // Do nothing, it's the expand-toggle
            } else if( el.className.indexOf('gtdi-box-popout-label-showall')>-1 ) {
                // Show all for this label
                var labelName = el.parentNode.getAttribute('gtdLabelName');
                //var search = this._main.gmail.getUIObject('searchBox');
                //search.setSearch( glSearch.l(labelName) );
                //search.submit();
                this._main.gmail.doc.defaultView.top.location.hash = "label/"+encodeURIComponent(labelName);
                this._hidePopout();
            } else if( el.className.indexOf('gtdi-box-row-extend')>-1 ) {
                // Clicked extend. 
                // Load label in popup
                
                while( el.parentNode && el.getAttribute && !el.getAttribute('gtdLabelName') ) el = el.parentNode;
                if( !el.parentNode || !el.getAttribute ) return;
                
                var labelName = el.getAttribute('gtdLabelName');
                if( labelName ) {
                    var type = this._gtd.labelsData.getLabelType(labelName).toUpperCase();
        
                    // Get the popup
                    var popup = this._gtd.getUIObject('browserPopup');
                    popup.load(gtdBrowser.createBrowser(this._gtd, labelName));
                    
                    this._hidePopout();
                }    
            } else if( el.className.indexOf('gtdi-box-popup-nolabels-create')>-1 ) {
                // Notification the user wanted to create a label
                var typeName = el.getAttribute('gtdi-newlabel-typeName');
                var prefix = el.getAttribute('gtdi-newlabel-prefix');
                
                if( this._gtd.gmail.page.getActiveViewType()=="cv" ) {
                    // Use DOM as it may need to be used in conversation view
                    this._gtd.gmail.actions.performAction(glActionManager.CREATE_LABEL, null, glActionManager.USE_DOM);
                    var actionsHeader = this._gtd.gmail.getCurrentViewObject().actionsHeader;
                    var timer = {count: 0, id: null};
                    timer.id = setInterval( function() {
                        var elInput = actionsHeader.getCreateLabelInput();
                        if( elInput || ++count>5) {
                            // found it - clear interval and set prefix
                            clearInterval( timer.id );
                            if( elInput ) elInput.value = prefix;
                        }
                    }, 200);
                } else {
                    var label = prompt(giI18N.getString('Box.Labels.NoLabels.Create.Prompt').replace('%1',typeName), prefix);
                    if( label ) {
                        this._gtd.gmail.actions.performAction(glActionManager.CREATE_LABEL, null, glActionManager.USE_AJAX, label);
                    }
                }
            } else {
                // Default - open in Gmail
                var labelName = el.getAttribute('gtdLabelName');
                //var search = this._main.gmail.getUIObject('searchBox');
                
                var searchStr = glSearch.l(labelName);
                var statuses = this._gtd.labelsData.getLabelsArray('statuses', false);
                var statusData = this._gtd.labelsData.getStatusData(true);
                for( var i = 0; i < statuses.length; i++ ) {
                    if( statusData.reverseLookup[statuses[i]]!="finished" ) {
                        if( i==0 ) searchStr += " (";
                        if( i>0 ) searchStr += " OR ";
                        searchStr += glSearch.l( statuses[i]);
                    }                    
                }
                if( statuses.length > 0 ) searchStr += ")";
                searchStr += " -" + glSearch.l( statusData.finished );
                
                this._main.gmail.doc.defaultView.top.location.hash = "search/"+encodeURIComponent(searchStr);
                
                //search.setSearch( searchStr );
                //search.submit();
                this._hidePopout();
                
            }
        } catch(e) {
            giLogger.warn(e, 'gtdBoxTypes._clickPopup', this._gtd.gmail);
        }
    },
    
    
    /**
     * Notification a label was created
     * @function {}?
     */
    _createLabelEvent: function(evt) {
        this._hidePopout(); // Ensure it is refreshed on the next load
    },
    
    /**
     * Expand or hide the expanded view of the section.
     * This is specific to each implementation of a section; and so should be handled by the subclass.
     * @function {Boolean} ?
     * @param {Boolean} v - If true, show as expanded. If false, hide. If undefined, return current status.
     * @return If no params, return true/false for expanded status.
     */
    expanded: function(v) {
        if( v ) {
            // Show everything
            for( var i = 0; i < this._rows.length; i++ ) {
                this._rows[i].el.style.display = "";
            }
        } else if( typeof v!="undefined" ) {
            // Hide everything
            for( var i = 0; i < this._rows.length; i++ ) {
                this._rows[i].el.style.display = "none";
            }
        }
        
        return this.base(v);
    },
    
    //}
});




/**
 * GTD Shortcuts section
 *
 * @class ? @extends boxSection
 */
var gtdBoxShortcuts = boxSection.extend({
    // Properties {
    
    /**
     * The gtdMainPagePlugin for the page
     * @property {gtdMainPagePlugin}
     */
    _gtd: null,
	
    /**
     * The thread ids for each view. Format: {viewId: String[]}
     * @property {Object}
     */
    _viewThreads: null,
    
    
    
    //}

    // Methods {
    /**
     * Connects the element to the given boxMain object's page. Called
     * automatically by {@link boxMain.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        giLogger.log('Connecting gtdBoxShortcuts object to view', 'gtdBoxShortcuts');
        
        
        this._gtd = gtd;
                
        // Now get the box plugin: TODO This is extremely ugly. Need to build into plugin framework.
        var box = giPageManager.getPluginInstance(boxMain, this._gtd.gmail);
        this.id = 'gtdshortcuts';
        this.base(box);
        
        this._viewThreads = {};
        
        // Control class:
        giUIObject.addClassName(this.el, 'gtdi-gtd-shortcuts');
        
        
        // Create a row for each view (and an overview)
        this.el.innerHTML = "";
        
        // Add Dashboard
        var elRow = this._main.gmail.doc.createElement("A");
        elRow.href = "#";
        elRow.innerHTML = giI18N.getString('Sidebar.Overview') + " <span class='gtdi-gtd-shortcuts-row-count'></span>";
        var data = this._addRow('dashboard', elRow);
        giUIObject.addClassName(data.el, 'gtdi-box-row-link-dashboard');
        
        // Add each view:
        var views = this._gtd.views.getViews();
        for( var i = 0; i < views.length; i++ ) {
            var elRow = this._main.gmail.doc.createElement("DIV");
                
            var elA = this._main.gmail.doc.createElement("A");
            elA.href = "#";
            elA.innerHTML = this._gtd.views.getViewName(views[i]) + " <span class='gtdi-gtd-shortcuts-row-count'></span>";
            elA.setAttribute('gtdi-view-id', views[i]);
            elA.setAttribute("gtdType", gtdBrowser.TYPES.VIEW);
            elA.setAttribute("gtdItem", views[i]);
            elRow.appendChild( elA );
    
            var elExtend = this._main.gmail.doc.createElement("IMG");
            elExtend.src = "chrome://gtdinbox/skin/gtd/popout.png";
            elExtend.className = 'gtdi-box-row-extend';
            elExtend.title = giI18N.getString('Box.filterextend').replace('%1', views[i]);
            elRow.appendChild( elExtend );
    
            this._addRow(views[i], elRow);
        }
        
        // Load initial data:
        this._refreshCount();
        
        
        // Listen for clicks
        this._eventsContainer.observe( this.el, 'click', this._click, false );
        
        
        // Listen for thread cache changes:
        this._gtd.statusThreadsContainer.addEventListener(gtdThreadContainer.LOADED, this._refreshCount, this);
        //this._gtd.statusThreadsContainer.addEventListener(gtdThreadContainer.LOADING_UPDATE, this._refreshCount, this);
        this._gtd.statusThreadsContainer.addEventListener(gtdThreadContainer.CHANGED, this._threadChanged, this);
        
        this._gtd.views.addEventListener(gtdViews.VIEW_EDITED, this._viewEdited, this);
        
        this._isExpandable(true);
    },
    
    
    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdBoxShortcuts from view', 'gtdBoxShortcuts');
        
        this._gtd.statusThreadsContainer.removeEventListener(gtdThreadContainer.LOADED, this._refreshCount, this);
        this._gtd.statusThreadsContainer.removeEventListener(gtdThreadContainer.LOADING_UPDATE, this._refreshCount, this);
        this._gtd.statusThreadsContainer.removeEventListener(gtdThreadContainer.CHANGED, this._threadChanged, this);
        this._gtd.views.removeEventListener(gtdViews.VIEW_EDITED, this._viewEdited, this);
        this._viewThreads = null;
        
        this.base();
    },

    
    /**
     * Notification a thread was changed.
     * Find all instances of it; remove them, and attempt to readd the thread to each view
     * Also update the overview
     * @function {} ?
     */
    _threadChanged: function(evt) {
        
        // Remove:
        for( var i = 0; i < evt.updatedThreads.length; i++ ) {
            var thread = evt.updatedThreads[i];
            
            for( viewId in this._viewThreads ) {
                var pos = this._viewThreads[viewId].indexOf(thread.threadId);
                if( pos>-1 ) {
                    // Remove it:
                    this._viewThreads[viewId].splice(pos,1);
                }
            }
        }
        
        // Add (has to be seperate to remove, or we could remove threads we add):
        for( var i = 0; i < evt.updatedThreads.length; i++ ) {
            var thread = evt.updatedThreads[i];
            for( viewId in this._viewThreads ) {
                // Attempt to add thread:
                if( evt.removedThreads.indexOf(thread)==-1 ) {
                    if( this._gtd.views.getViewSearch(viewId).matchThread(thread) ) {
                        this._viewThreads[viewId].push(thread.threadId);
                    }
                }
            }
        }
        
        // Update count for changed views:
        this._refreshCount();

    },
    
    /**
     * Notification that a view was edited.
     * Update the results for just this view
     * @function {} ?
     */
    _viewEdited: function(evt) {
        // Update search:
        for( var i = 0; i < this._rows.length; i++ ) {
            if( this._rows[i].id==evt.viewId ) {
                this._rows[i].getElementsByTagName("A")[0].setAttribute("gtdItem", this._gtd.views.getViewSearch(evt.viewId).toString());
            }
        }
        
        // Reload threads:
        this._refreshView(evt.viewId);
        
        // Update count:
        this._updateRowCount(evt.viewId, this._viewThreads[evt.viewId].length );
    },
    
    /**
     * Update the counts on each row
     * @function {} ?
     */
    _refreshCount: function() {
        
        for( var i = 0; i < this._rows.length; i++ ) {
            if( this._rows[i].id=='dashboard' ) {
                // Dashboard :
                this._updateRowCount('dashboard', this._gtd.statusThreadsContainer.getTotalCount());
            } else {
                // View:
                var viewId = this._rows[i].id;
                
                // Update view threads:
                this._refreshView(viewId);
                
                // Render count:
                this._updateRowCount(viewId, this._viewThreads[viewId].length );
                
            }
        }
    },
    
    /**
     * Reload the thread ids for the given view
     * @function {} ?
     * @param {String} viewId
     */
    _refreshView: function(viewId) {
        if( !this._viewThreads[viewId] ) this._viewThreads[viewId] = [];
        this._viewThreads[viewId].length = 0;
        var searchConditions = this._gtd.views.getViewSearch(viewId);
        var statusThreads = this._gtd.statusThreadsContainer.threads;
        for( var i = 0 ; i < statusThreads.length; i++ ) {
            if( searchConditions.matchThread(statusThreads[i]) ) {
                this._viewThreads[viewId].push(statusThreads[i].threadId);
            }
        }
    },
    
    /**
     * Update the count for the given element
     * @function {} ?
     * @param {String} viewId
     * @param {int} count
     */
    _updateRowCount: function(rowId, count) {
        for( var i = 0; i < this._rows.length; i++ ) {
            if( this._rows[i].id==rowId ) {
                var el = this._rows[i].el;
                break;
            }
        }
        
        if( el ) {
            var elCount = el.ownerDocument.evaluate(".//*[@class='gtdi-gtd-shortcuts-row-count']", el, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
            if( count==0 ) {
                elCount.innerHTML = "";
                el.style.fontWeight = 'normal';
            } else {
                elCount.innerHTML = "("+count+")";
                el.style.fontWeight = 'bold';
            }
        }
    },
    
    /**
     * Detect item clicked and respond appropriately
     * @function {}
     */
    _click: function(evt) {
        try {
            var el = evt.target;
            
            var elRow = null;
            while( el.parentNode && !elRow ) {
                if( el==this.el ) break;
                if( el.getAttribute('gtdi-box-row-id') ) elRow = el;
                el = el.parentNode;
            }
            
            if( elRow ){
                giUtil.stopEvent(evt);
                
                
                // Find out which row it was:
                for( var i = 0; i < this._rows.length; i++ ) {
                    if( this._rows[i].el==elRow ){
                        // Match
                        if( this._rows[i].id=='dashboard' ){
                            // Load the Dashboard
                            
                            var popup = this._gtd.getUIObject('browserPopup');
                            popup.load(gtdBrowser.createBrowser(this._gtd));
                            
                        } else {
                            // Load a view:
                            
                            var viewId = this._rows[i].id;
                            
                            if( evt.target.className.indexOf('gtdi-box-row-extend')>-1 ) {
                                // Load the Label Popup for a View
                                
                                var popup = this._gtd.getUIObject('browserPopup');
                                popup.load(gtdBrowser.createBrowser(this._gtd, viewId, gtdBrowser.TYPES.VIEW)); // TODO Create a 'View' type?
                                
                            } else {
                                // Open in Thread List

                                this._main.gmail.doc.defaultView.top.location.hash = "search/"+encodeURIComponent(this._gtd.views.getViewSearch(viewId).toString());
                                
                                
                            }
                        }
                    }
                }
            }
        } catch(e) {
            giLogger.warn(e, 'gtdBox._click');
        }
        
    },
    
    
    /**
     * Expand or hide the expanded view of the section.
     * This is specific to each implementation of a section; and so should be handled by the subclass.
     * @function {Boolean} ?
     * @param {Boolean} v - If true, show as expanded. If false, hide. If undefined, return current status.
     * @return If no params, return true/false for expanded status.
     */
    expanded: function(v) {
        if( v ) {
            // Show everything
            for( var i = 0; i < this._rows.length; i++ ) {
                this._rows[i].el.style.display = "";
            }
        } else if( typeof v!="undefined" ) {
            // Hide everything
            for( var i = 0; i < this._rows.length; i++ ) {
                this._rows[i].el.style.display = "none";
            }
        }
        
        return this.base(v);
    },
    
    //}
});




/**
 * GTD Setup section
 *
 * @class ? @extends boxSection
 */
var gtdBoxSetup = boxSection.extend({
    // Properties {
    
    /**
     * The gtdMainPagePlugin for the page
     * @property {gtdMainPagePlugin}
     */
    _gtd: null,
	
    //}

    // Methods {
    /**
     * Connects the element to the given boxMain object's page. Called
     * automatically by {@link boxMain.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        giLogger.log('Connecting gtdBoxSetup object to view', 'gtdBoxSetup');
        
        
        this._gtd = gtd;
                
        // Now get the box plugin: TODO This is extremely ugly. Need to build into plugin framework.
        var box = giPageManager.getPluginInstance(boxMain, this._gtd.gmail);
        this.id = 'gtdsetup';
        this.base(box);
        
        // Render message:
        var doc = this._main.gmail.canvasDocument;
        var elMsg = doc.createElement("DIV");
        elMsg.id = "gtdi-gtd-setup-msg";
        elMsg.innerHTML = giI18N.getString('Box.setup.msg');
        this.el.appendChild( elMsg );
        
        var elInstall = doc.createElement("BUTTON");
        elInstall.id = "gtdi-gtd-setup-install";
        elInstall.innerHTML = giI18N.getString('Box.setup.install');
        this.el.appendChild( elInstall );
        
        // Listen for clicks
        this._eventsContainer.observe( this.el, 'click', this._click, false );
        
        // Listen for label changes
        this._gtd.labelsData.addEventListener(gtdLabelsData.LABEL_CHANGED, this._labelsChanged, this);
        
    },
    
    
    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdBoxShortcuts from view', 'gtdBoxShortcuts');
        this.base();
    },
    
    
    
    
    /**
     * If all labels present, disconnect it
     * @function
     */
    _labelsChanged: function(evt) {
        
        var sd = this._gtd.labelsData.getStatusData();
        var statuses = this._gtd.labelsData.getLabelsHash('statuses');
        if( statuses[sd.action] && statuses[sd['waiting_on']] && statuses[sd['waiting_on']] && statuses[sd['some_day']] && statuses[sd.finished] ) {
            // All present
            this.disconnect();
        }
    },
    
    
    /**
     * Detect item clicked and respond appropriately
     * @function {}
     */
    _click: function(evt) {
        try {
            var el = evt.target;
            
            if( el.id=='gtdi-gtd-setup-install' ) {
                // Show the install screen
                try {
                    this._gtd.getUIObject('setup');
                } catch(e) {
                    giLogger.warn(e, 'gtdBoxSetup._click');
                    // Open FAQ
                    gBrowser.selectedTab = gBrowser.addTab("http://www.gtdinbox.com/faq.htm#gtdsetup");
                }
            }
            
        } catch(e) {
            giLogger.warn(e, 'gtdBoxSetup._click');
        }
        
    },
    
    //}
});

/**
 * Shown when a conversation thread appears out of sync with the labels
 *
 * @class ? @extends boxSection
 */
var gtdBoxCvWarning = boxSection.extend({
    // Properties {
    
    /**
     * The gtdMainPagePlugin for the page
     * @property {gtdMainPagePlugin}
     */
    _gtd: null,
	
    //}

    // Methods {
    /**
     * Connects the element to the given boxMain object's page. Called
     * automatically by {@link boxMain.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        giLogger.log('Connecting gtdBoxCvWarning object to view', 'gtdBoxCvWarning');
        
        
        this._gtd = gtd;
                
        // Now get the box plugin: TODO This is extremely ugly. Need to build into plugin framework.
        var box = giPageManager.getPluginInstance(boxMain, this._gtd.gmail);
        this.id = 'gtdcvwarning';
        this.base(box);
        
        // Render message:
        var doc = this._main.gmail.canvasDocument;
        var elMsg = doc.createElement("DIV");
        elMsg.id = "gtdi-gtd-cv-warning";
        elMsg.innerHTML = giI18N.getString('Box.cvwarning.msg');
        this.el.appendChild( elMsg );
        
        var elFAQ = doc.createElement("A");
        elFAQ.id = "gtdi-gtd-cv-warning-faq";
        elFAQ.href = "http://www.gtdinbox.com/faq.htm#cvwarning";
        elFAQ.innerHTML = giI18N.getString('Box.cvwarning.faq');
        this.el.appendChild( elFAQ );
        
        var elHide = doc.createElement("SPAN");
        elHide.id = "gtdi-gtd-cv-warning-hide";
        elHide.innerHTML = giI18N.getString('Box.cvwarning.hide');
        this.el.appendChild( elHide );
        
        // Listen for clicks
        this._eventsContainer.observe( elHide, 'click', this._click, false );
        
    },
    
    
    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdBoxCvWarning from view', 'gtdBoxCvWarning');
        this.base();
    },
    
    
    
    /**
     * Detect item clicked and respond appropriately
     * @function {}
     */
    _click: function(evt) {
        try {
            this.disconnect();
        } catch(e) {
            giLogger.warn(e, 'gtdBoxCvWarning._click');
        }
    },
    
    //}
});

/**
 * Shown when malformed settings are detected.
 *
 * @class ? @extends boxSection
 */
var gtdBoxPrefsWarning = boxSection.extend({
    // Properties {
    
    /**
     * The gtdMainPagePlugin for the page
     * @property {gtdMainPagePlugin}
     */
    _gtd: null,
	
    //}

    // Methods {
    /**
     * Connects the element to the given boxMain object's page. Called
     * automatically by {@link boxMain.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        giLogger.log('Connecting gtdBoxCvWarning object to view', 'gtdBoxCvWarning');
        
        
        this._gtd = gtd;
                
        // Now get the box plugin: TODO This is extremely ugly. Need to build into plugin framework.
        var box = giPageManager.getPluginInstance(boxMain, this._gtd.gmail);
        this.id = 'gtdcvwarning';
        this.base(box);
        
        // Render message:
        var doc = this._main.gmail.canvasDocument;
        var elMsg = doc.createElement("DIV");
        elMsg.id = "gtdi-gtd-prefs-warning";
        elMsg.innerHTML = giI18N.getString('Box.prefswarning.msg');
        this.el.appendChild( elMsg );
        
        var elFAQ = doc.createElement("A");
        elFAQ.id = "gtdi-gtd-prefs-warning-faq";
        elFAQ.href = "http://www.gtdinbox.com/faq.htm#prefswarning";
        elFAQ.innerHTML = giI18N.getString('Box.prefswarning.faq');
        this.el.appendChild( elFAQ );
                
    },
    
    
    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdBoxCvWarning from view', 'gtdBoxCvWarning');
        this.base();
    },
    
    
    
    /**
     * Detect item clicked and respond appropriately
     * @function {}
     */
    _click: function(evt) {
        try {
            this.disconnect();
        } catch(e) {
            giLogger.warn(e, 'gtdBoxCvWarning._click');
        }
    },
    
    //}
});

