/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Add labels and pre-labelling to the compose screen, when connected
 *
 * @class ? @extends gtdUIObject
 */
var gtdComposeLabels = giUIObject.extend({
    // Properties {
        	
	
	/** @property {Element} ? The labels container */
	el: null,
	
	/**
	* The {@link gtdLabellingBox} object
	* @property {gtdLabellingBox} ?
	*/
	conversationLabels: null,
	
	/**
	* The array of labels currently selected
	* @property {String[]}
	*/
	_labels: null, 
	
    /** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting gtdComposeLabels object to view', 'gtdComposeLabels');

        this.base(main);
		this._eventsContainer = new giEventsContainer(this);

		this._labels = [];
            
        var doc = main.gmail.canvasDocument;
        
        
        this.conversationLabels = main.getUIObject("labellingBox");
        giUIObject.addClassName(this.conversationLabels.el, 'gtdi-compose-labels');
        this.conversationLabels.addEventListener(gtdLabellingBox.SELECT_LABEL, this._selectLabel, this);
        this.conversationLabels.addEventListener(gtdLabellingBox.UNSELECT_LABEL, this._unSelectLabel, this);
        this.conversationLabels.load(null, null, true);
        
        
        this.el = doc.createElement("TR");
        var elTd = doc.createElement("TD");
        elTd.setAttribute("colspan", "3");
        elTd.appendChild( this.conversationLabels.el );	
        this.el.appendChild( elTd );
        
        var composeView = main.gmail.getCurrentViewObject();
        composeView.composeTable.getElementsByTagName("TBODY")[0].appendChild( this.el );
        
        
		// Attach listeners to Send button:
		this._eventsContainer.observe( composeView.topSendButton, "click", this._clickSend, true );
		this._eventsContainer.observe( composeView.bottomSendButton, "click", this._clickSend, true );
    },
    
	

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdComposeLabels from view', 'gtdComposeLabels');

        
		this.conversationLabels.removeEventListener(gtdLabellingBox.SELECT_LABEL, this._selectLabel, this);
		this.conversationLabels.removeEventListener(gtdLabellingBox.UNSELECT_LABEL, this._unSelectLabel, this);
		this.conversationLabels.disconnect();
		
        if( this.el ) this.el.parentNode.removeChild( this.el );
        this.el = null;

		this._eventsContainer.reset();
        
        this.base();
    },
    

	/**
	* Respond to the Send button click - if we have any labels, pass them to the pre-labeller
	* @function ?
	*/
	_clickSend: function(event){
		try{
			if( this._labels.length > 0 ) this._main.preLabeler.labelNextSent(this._labels);
		}catch(e){
			giLogger.warn(e, 'gtdComposeLabels._clickSend');
		}
	},

	/**
	* Add label to our array and refresh the labelling box
	* @function ?
	* @param {Object} event - {label:labelname}
	*/
	_selectLabel: function(event){
		for( var i = 0; i < this._labels.length; i++ ) if( this._labels[i]==event.label ) return;
		this._labels.push(event.label);
		this.conversationLabels.uiSelectLabels(this._labels);
	},
	
	/**
	* Remove label from our array and refresh the labelling box
	* @function ?
	* @param {Object} event - {label:labelname}
	*/
	_unSelectLabel: function(event){
        for( var i = 0; i < this._labels.length; i++ ) {
            if( this._labels[i]==event.label ){
                this._labels.splice(i,1);
                break;
            }
        }
        this.conversationLabels.uiSelectLabels(this._labels);
	},

    
    //}
});