/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Add "Compose Personal" to the nav bar
 *
 * @class ? @extends giUIObject
 */
var gtdComposePersonal = giUIObject.extend({
    // Properties {
        	
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
	
	/** @property {Element} ? The Compose Personal shortcut */
	el: null,
	
    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting gtdComposePersonal object to view', 'gtdComposePersonal');

        this.base(main);
        this._eventsContainer = new giEventsContainer(this);
    
        var doc = main.gmail.canvasDocument;
            
        var navBar = main.gmail.getUIObject("navBar");
        this.el = navBar.composeLink.cloneNode(true);
        this.el.innerHTML = giI18N.getString("Compose.Personal");
        navBar.composeLink.parentNode.appendChild( doc.createElement("BR") );
        navBar.composeLink.parentNode.appendChild( this.el );
        
        this._eventsContainer.observe( navBar.composeLink, "click", this._clickMail, false );
        this._eventsContainer.observe( this.el, "click", this._click, false );
        
        
        giLogger.log('Connected gtdComposePersonal object to view', 'gtdComposePersonal');
    },
    
    
	/**
	* Called when user clicks on compose mail
	* @function ?
	*/
	_clickMail: function(event) {

	},
    
	/**
	* Called when user clicks on compose personal
	* @function ?
	*/
	_click: function(event) {	// TODO - better to set up some sort of function queue for a thread switch to process after loaded? in gtdMainPagePlugin
		try {
			var navBar = this._main.gmail.getUIObject("navBar");
			giUIObject.simulateMouseEvent(navBar.composeLink, "click" );
		
			var f = giUtil.bind( this, function() {
				if( this._main.gmail.page.getActiveViewType()=="co" ) {
					this._main.getCurrentViewObject().composeTabs.select('compose-tab-personal');
				} else {
					setTimeout( giUtil.bind( this, function() {
						if( this._main.gmail.page.getActiveViewType()=="co" ) this._main.getCurrentViewObject().composeTabs.select('compose-tab-personal');
					}), 2000);
				}
			});
			setTimeout(f, 500);
		} catch(e) {
			giLogger.warn(e, 'gtdComposePersonal._click', this._main.gmail);
		}
	},
	
	

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdComposePersonal from view', 'gtdComposePersonal');

        this._eventsContainer.reset();
        
        if( this.el && this.el.parentNode ) this.el.parentNode.removeChild( this.el );
        this.el = null;
        this.base();
    },
    
    
    /**
     * Whether the UI object can be cached
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return true;
    },
    
    //}
});