/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Add Tabs to Compose view when connected
 *
 * @class ? @extends giUIObject
 */
var gtdComposeTabs = giUIObject.extend({
    // Properties {
    
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
    
	/** @property {private Array} ? An array of els hidden after a tab click */
	_elsHidden: null,
	
	/** @property {Element} ? The tab container */
	el: null,
	
	/** @property {Element} ? The tab container for tab items */
	elTabs: null,
    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting gtdComposeTabs object to view', 'gtdComposeTabs');

        this.base(main);

        this._elsHidden = [];
        this._eventsContainer = new giEventsContainer(this);
    
        var doc = main.gmail.canvasDocument;
        
        this.elTabs = doc.createElement("DIV");
		this.elTabs.className = "compose-tabs";
        this.elTabs.innerHTML = "<div id='compose-tab-mail' class='compose-tab gtdi-active'>" + giI18N.getString("Compose.Mail") + "</div><div id='compose-tab-personal' class='compose-tab'>" + giI18N.getString("Compose.Personal") + "</div>";
	
        this.el = doc.createElement("TR");
        var elTd = doc.createElement("TD");
		elTd.setAttribute('colspan', '3');
        elTd.appendChild( this.elTabs );
        this.el.appendChild( elTd );
        
        var composeView = main.gmail.getCurrentViewObject();
        var elTBody = composeView.composeTable.getElementsByTagName("TBODY")[0];
        elTBody.insertBefore( this.el, elTBody.firstChild );
        
        this._eventsContainer.observe( this.el, "mousedown", this._click, false );

        //}
    },
    
	/**
	* Called when user clicks on tabs
	* @function ?
	*/
	_click: function(event) {
		try {
			// Get the tab item clicked
			var elTab = event.target;
			
			this.select( elTab.id );
		} catch(e) {
			giLogger.warn(e, 'gtdComposeTabs._click', this._main.gmail);
		}
	},
	
	/**
	* Select the tab with the specified id
	* @function ?
	* @param {String} id - The ID of the tab-item element to select
	*/
	select: function(id) {
		
		var doc = this._main.gmail.canvasDocument;
		var elTab = doc.getElementById(id);
		if( !elTab ) return;

		// Remove anything selected
		for( var i = 0; i < this.elTabs.childNodes.length; i++ ) {
			if( this.elTabs.childNodes[i].id==id && giUIObject.hasClassName(this.elTabs.childNodes[i], 'gtdi-active') ) return;	// Currently selected right one; do nothing
			giUIObject.removeClassName( this.elTabs.childNodes[i], "gtdi-active" );
		}
		
		// Restore view to normal (anything hidden)
		for( var i = 0; i < this._elsHidden.length; i++ ) this._elsHidden[i].style.display = "";
		this._elsHidden.length = 0;
		
		
		// Now update the compose table based on what was clicked
		var composeView = this._main.gmail.getCurrentViewObject();
		
		
		switch( elTab.id ) {
			case "compose-tab-mail":
				composeView.toField.value = this._lastToFieldValue || "";
				break;
			case "compose-tab-personal":	// Set and hide To/From field
				var elTable = composeView.composeTable;
				var elRows = elTable.getElementsByTagName("TR");
				this._lastToFieldValue = composeView.toField.value;
				composeView.toField.value = this._main.gmail.environment.getAccount().replace("@", "+action@");
				var elSubject = null;
				for( var i = 1; i <= 6; i++ ) {
					var elTrInput = elRows[i].getElementsByTagName("INPUT")[0];
					if( elTrInput && elTrInput.getAttribute("name")=="subject" ) {
						elSubject = elTrInput;
						break; // Stop before Subject box. Can be located in alternate places depending on number whether user has multiple accounts.
					}
					if( elRows[i].style.display!="none" ) {
						this._elsHidden.push( elRows[i] );
						elRows[i].style.display = "none";
					}
				}
				if( elSubject ) {
					giUIObject.simulateMouseEvent(elSubject, 'click', -1, -1);
					elSubject.focus();
					setTimeout(function() {
						giUIObject.simulateMouseEvent(elSubject, 'click', -1, -1);
						elSubject.focus();
					},400);
				}
				break;
		}
		
		// Now mark current choice selected
		giUIObject.addClassName( elTab, "gtdi-active");
	},
	

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdComposeTabs from view', 'gtdComposeTabs');

        
        if( this.el && this.el.parentNode ) this.el.parentNode.removeChild( this.el );
        this.el = null;
        this.elTabs = null;
        this._eventsContainer.reset();
        
        // Restore view to normal (anything hidden)
        for( var i = 0; i < this._elsHidden.length; i++ ) this._elsHidden[i].style.display = "";
        this._elsHidden.length = 0;
        this.base();
    },
    
    
    
    //}
});