/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * The conversation view abstraction layer.
 *
 * @class ? @extends glUIView
 */
var gtdConversationView = giUIObject.extend({
    // Properties {
    

    
    /**
    * The {@link gtdLabellingBox} object
    * @property {gtdLabellingBox} ?
    */
    _conversationLabels: null,
    
    /**
    * The {@link gtdConversationAlerts} object
    * @property {gtdConversationAlerts} ?
    */
    _conversationAlerts: null,
	
    //}

    // Methods {
    /**
     * Connects the element to the given glGmail object's page. Called automatically
     * by {@link glGmail.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting gtdConversationView object to view', 'gtdConversationView');
        this.base(main);
                            
        var doc = main.gmail.canvasDocument;
        var viewRoot = main.gmail.page.getViewRoot();
        
        
        // Add labels:
    

        var conversationView = main.gmail.getCurrentViewObject();
        

        var thread = conversationView.getThread();
        
        if( thread ) {
            
            //if( !this._main.gmail.offline.isUnstable() ) {
            //    this._attachLabellingBox(thread);
            //}
            this._attachLabellingBox(thread);
            
            // Add listener for alert boxes
            if( main.prefs.getPref('components.conversation.reply_suggest.enabled') ) {
                this._conversationAlerts = main.getUIObject("conversationAlerts");
            }
            
            
        } else {
            giLogger.warn('could not load thread in conversation (glConversation threadid: ' + conversationView.threadId  + ')', 'gtdConversationView.connect');
        }
        
        this._main.gmail.actions.addEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);
        
        //this._main.gmail.offline.addEventListener(glOffline.GO_ONLINE, this._offlineStateChange, this);
        //this._main.gmail.offline.addEventListener(glOffline.GO_UNSTABLE, this._offlineStateChange, this);
    },


    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdConversationView from view', 'gtdConversationView');

        this._main.gmail.actions.removeEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);
        //this._main.gmail.offline.removeEventListener(glOffline.GO_ONLINE, this._offlineStateChange, this);
        //this._main.gmail.offline.removeEventListener(glOffline.GO_UNSTABLE, this._offlineStateChange, this);
        
        if( this._conversationLabels ) {
            this._conversationLabels.removeEventListener(gtdLabellingBox.CLICK_ARCHIVE, this._requestArchive, this);
            this._conversationLabels.disconnect();
            this._conversationLabels = null;
        }
        
        if( this._conversationAlerts ) {
            this._conversationAlerts.disconnect();
            this._conversationAlerts = null;
        }

        this.base();
    },

    /**
     * Attach the conversation labels to the header; if enabled.
     * @function {} ?
     * @param {optional glThread} thread - Passed to spare us having to call it
     */
    _attachLabellingBox: function(thread) {
        if( this._main.prefs.getPref('components.conversation.labelling_box.enabled') ) {
            
            var conversationView = this._main.gmail.getCurrentViewObject();
            
            if( !this._conversationLabels ) {
                // Create it
                this._conversationLabels = this._main.getUIObject("labellingBox");
                this._conversationLabels.setServerActionType(glActionManager.USE_DOM);
                giUIObject.addClassName(this._conversationLabels.el, 'conversation-labels');
                                
                // Add listener for Archive and Move Next
                this._conversationLabels.addEventListener(gtdLabellingBox.CLICK_ARCHIVE, this._requestArchive, this);
                
                // Load thread
                if( !thread ) thread = conversationView.getThread();
                if( thread.isLoading() ) {
                    thread.addEventListener(glThread.LOADED, this._threadLoaded, this)
                } else {
                    this._threadLoaded();
                }
            }
            
            // Add to UI:
            conversationView.getTitle().parentNode.appendChild( this._conversationLabels.el );
        }
    },

    /**
    * Notification that a thread was fully loaded (inc messages)
    * Load thread in labelling box
    * @function {} ?
    */
    _threadLoaded: function(evt) {
        var conversationView = this._main.gmail.getCurrentViewObject();
        var thread = conversationView.getThread();
        thread.removeEventListener(glThread.LOADED, this._threadLoaded);
        
        if( this._conversationLabels ) { // Enabled and active
            this._conversationLabels.load(thread, true);
        }
        
        // Now do error checking
        // Is this the first load of this thread?
        if( gtdConversationView.THREADS_SEEN_CACHE.indexOf(thread.threadId)==-1 ) {
            // First load
            
            
            // Record having seen it:
            gtdConversationView.THREADS_SEEN_CACHE.push(thread.threadId);
            
            // Get the current labels and compare against thread.labels:
            var results = conversationView.getHeaderLabels();
            var badSync = false;
            for( var i = 0; i < results.snapshotLength; i++ ) {
                var labelname = results.snapshotItem(i).getAttribute('name');
                if( thread.labels.indexOf(labelname)==-1 ) {
                    badSync = true;
                    break;
                }
            }
            if( badSync ) { 
                // Show error box
                
                var box = giPageManager.getPluginInstance(boxMain, this._main.gmail);
                if( box ) {
                    var boxCvWarning = this._main.getUIObject('boxCvWarning');
                    box.getUIObject('box').addSection(boxCvWarning, true);
                }
            }
        } else {
            // Later load - check sanctity of labels
            var removedLabels = [];
            //this._cleanseHeaderLabels(elH1, thread, removedLabels);
            if( removedLabels.length>0 ) {
                // This is a notable/undesriable event, log it:
                giLogger.log('Automatically removed labels that are out of sync: \n' + removedLabels.join(", ") + "\nThread labels: " + thread.labels.join(", "), "gtdConversationView._threadLoaded");
            }
        }
    },

    /**
     * Strip either the ^i label (to archive it) or the status labels (excl S/Finished) to finish it
     * @function {} ?
     * @param {Boolean} archive - If true, archive it (remove ^i)
     * @param {Boolean} finished - If true, remove status labels (Excl S/Finished)
     * @param {optional glConversationView} conversationView - Cached copy of current ui object
     * @param {optional Object} statusData - Cached copy of status data
     */
    /*
    _stripStatusAndArchiveWithHeaderLabels: function(archive, finished, conversationView, statusData) {
        if( !conversationView ) conversationView = this._main.gmail.getCurrentViewObject();
        if( finished && !statusData ) statusData = this._main.labelsData.getStatusData();
        var results = conversationView.getHeaderLabels();
        for( var i = 0; i < results.snapshotLength; i++ ) {
            var elLabel = results.snapshotItem(i);
            var name = elLabel.getAttribute('name');
            if( (archive && name=="^i") || (finished && gtdLabelsData.testPrefix(name, statusData.prefix) && name!=statusData.finished ) ) {
                // Label UI needs removing
                while( elLabel.parentNode && elLabel.nodeName!='TR' ) elLabel = elLabel.parentNode;
                if( elLabel.nodeName=='TR' ) {
                    var elX = this._main.gmail.canvasDocument.evaluate(".//td/span[@class='hO']", elLabel, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                    giUIObject.simulateMouseEvent(elX, 'click');
                    // recurse:
                    this._stripStatusAndArchiveWithHeaderLabels(archive, finished, conversationView, statusData);
                }
            }
        }
    },
    */

    /**
     * Strip the labels using the conversations' header labels (which is faster than using the dropdown menu).
     * @function {} ?
     * @param {String[]} remove - An array of labels
     */
    stripLabels: function(remove, conversationView) {
        if( !conversationView ) conversationView = this._main.gmail.getCurrentViewObject();
        var results = conversationView.getHeaderLabels();
        for( var i = 0; i < results.snapshotLength; i++ ) {
            var elLabel = results.snapshotItem(i);
            var name = elLabel.getAttribute('name');
            if( remove.indexOf(name)>-1 ) {
                // Label UI needs removing
                while( elLabel.parentNode && elLabel.nodeName!='TR' ) elLabel = elLabel.parentNode;
                if( elLabel.nodeName=='TR' ) {
                    var elX = this._main.gmail.canvasDocument.evaluate(".//td/span[@class='hO']", elLabel, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                    giUIObject.simulateMouseEvent(elX, 'click');
                    // recurse:
                    this.stripLabels(remove, conversationView);
                }
                
            }
        }
    },

    /**
     * @function {} ?
     * @param {glThread} thread - The thread with accurate labels
     * @param {String[]} removedLabels - Array of labels removed
     */
    /*
    _removeHeaderLabels: function(elH1, thread, removedLabels) {
        if( !removedLabels ) removedLabels = [];
        var results = this._getHeaderLabels(elH1);
        for( var i = 0; i < results.snapshotLength; i++ ) {
            var elLabel = results.snapshotItem(i);
            var name = elLabel.getAttribute('name');
            if( !thread.hasLabel(name) && removedLabels.indexOf(name)==-1 ) {
                // Label UI needs removing
                removedLabels.push(name);
                while( elLabel.parentNode && elLabel.nodeName!='TR' ) elLabel = elLabel.parentNode;
                if( elLabel.nodeName=='TR' ) {
                    var elX = this._main.gmail.canvasDocument.evaluate(".//td/span[@class='hO']", elLabel, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                    giUIObject.simulateMouseEvent(elX, 'click');
                    // recurse:
                    this._cleanseHeaderLabels(elH1, thread, removedLabels);
                }
            }
        }
    },
    */


    /**
    * Notification that a thread was archived.
    * If code-requested via ajax, then move to next item in conversation
    * @function {} ?
    */
    _requestArchive: function(event) {
        
        giLogger.log('request archive', 'gtdConversationView._threadArchived');
        //this._stripStatusAndArchiveWithHeaderLabels(true, false);
        
        
        var conversationView = this._main.gmail.getCurrentViewObject();
        var thread = conversationView.getThread();
        if( thread.hasId(event.threadId) ) {
            
            this.stripLabels( ["^i"] );
            
            if( event.moveNext ) {
                var elNext = conversationView.getMoveNextElement();
                if( elNext ) giUIObject.simulateMouseEvent(elNext, 'click');
            }
        }
    },
    
    /**
     * Notification from action manager that a thread changed
     * @function {} ?
     */
    _threadChanged: function(event) {
        
        
        giLogger.log('thread changed', 'gtdConversationView._threadChanged');
        if( event.action===glActionManager.ADD_LABEL ) {
            var statusData = this._main.labelsData.getStatusData();
            if( event.label==statusData.finished  ) {
                // Finished label, is it our thread?
                var conversationView = this._main.gmail.getCurrentViewObject();
                if( conversationView.getThread().hasId(event.thread) ) {
                    giLogger.log('thread finished', 'gtdConversationView._threadChanged');
                    
                    // Archive it, and move next
                    
                    this.stripLabels(["^i"]);
                    var elNext = conversationView.getMoveNextElement();
                    if( elNext ) giUIObject.simulateMouseEvent(elNext, 'click');
                    
                }
            }
        }
        
    },
    

    /**
     * Notification that the Offline state changed
     * @function {} ?
     */
    /*
    _offlineStateChange: function(evt) {
        
        
        if( evt.name==glOffline.GO_ONLINE ) {
            // Attach (and create if required) Labelling box
            this._attachLabellingBox();
        } else if( evt.name==glOffline.GO_UNSTABLE ) {
            if( this._conversationLabels ) {
                // Disconnect el
                if( this._conversationLabels.el.parentNode ) this._conversationLabels.el.parentNode.removeChild( this._conversationLabels.el );
            }
        }
        
        
    },
    */
    
    //}
},{
    
    /**
     * Array of threads seen in this session by gtdConversationView
     * Used for checking thread is sync'd correctly with UI (and vice versa)
     * @property {String[]}
     */
    THREADS_SEEN_CACHE: [],
    
});
