/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Self-contained view in the ExplorerThreadsViewer
 * // TODO Why is this not giUIObject ?
 *
 * @class ?
 */
var gtdExplorerBundle = giBase.extend({
    // Properties {
    
    
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	id: null,
	
	_type: null,
	_item: null,
	_parentThreadFilter: null,
	
	_elView: null,
	_elTab: null,
	_elTabCount: null,
	
    /** @property {private gtdThreadContainer} ? The shared thread container for all statuses. */
   	_statusThreadContainer: null,
    	
	/** @property {private String} ? The method for updating the server */
	_serverActionType: null,
    	
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	//_eventsContainer: null,
    
	_labelCache: null,
	
	
	
	
	
    //}


	// Methods {
	
	constructor: function(gtd, id) {
		this.base(gtd);
		this._main = gtd;
		//this._eventsContainer = new giEventsContainer(this);
		this.id = id;
		
		var doc = this._main.gmail.canvasDocument;
		
		// Create tab els:
		this._elTab = doc.createElement("LI");
		this._elTab.className = 'gtdi-menu-item';
		this._elTab.setAttribute('gtdi-tabid', this.id);
		
		var todayStr = (this.id==gtdExplorerBundle.TODAY)? " <img src='chrome://gtdinbox/skin/gtd/i/gtdi-star.png'/>" : "";
		this._elTab.innerHTML = "<a href='#'><span class='gtdi-menuitem'>" + giI18N.getString('ExplorerBundle.' + this.id + '.tabname') + todayStr + "</span><span class='gtdi-count></span>";
		this._elTabCount = this._elTab.ownerDocument.evaluate(".//*[@class='gtdi-count']", this._elTab, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
		
		
		// Create view container:
		this._elView = doc.createElement("DIV");
        this._elView.id = "gtdi-threads";
		
		// Refresh labels
		this._labelsChanged();
		this._main.labelsData.addEventListener(gtdLabelsData.LABEL_CHANGED, this._labelsChanged, this);
	},
	
	unload: function() {
		
		this._main.labelsData.removeEventListener(gtdLabelsData.LABEL_CHANGED, this._labelsChanged, this);
		
		
		if( this._elTab.parentNode ) this._elTab.parentNode.removeChild(this._elTab);
		this._elTab = null;
		
		if( this._elView.parentNode ) this._elView.parentNode.removeChild(this._elView);
		this._elView = null;
		
		// Wipe references to elements
		for( p in this ) {
			if( this[p] && this[p].nodeName && this[p].nodeType ) {
				this[p] = null;
			}
		}
		
		this._main = null;
		//this._eventsContainer.reset();
		
		this.base();
	},
	
	
	getViewEl: function() {
		return this._elView;
	},
	
	getTabEl: function() {
		return this._elTab;
	},

	testCondition: function() {	// TODO should we allow expandable (array of) conditions?
		return this._condition? this._condition() : true;
	},
	
	
	connect: function(type, item, parentThreadFilter) {
        
        this._type = type;
        this._item = item;
        this._parentThreadFilter = parentThreadFilter;
        
        
        // Attach to active bundles:
        gtdExplorerBundle._connectedBundles.push(this);
        
        this.dispatchEvent({name:gtdExplorerBundle.ACTIVATE});
        
	},
	
	disconnect: function() {
        // Deactivate
        
        // Remove from active bundles:
        for( var i = gtdExplorerBundle._connectedBundles.length - 1; i>=0; i-- ) {
            if( gtdExplorerBundle._connectedBundles[i].id==this.id ) {
                gtdExplorerBundle._connectedBundles.splice(i,1);
            }
        }
        
        this.dispatchEvent({name:gtdExplorerBundle.DEACTIVATE});
	},
	
	selected: function(selected) {
		if( selected ) {
			// Selected
			// Update tab
			giUIObject.addClassName(this._elTab, 'gtdi-active');
			this.dispatchEvent({name:gtdExplorerBundle.SELECT});
		} else {
			// Not Selected
			// Update tab
			giUIObject.removeClassName(this._elTab, 'gtdi-active');
			this.dispatchEvent({name:gtdExplorerBundle.DESELECT});
		}
	},
	
	resize: function(deltaY) {
		this.dispatchEvent({name:gtdExplorerBundle.RESIZE, deltaY:deltaY});
	},
	
	_labelsChanged: function() {
		var statusData = this._main.labelsData.getStatusData(true);
        var statuses = this._main.labelsData.getLabelsArray("statuses");
		var references = this._main.labelsData.getLabelsArray("references", false, true);
		this._labelCache = {
			statusData: statusData,
			tasksToDo: [],
            statuses: statuses,
			references: references,
		};
		var statuses = this._main.labelsData.getLabelsArray("statuses");
		for( var i = 0; i < statuses.length; i++ ) {
			var sKey = statusData.reverseLookup[statuses[i]];
			if( !sKey || !(sKey=='finished' || sKey=='today' || sKey=='waiting_on' || sKey=='some_day') ) {
				this._labelCache.tasksToDo.push(statuses[i]);
			}
		}
		
	},
	
	/**
	* Set method for updating server. Ensure future created objects can use this info; and existing objects are updated.
	* @function ?
	* @param {String} type - The type. A static variable of glActionManager
	*/
	setServerActionType: function(type) {
		this._serverActionType = type;
	},
	
	/**
	* Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
	* @function {Boolean} ?
	* @return Whether the UI can be cached
	*/
	isCacheable: function() {
		return false;
	},
	

}, {
	
	// Events:
	ACTIVATE: 'activate',
	DEACTIVATE: 'deactivate',
	SELECT: 'select',
	DESELECT: 'unselect',
	RESIZE: 'resize',
	
	
	// Default Ids:
	ALL: 'all',
	TODAY: 'today',
	PERSONAL: 'personal',
	RECENT: 'recent',
	TASKS_TO_DO: 'tasksToDo',
	TASKS_WAITING_ON: 'tasksWaitingOn',
	TASKS_SOME_DAY: 'tasksSomeDay',
	RESOURCES: 'resources',
	REVIEWS: 'reviews',
	
	_connectedBundles: [],
});
gtdExplorerBundle.implement(giEventDispatcher); // Add event dispatching support








/**
 * Specialised for Bundles that contain ThreadLists
 *
 * @class ?
 */
var gtdExplorerBundleTlc = gtdExplorerBundle.extend({
    // Properties {
    
    
    /** @property {private gtdThreadContainer} ? The shared thread container for all statuses. */
   	_threadContainer: null,	

    /** @property {private gtdThreadList} ? */
	_threadList: null,
	
    //}


	// Methods {
	
	constructor: function(gtd, id) {
		this.base(gtd, id);
	},
	
	unload: function() {
		
		if( this._threadList ) this._threadList.disconnect();
		this._threadList = null;
		
		if( this._threadContainer ) {
			this._threadContainer.removeEventListener(gtdThreadContainer.LOADED, this._threadChanged, this);
			this._threadContainer.removeEventListener(gtdThreadContainer.LOADING_UPDATE, this._threadChanged, this);
			this._threadContainer.removeEventListener(gtdThreadContainer.CHANGED, this._threadChanged, this);
		}
		this._threadContainer = null
		
		this.base();
	},
	
	connect: function(type, item, parentFilter) {
		this.base(type, item, parentFilter);
		this.syncThreadContainer();
	},
	
	disconnect: function() {
		this.selected(false);
		
		if( this._threadContainer ) {
			this._threadContainer.removeEventListener(gtdThreadContainer.LOADED, this._threadChanged, this);
			this._threadContainer.removeEventListener(gtdThreadContainer.LOADING_UPDATE, this._threadChanged, this);
			this._threadContainer.removeEventListener(gtdThreadContainer.CHANGED, this._threadChanged, this);
		}
		if( this._threadList ) this._threadList.disconnect();
		this._threadList = null;
		this._threadContainer = null;
		this.base();
	},
	
	
	selected: function(selected) {
		this.base(selected);
		if( selected ) {
			// Selected
			
			// Check threadContainer exists
			if( !this._threadContainer ) {
				this._threadContainer = this._getThreadContainer();	// Get correct one for type/item
				this._threadContainer.addEventListener(gtdThreadContainer.LOADED, this._threadChanged, this);
				this._threadContainer.addEventListener(gtdThreadContainer.LOADING_UPDATE, this._threadChanged, this);
				this._threadContainer.addEventListener(gtdThreadContainer.CHANGED, this._threadChanged, this);
				this._refreshTabCount();
			}
			
			// Create container, if required
			if( !this._threadList ) {
				this._threadList = this._main.getUIObject("threadList");
				this._threadList.load(null, this._threadContainer, this.id);
                this._threadList.addActionsBar();
				//this._threadList.setType( this._type, this.id );
				var f = this._filter? giUtil.bind(this, this._filter) : null;
				this._threadList.setFilter('tab', f);
				
				// Update how it interacts with server
				this._threadList.setServerActionType(this._serverActionType);
				
				this._elView.appendChild( this._threadList.el );
			}
			
			this._threadList.setFilter('explorerThreadsViewer', this._parentThreadFilter); // Could have changed, depending on item
			this._threadList.active(true);
			
			
			// Nb - leave it to parent to decide how to integrate elView into the DOM - we just prep it
		} else {
			// Not Selected
			
			if( this._threadList ) {
				this._threadList.active(false);
			}
			
			// Nb - leave it to parent to decide how to remove elView from DOM
		}
	},
	
	resize: function(deltaY) {
		this.base(deltaY);
	},
	
	_threadChanged: function(event) {
		// Refresh tab counts
		this._refreshTabCount();
	},
	
	_refreshTabCount: function() {
		if( !this._threadContainer ) {
			this._elTabCount.innerHTML = "?";
			return;
		}
		
		var c = 0;
		for( var i = 0; i < this._threadContainer.threads.length; i++ ) {
			var thread = this._threadContainer.threads[i];
			if( !this._parentThreadFilter || this._parentThreadFilter(thread) ) {
				if( !this._filter || this._filter(thread) ) {
					c++;
				}
			}
		}
		this._elTabCount.innerHTML = c;
	},
	
	getThreadList: function() {
		return this._threadList;
	},
	
	syncThreadContainer: function() {
		if( !this._threadContainer ) {
			this._threadContainer = this._getThreadContainer(true);	// Get whatever is cached
			if( this._threadContainer ) {
				this._threadContainer.addEventListener(gtdThreadContainer.LOADED, this._threadChanged, this);
				this._threadContainer.addEventListener(gtdThreadContainer.LOADING_UPDATE, this._threadChanged, this);
				this._threadContainer.addEventListener(gtdThreadContainer.CHANGED, this._threadChanged, this);
			}
		}	
		this._refreshTabCount();
	},
	
	_getThreadContainer: function(doNotCreate) {
		// Sub classed. Should not execute unless not specified.
		return this._threadList;
	},
	
	/**
	* Try to find a threadContainer for a given search. If not found, create one
	* @function {gtdThreadContainer} ?
	* @paramset Input 1
	* @param {glSearchConditions} query - The search to find/match
	* @param {optional int} numberThreads - The number of threads to retrieve 
	* @param {optional boolean} doNotCreate - If true, only try to find existing gtdThreadContainers
	* @paramset Input 2
	* @param {String} query - The search to find/match
	* @param {optional int} numberThreads - The number of threads to retrieve 
	* @param {optional boolean} doNotCreate - If true, only try to find existing gtdThreadContainers
	* @return The gtdThreadContainer for this search
	*/
	_getThreadContainerBySearch: function(query,numberThreads, doNotCreate) {
	
		if( typeof numberThreads=='undefined' ) numberThreads = gtdExplorerThreadsViewer.DEFAULT_NUMBER_THREADS;
		var threadContainer = null;
		// Try to find in other bundles
		for( var i = 0; i < gtdExplorerBundle._connectedBundles.length; i++ ) {
			var bundle = gtdExplorerBundle._connectedBundles[i];
			if( bundle!=this && bundle._threadContainer ) {
				if( bundle._threadContainer._search && bundle._threadContainer._search.getQuery()==query.toString() ) {
					threadContainer = bundle._threadContainer;
					break;
				}
			}
		}
		// Create own if needed:
		if( !threadContainer && !doNotCreate ) {
			var search = new glSearch(this._main.gmail);
			search.find(query);
			threadContainer = gtdThreadContainer.createWithSearch(this._main, search);
			threadContainer.search(numberThreads);
		}
		return threadContainer;
	},
	
	setServerActionType: function(type) {
		if( this._threadList ) this._threadList.setServerActionType(type);
		this.base(type);
	},
	

});










var gtdExplorerBundleTlcAll = gtdExplorerBundleTlc.extend({
	
	constructor: function(gtd) {
		this.base(gtd, gtdExplorerBundle.ALL);
	},
	
	_getThreadContainer: function(doNotCreate) {
		if( !this._threadContainer ) {
			switch( this._type ) {
				case gtdExplorerThreadsViewer.STATUSES:
					this._threadContainer = this._main.statusThreadsContainer;
					break;
				case gtdExplorerThreadsViewer.SEARCH:
					this._threadContainer = this._getThreadContainerBySearch(this._item, null, doNotCreate);
					break;
			}
		}
		return this._threadContainer;
	},
	
	
});

var gtdExplorerBundleTlcToday = gtdExplorerBundleTlc.extend({
	
	constructor: function(gtd) {
		this.base(gtd, gtdExplorerBundle.TODAY);
	},
	
	_filter: function(thread) {
		return thread.hasLabel( this._labelCache.statusData.today );
	},
	
	_condition: function() {		
		if( this._main.labelsData.hasLabel(this._labelCache.statusData.today) ) return true;
		else return false;
	},
	
	_getThreadContainer: function(doNotCreate) {
		if( !this._threadContainer ) {
			switch( this._type ) {
				case gtdExplorerThreadsViewer.STATUSES:
				case gtdExplorerThreadsViewer.PROJECTS:
				case gtdExplorerThreadsViewer.CONTEXTS:
				case gtdExplorerThreadsViewer.REFERENCES:
				case gtdExplorerThreadsViewer.MISC:
				case gtdExplorerThreadsViewer.DASHBOARD:
				case gtdExplorerThreadsViewer.CONTACT:
					this._threadContainer = this._main.statusThreadsContainer;
					break;
				case gtdExplorerThreadsViewer.SEARCH:
					this._threadContainer = this._getThreadContainerBySearch(this._item, null, doNotCreate);
					break;
			}
		}
		return this._threadContainer;
	},
	
});

var gtdExplorerBundleTlcRecent = gtdExplorerBundleTlc.extend({

	constructor: function(gtd) {
		this.base(gtd, gtdExplorerBundle.RECENT);
	},

	_getThreadContainer: function(doNotCreate) {
		if( !this._threadContainer ) {
			switch( this._type ) {
				case gtdExplorerThreadsViewer.PROJECTS:
				case gtdExplorerThreadsViewer.CONTEXTS:
				case gtdExplorerThreadsViewer.REFERENCES:
				case gtdExplorerThreadsViewer.MISC:
					this._threadContainer = this._getThreadContainerBySearch(glSearch.l(this._item), 25, doNotCreate);
					break;
				case gtdExplorerThreadsViewer.CONTACT:	
					var c = new glSearchAndConditions();
					c.addCondition( glSearch.restrictToUser(this._item, "OR") );
					this._threadContainer = this._getThreadContainerBySearch(c, 25, doNotCreate);
				
					//this._threadContainer = this._getThreadContainerBySearch(glSearch.restrictToUser(this._item, "OR"), 25, doNotCreate);
					break;
			}
		}
		return this._threadContainer;
	},

});


var gtdExplorerBundleTlcTasksToDo = gtdExplorerBundleTlc.extend({

	constructor: function(gtd) {
		this.base(gtd, gtdExplorerBundle.TASKS_TO_DO);
	},

	_filter: function(thread) {
			for( var i = 0; i < this._labelCache.tasksToDo.length; i++ ) if( thread.hasLabel( this._labelCache.tasksToDo[i] ) ) return true;
			return false;
	},

	_condition: function() {
        var statuses = this._labelCache.statuses;
        var c = 0;
        for( var i = 0; i < statuses.length; i++ ) {
            if( statuses[i]!=this._labelCache.statusData.waiting_on && statuses[i]!=this._labelCache.statusData.some_day && statuses[i]!=this._labelCache.statusData.finished ) {
                c++;
            }
        }
		if( c>0 ) return true;
		else return false;
	},

	
	_getThreadContainer: function(doNotCreate) {
		if( !this._threadContainer ) {
			switch( this._type ) {
				case gtdExplorerThreadsViewer.PROJECTS:
				case gtdExplorerThreadsViewer.CONTEXTS:
				case gtdExplorerThreadsViewer.REFERENCES:
				case gtdExplorerThreadsViewer.MISC:
				case gtdExplorerThreadsViewer.DASHBOARD:
				case gtdExplorerThreadsViewer.CONTACT:
					this._threadContainer = this._main.statusThreadsContainer;
					break;
				case gtdExplorerThreadsViewer.SEARCH:
					this._threadContainer = this._getThreadContainerBySearch(this._item, null, doNotCreate);
					break;
			}
		}
		return this._threadContainer;
	},

});


var gtdExplorerBundleTlcTasksWaitingOn = gtdExplorerBundleTlc.extend({

	constructor: function(gtd) {
		this.base(gtd, gtdExplorerBundle.TASKS_WAITING_ON);
	},

	_filter: function(thread) {
		return thread.hasLabel( this._labelCache.statusData.waiting_on );
	},

	_condition: function() {		
		if( this._main.labelsData.hasLabel(this._labelCache.statusData.waiting_on) ) return true;
		else return false;
	},
	
	_getThreadContainer: function(doNotCreate) {
		if( !this._threadContainer ) {
			switch( this._type ) {
				case gtdExplorerThreadsViewer.PROJECTS:
				case gtdExplorerThreadsViewer.CONTEXTS:
				case gtdExplorerThreadsViewer.REFERENCES:
				case gtdExplorerThreadsViewer.MISC:
				case gtdExplorerThreadsViewer.DASHBOARD:
				case gtdExplorerThreadsViewer.CONTACT:
					this._threadContainer = this._main.statusThreadsContainer;
					break;
				case gtdExplorerThreadsViewer.SEARCH:
					this._threadContainer = this._getThreadContainerBySearch(this._item, null, doNotCreate);
					break;
			}
		}
		return this._threadContainer;
	},

});


var gtdExplorerBundleTlcTasksSomeDay = gtdExplorerBundleTlc.extend({

	constructor: function(gtd) {
		this.base(gtd, gtdExplorerBundle.TASKS_SOME_DAY);
	},

	_filter: function(thread) {
		return thread.hasLabel( this._labelCache.statusData.some_day );
	},

	_condition: function() {		
		if( this._main.labelsData.hasLabel(this._labelCache.statusData.some_day) ) return true;
		else return false;
	},
	
	_getThreadContainer: function(doNotCreate) {
		if( !this._threadContainer ) {
			switch( this._type ) {
				case gtdExplorerThreadsViewer.PROJECTS:
				case gtdExplorerThreadsViewer.CONTEXTS:
				case gtdExplorerThreadsViewer.REFERENCES:
				case gtdExplorerThreadsViewer.MISC:
				case gtdExplorerThreadsViewer.DASHBOARD:
				case gtdExplorerThreadsViewer.CONTACT:
					this._threadContainer = this._main.statusThreadsContainer;
					break;
				case gtdExplorerThreadsViewer.SEARCH:
					this._threadContainer = this._getThreadContainerBySearch(this._item, null, doNotCreate);
					break;
			}
		}
		return this._threadContainer;
	},

});


var gtdExplorerBundleTlcResources = gtdExplorerBundleTlc.extend({

	constructor: function(gtd) {
		this.base(gtd, gtdExplorerBundle.RESOURCES);
	},

	_filter: function(thread) {
		if( thread.attachmentsCSV ) return true;
		for( var i = 0; i < this._labelCache.references.length; i++ ) if( thread.hasLabel( this._labelCache.references[i] ) ) return true;
	},

	_getThreadContainer: function(doNotCreate) {
		if( !this._threadContainer ) {
			switch( this._type ) {
				case gtdExplorerThreadsViewer.PROJECTS:
				case gtdExplorerThreadsViewer.CONTEXTS:
				case gtdExplorerThreadsViewer.REFERENCES:
				case gtdExplorerThreadsViewer.MISC:
					var cResources = glSearch.joinLabels(this._main.labelsData.getLabelsArray("references", false, true), 'OR');
					cResources.addCondition("has:attachment");
					var c = new glSearchAndConditions();
					c.addCondition( glSearch.l(this._item) );
					c.addCondition(cResources);
					this._threadContainer = this._getThreadContainerBySearch(c, null, doNotCreate);
					
					//this._threadContainer = this._getThreadContainerBySearch(glSearch.l(this._item) + " AND (has:attachment OR " + glSearch.joinLabels(this._main.labelsBox.getLabelsArray("references", false, true), "OR") + ")", null, doNotCreate);
					break;
				case gtdExplorerThreadsViewer.CONTACT:
					var cResources = glSearch.joinLabels(this._main.labelsData.getLabelsArray("references", false, true), 'OR');
					cResources.addCondition("has:attachment");
					var c = new glSearchAndConditions();
					c.addCondition( glSearch.restrictToUser(this._item, "OR") );
					c.addCondition(cResources);
					this._threadContainer = this._getThreadContainerBySearch(c, null, doNotCreate);
					
					//this._threadContainer = this._getThreadContainerBySearch(glSearch.restrictToUser(this._item, "OR") + " AND (has:attachment OR " + glSearch.joinLabels(this._main.labelsBox.getLabelsArray("references", false, true), "OR") + ")", null, doNotCreate);
					break;
				case gtdExplorerThreadsViewer.SEARCH:
					this._threadContainer = this._getThreadContainerBySearch(this._item, null, doNotCreate);
					break;
			}
		}
		return this._threadContainer;
	},

});
