/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Add labels and pre-labelling to the compose screen, when connected
 *
 * @class ? @extends gtdUIObject
 */
var gtdGmailLabelsBox = giUIObject.extend({
    // Properties {
        	
	
	/** @property {Element} ? The title on Recent items */
	elRecentTitle: null,
	
    /** @property {Element} ? The recent table */
	elRecent: null,

	
    /** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
    //}

    // Methods {
    /**
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting gtdGmailLabelsBox object to view', 'gtdGmailLabelsBox');

        this.base(main);
		this._eventsContainer = new giEventsContainer(this);

        var doc = this._main.gmail.canvasDocument;
        
        // Retrieve the Recent container:
        var elFirstRecentItem = doc.evaluate(".//div[@class='n3']/div/table//a[contains(@href, '#label/')]", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        while( elFirstRecentItem.parentNode && elFirstRecentItem.nodeName!="TABLE" ) {
            elFirstRecentItem = elFirstRecentItem.parentNode;
        }
        this.elRecent = elFirstRecentItem;
        
        
        this.elRecentTitle = doc.createElement("DIV");
        this.elRecentTitle.id = "gtdi-gmaillabelsbox-recent-title";
        this.elRecentTitle.className = "n2";
        this.elRecentTitle.innerHTML = "<IMG src=''/> " + giI18N.getString("GmailLabelsBox.Recent.Title");
        this.elRecentTitle.title = giI18N.getString("GmailLabelsBox.Recent.Title.tooltip");
        this.elRecent.parentNode.insertBefore(this.elRecentTitle, this.elRecent);
        
		// Attach listeners to Title button:
		this._eventsContainer.observe( this.elRecentTitle, "mousedown", this._clickRecentTitle, false );
        this._eventsContainer.observe( this.elRecentTitle, "mouseover", this._mouseoverRecentTitle, false );
        
        // Set history:
        this._recentHidden( this._main.prefs.getPref("components.navigation.gmail_labels_box.recent_hidden") );
    },
    
	

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdGmailLabelsBox from view', 'gtdGmailLabelsBox');

        if( this.elRecentTitle ) this.elRecentTitle.parentNode.removeChild( this.elRecentTitle );
        this.elRecentTitle = null;
        this.elRecent = null;

		this._eventsContainer.reset();
        this._eventsContainer = null;
        
        this.base();
    },
    
    /**
     * Notification of a click on the recent title
     * @function {} ?
     */
    _clickRecentTitle: function(evt) {
        try {
            // Toggle:
            this._recentHidden( !this._recentHidden() );
        } catch(e) {
            giLogger.warn(e, 'gtdGmailLabelsBox._clickRecentTitle');
        }
    },
    
    /**
     * Notification of a mouseover on the recent title
     * Used to see if its carrying a label; in which case, expand it
     * @function {} ?
     */
    _mouseoverRecentTitle: function(evt) {
        try {
            if( this._recentHidden() ) {
                var doc = this._main.gmail.canvasDocument;
                var elMover = doc.evaluate("./div[@class='lE']", doc.body, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                if( elMover ) {
                    // It is dragging a label
                    this._recentHidden(false);
                }
            }
        } catch(e) {
            giLogger.warn(e, 'gtdGmailLabelsBox._mouseoverRecentTitle');
        }
    },
    
    /**
     * Set the visibility of the Recent Labels container
     * @function {Boolean} ?
     * @param {Boolean} v - If present, set it hidden (if true) or visible (if false)
     * @return Return the visibility
     */
    _recentHidden: function(v) {
        if( typeof v!="undefined" ) {
            this.elRecent.style.display = v? "none" : "";
            if( v ) {
                giUIObject.addClassName(this.elRecentTitle, "gtdi-gmaillabelsbox-recent-hidden");
                this.elRecentTitle.getElementsByTagName("IMG")[0].src = "chrome://gtdinbox/skin/gtd/group-closed.gif";
            } else {
                this.elRecentTitle.getElementsByTagName("IMG")[0].src = "chrome://gtdinbox/skin/gtd/group-open.gif";
                giUIObject.removeClassName(this.elRecentTitle, "gtdi-gmaillabelsbox-recent-hidden");
            }
            // Update settings:
            this._main.prefs.setPref("components.navigation.gmail_labels_box.recent_hidden", v);
        }
        return this.elRecent.style.display=="none";
    },
    
    //}
});