/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * The label add/remover UI for any thread object. 
 *
 * @class ? @extends giUIObject
 */
var gtdLabellingBox = giUIObject.extend({
    // Properties {
    
	
	/** @property {private boolean} ? If true, when a label is clicked, it is automatically added/removed from the thread. */
	_autoUpdate: null,
    
    /** @property {private boolean} ? If true, hide the Archive/Finish cmds
	_hideCmds: null,
	
	/** @property {Object} ? A hash of label rows. Format: _uiRows[<type>] = {el:, disabled:true/false} */
	_uiRows: null,
    
    /** @property {Object} ? A hash of labelling dropdowns. Format: {<labeltype>: gtdLabellingDropdown} */
    _labellingDropdowns: null,
	
    	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
	/** @property {glThread} ? The associated thread object */
	thread: null,
	
	/** @property {Element} ? The label container */
	el: null,
	
	/** @property {Element} ? The row container (child of 'el') */
	elRows: null,
	
	/** @property {private String} ? Method for updating server (e.g. labels) */
	_serverActionType: null,
	
	/** @property {Element} ? Contains archive/finish/etc. */
	_elCmdButtons: null,
    
    /** @property {Element} ? The +/- row toggle */
    _elExpander: null,
	
	/** @property {Object} ?  */	
	_btnArchive: null,
	
	/** @property {Object} ?  */	
	_btnFinish: null,
    
    /**
     * Has a thread if the user has clicked the move next arrow on the Archive/Finish button.
     * @property {glThread} ?
     */
    _moveNextRequest: false,
    
    /**
     * Has a thread if the user has clicked the main portion on the Archive/Finish button.
     * @property {glThread} ?
     */
    _archiveRequest: false,

    /**
     * The timeout ID to close the temporary expander
     * @property {int}
     */
    _popupTimeout: null,

    //}

	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdLabellingBox object to view', 'gtdLabellingBox');
	
		this.base(main);
		this._eventsContainer = new giEventsContainer(this);
		this._uiRows = {};
        this._labellingDropdowns = {};
		
		// Create the nodes:
		var doc = main.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.className = "gtdi";
		
		var elInner = doc.createElement("DIV");
		elInner.className = "gtdi-labelling";
		this.el.appendChild( elInner );
		this.elRows = doc.createElement("UL");
		elInner.appendChild( this.elRows );
		
		
		// Create basic structure:
		this._createLabelStructure();

        // Add clear fix
        var elSpan = doc.createElement('SPAN');
        elSpan.className = 'gtdi-clearfix';
        this.el.appendChild( elSpan );
		
		// Listen for changes that will require a structural change:
		this._main.labelsData.addEventListener(gtdLabelsData.LABEL_CHANGED, this._createLabelStructure, this);
		
		// Listen for changes to threads
		this._main.gmail.actions.addEventListener(glActionManager.THREAD_CHANGED, this._threadUpdated, this);
		
		// Listen for clicks
		this._eventsContainer.observe( this.el, "click", this._click, false );

	},
	

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdLabellingBox from view', 'gtdLabellingBox');
        
        
        for( t in this._labellingDropdowns ) {
            this._labellingDropdowns[t].disconnect();
        }
        
        if( this._popupTimeout ) {
            clearTimeout( this._popupTimeout );
            this._popupTimeout = null;
        }
        
        this._main.labelsData.removeEventListener(gtdLabelsData.LABEL_CHANGED, this._createLabelStructure, this);
        this._main.gmail.actions.removeEventListener(glActionManager.THREAD_CHANGED, this._threadUpdated, this);

        if( this.el && this.el.parentNode ) this.el.parentNode.removeChild( this.el );
        this.el = null;
        this.elRows = null;
        
        this._moveNextRequest = null;
        this._archiveRequest = null;
        
        this._eventsContainer.reset();
        this._eventsContainer = null;
        
        this.base();
    },
	
	/**
	* Clear the container el, create a row for statuses, and a row for labels
	* If thread loaded, reload it
	*/
    _createLabelStructure: function() {
        
        
        // Store any previous selections:
        var currentLabels = this._getCurrentLabels();
        
		// Clear out any existing structure:
        for( t in this._labellingDropdowns ) {
            this._labellingDropdowns[t].disconnect();
        }
		this.elRows.innerHTML = "";
		this._uiRows = {};
        this._labellingDropdowns = {};
				
		// Retrieve the types
		var doc = this._main.gmail.canvasDocument;
		var typeSettings = this._main.prefs.getPref('labels.types');
		labelsData = this._main.labelsData;

		// Create statuses row:
        this._uiRows.statuses = {};
        this._uiRows.statuses.el = doc.createElement("LI");
        this.elRows.appendChild( this._uiRows.statuses.el );
        
        this._uiRows.statuses.el.className = "gtdi-row-actions";
        var elUL = doc.createElement("UL");
        this._uiRows.statuses.el.appendChild( elUL );
        
        
        var statuses = labelsData.getLabelsArray("statuses", true, true);
        var statusData = this._main.labelsData.getStatusData(true);
        var pos = statuses.indexOf(statusData.finished);
        if( pos>-1 ) {
            // Push finished to the end
            statuses.splice(pos, 1);
            statuses.push(statusData.finished);
        }
        for( var i = 0; i < statuses.length; i++ ) {
            var key = statusData.reverseLookup[statuses[i]];
            
                if( key=="today" ) {
                    var elLi = doc.createElement("LI");
                    elLi.className = "gtdi-label-status gtdi-label-"+key;
                    elLi.setAttribute("gtdLabelName", statuses[i]);
                    elLi.innerHTML = "<a href='#'>" + statuses[i].replace(statusData.prefix,"") + "</a>";
                    elUL.appendChild( elLi );
                } else {
                    var elLi = doc.createElement("LI");
                    elLi.className = "gtdi-label-status gtdi-label-" + key;
                    elLi.setAttribute("gtdLabelName", statuses[i]);
                    elLi.innerHTML = "<a href='#'><b><span class='gtdi-labelname'>" + statuses[i].replace(statusData.prefix,"") + "</span></b></a>";
                    elUL.appendChild( elLi );
                    if( key=="finished" ) {
                        this._btnFinish = elLi;
                    }
                }
        }
        
        
        // Add the Archive/Finish buttons
        this._elCmdButtons = doc.createElement("DIV");
        this._elCmdButtons.id = "gtdi-arcfin";
        this._elCmdButtons.className = "gtdi-btngrp";
        this._uiRows.statuses.el.appendChild( this._elCmdButtons );

        this._btnArchive = gtdLabellingBox.createCmdButton(doc, 'archive', giI18N.getString("Labelling.buttons.archive"), giI18N.getString("Labelling.buttons.archive.tooltip"), true, giI18N.getString("Labelling.buttons.archive.next.tooltip"))
        this._elCmdButtons.appendChild( this._btnArchive.container );
        
        /*
        this._btnFinish = gtdLabellingBox.createCmdButton(doc, 'finish', giI18N.getString("Labelling.buttons.finish"), giI18N.getString("Labelling.buttons.finish.tooltip"), true, giI18N.getString("Labelling.buttons.finish.next.tooltip"))
        this._elCmdButtons.appendChild( this._btnFinish.container );
        */
        
		
        
        // Add a row for labels:
        this._uiRows.labels = {};
        this._uiRows.labels.el = doc.createElement("LI");
        this._uiRows.labels.el.className = "gtdi-row";
		//this._uiRows.labels.el.innerHTML = "<h3>" + giI18N.getString("Labelling.labels") + "</h3>";
        this.elRows.appendChild( this._uiRows.labels.el );
        
        // Create each dropdown:
        for( t in typeSettings ) {
            if( t!='statuses' && t!='old' ) {
                this._labellingDropdowns[t] = this._main.getUIObject('labellingDropdown');
                this._labellingDropdowns[t].setType(t);
                this._uiRows.labels.el.appendChild( this._labellingDropdowns[t].el );
            }
        }
        
        
		this._testCmdButtons();
        
        // Restore any:
        if( this.thread ) {
            this.uiSelectLabels(this.thread.labels);
        } else if( currentLabels.length > 0 ) {
            this.uiSelectLabels(currentLabels);
        }
                
	},
	
	
	/**
	* Associate a thread with this box, load its labels and listen for changes to that thread.
	* @function ?
	* @param {glThread} - The thread object
	* @param {Boolean} autoUpdate - If true, when a label is clicked, it is automatically added/removed from the thread.
	* @param {Boolean} hideCmds - Hide the Archive/Finish box
	*/
	load: function(thread, autoUpdate, hideCmds) {
        
        
		// Disassociated any existing thread
		if( this.thread ) {
			this.thread = null;
		}
                
		
		// Attach the new thread (might be null)
		this.thread = thread;
		                
		this._autoUpdate = autoUpdate;
        this._hideCmds = hideCmds;
        
        // Nb Do NOT reset move-next request; as multiple .load requests may occur (async environment) before the Archive event the move-next request is waiting for.
        if( this._hideCmds ) {
            this._elCmdButtons.style.display = "none";
            this._btnFinish.style.display = "none";
        } else {
            this._elCmdButtons.style.display = "";
            this._btnFinish.style.display = "";
        }
        
		// Select labels
		if( this.thread ) {
			this.uiSelectLabels( this.thread.labels );
			
			// Set cmds
			this._testCmdButtons();
			//if( this._elCmdButtons ) this._elCmdButtons.style.display = "";
		} else {
			//if( this._elCmdButtons ) this._elCmdButtons.style.display = "none";
		}
        
    },
	
	/**
	* Update the UI to show all provided labels as 'selected' and all other labels as unselected.
	* Used by pre-labelling in Compose; and when loading a thread
	* @function ?
	* @param {string[]} - Array of label names to mark as selected
	*/
	uiSelectLabels: function(labels) {
		// Mark all previous labels as unselected (including dropdowns):
        
		var doc = this._main.gmail.canvasDocument;
		
		// Get active labels:
        var currentLabels = this._getCurrentLabels();
		// Remove any still desired labels, or any private ones:
		for( var i = currentLabels.length - 1; i>=0; i-- ) {
			if( labels.indexOf(currentLabels[i])>-1 ) {
                currentLabels.splice(i, 1); // Ensure it isn't removed
            }
		}
		// Now remove the no longer wanted labels:
		for( var i = 0; i < currentLabels.length; i++ ) {
			this._setLabel(currentLabels[i], false);
		}
		
		// Mark labels as selected:
		for( var i = 0; i < labels.length; i++ ) {
			if( labels[i].indexOf("^")==-1 ) this._setLabel( labels[i], true );
		}
        
	},
	
    /**
     * Return an array of currently active labels
     * @function {String[]}
     * @return Array of labels
     */
    _getCurrentLabels: function() {
        var currentLabels = [];
		var result = this._main.gmail.canvasDocument.evaluate(".//*[contains(@class,'gtdi-active')]", this.elRows, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
		for( var i = 0; i < result.snapshotLength; i++ ) {
			currentLabels.push( result.snapshotItem(i).getAttribute("gtdLabelName") );
		}
        return currentLabels;
    },
	
	/**
	* Visually set a label as selected or unselected in the box, and propagate to dropdowns.
	* @function ?
	* @param {string} labelname - The name of the label to update
	* @param {boolean} isOn - If true, mark selected. Otherwise, mark unselected.
	*/
	_setLabel: function(labelname, isOn) {
		if( !labelname ) return;
		// Find the type
		var foundType = 'misc';
		var typeSettings = this._main.prefs.getPref('labels.types');
		for( t in typeSettings ) {
			if( t!='misc' && gtdLabelsData.testPrefix(labelname, typeSettings[t].prefix) ) {
				foundType = t;
				break;
			}
		}

		if( !foundType || (!this._uiRows[foundType] && !this._labellingDropdowns[foundType]) ) return;
		
		// Choose how to handle the label. Statuses are treated uniquely.
		var doc = this._main.gmail.canvasDocument;
		if( foundType=="statuses" ) {
			// Get the relevant label
			var elLabel = null;
			if( labelname.indexOf("'")>-1 ) {	// Contains ', do long way. TODO - find native XPath solution for apostrophe
				var result = doc.evaluate("./li[@gtdlabelname]", this._uiRows.statuses.el, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
				for( var i = 0; i < result.snapshotLength; i++ ) {
					if( result.snapshotItem(i).getAttribute("gtdLabelName")==labelname ) {
						elLabel = result.snapshotItem(i);
						break;
					}
				}
			} else {
				elLabel = doc.evaluate(".//li[@gtdlabelname='" + labelname + "']", this._uiRows.statuses.el, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
			}
			if( elLabel ) {
				if( isOn ) giUIObject.addClassName( elLabel, "gtdi-active" );	// Mark it selected
				else giUIObject.removeClassName( elLabel, "gtdi-active" );	// Mark it unselected
			}
		} else {
			// Now notify the labellingDropdown
			this._labellingDropdowns[foundType].setLabel(labelname, isOn);
		}
	},
	
	
	/**
	* Set method for updating server (e.g. labels)
	* @function ?
	* @param {String} type - The type. A static variable of glActionManager
	*/
	setServerActionType: function(type) {
		this._serverActionType = type;
	},
	
	
	/**
	* Respond to a click event.
	* Could be on a status button, on a label 'X', or the expander.
	* @function ?
	*/
	_click: function(event) {
		try {
			var el = event.target;
			if( el.className=="gtdi-labelremove" ) {	// Clicked 'X'
				var labelname = el.parentNode.getAttribute("gtdLabelName");
				this._removeLabel(labelname);
			} else if( el.getAttribute('gtdLabelName') && !el.hasAttribute('gtdi-placeholder') ) {
                if( giUIObject.hasClassName( el, "gtdi-active" ) ) {
                    this._removeLabel( el.getAttribute('gtdLabelName') );
                } else {
                    this._addLabel( el.getAttribute('gtdLabelName') );
                }
			} else {					// Work up tree to understand it
				while( el.parentNode ) {
                    if( giUIObject.hasClassName(el, 'gtdi-droppeddown-create') ) { // Create label
                    
                        // Popup create:
                        var doc = el.ownerDocument;
                        var type = el.getAttribute("gtdLabelType");
                        var prefix = this._main.prefs.getPref('labels.types.'+type+'.prefix');
                        
                        if( this._main.gmail.page.getActiveViewType()=="cv" && !this._main.getUIObject('browserPopup').isActive() ) {
                            // Must use the Labels Menu dropdown, or else the thread does not know about.
                            // Only alternative is to devise a refresh for the conversation view.
                            
                            this._main.gmail.actions.performAction(glActionManager.CREATE_LABEL, null, glActionManager.USE_DOM);
                            
                            // Now wait for the input box to give it the prefix:
                            var actionsHeader = this._main.gmail.getCurrentViewObject().actionsHeader;
                            var timer = {count: 0, id: null};
                            timer.id = setInterval( function() {
                                var elInput = actionsHeader.getCreateLabelInput();
                                if( elInput || ++count>5) {
                                    // found it - clear interval and set prefix
                                    clearInterval( timer.id );
                                    if( elInput ) elInput.value = prefix;
                                }
                            }, 200);
                            
                        } else {
                            var typeName = giI18N.getString('Labels.Types.'+type, this._type);
                            var label = prompt(giI18N.getString('Labelling.dropdown.create.prompt').replace('%1',typeName), prefix);
                            if( label && !this._main.labelsData.hasLabel(label) ) {
                                if( this.thread ) {
                                    // Create label and add to thread
                                    this._main.gmail.actions.performAction(glActionManager.ADD_LABEL, this.thread, glActionManager.USE_AJAX, label);
                                } else {
                                    // Most likely we are in Compose. No thread to use.
                                    this._main.gmail.actions.performAction(glActionManager.CREATE_LABEL, null, glActionManager.USE_AJAX_SYNC, label);
                                    // In this case, we must fake add the label temporarily. When it completes, it will add the label properly, so long as it's selected now (createLabelStructure will restore all _getCurrentLabel items)
                                    //this._labellingDropdowns[type].unsafeAddLabel(label);
                                    // Take advantage of the fact CREATE_LABEL fires instantly (i.e. in sync). If wanted to be resolute, we could use USE_AJAX_SYNC.
                                    this._addLabel(label);
                                }
                            }
                        }
                    } else if( el.className.indexOf("gtdi-archive")>-1 ) {
                        if( this.thread.hasLabel("^i") ) {
                            giLogger.log('Click Archive', 'gtdLabellingBox._click');
                            var isNext = el.className.indexOf('gtdi-archive-next')>-1;
                            this.dispatchEvent({name: gtdLabellingBox.CLICK_ARCHIVE, moveNext: isNext, threadId: this.thread.threadId});
                            /*
                            if( el.className.indexOf("gtdi-archive-next")>-1 ) {
                                this._moveNextRequest = this.thread;   // Store the Thread ID so even if .load is called before archive callback reached, we can catch it.
                            } else {
                                this._archiveRequest = this.thread;
                            }
                            var serverActionType = (this._moveNextRequest && this._main.gmail.page.getActiveViewType()=='cv')? glActionManager.USE_AJAX : this._serverActionType;
                            if( this._main.gmail.page.getActiveViewType()!='cv' ) this._main.gmail.actions.performAction(glActionManager.MARK_AS_READ, this.thread.getRecentId(), serverActionType);
                            this._main.gmail.actions.performAction(glActionManager.ARCHIVE, this.thread.getRecentId(), serverActionType);
                            */
                            
                        }
                    /*
                    } else if( el.className.indexOf("gtdi-finish")>-1 ) {
                        var statusData = this._main.labelsData.getStatusData();
                        if( !this.thread.hasLabel(statusData.finished) ) {
                            this._addLabel( statusData.finished );
                            if( el.className.indexOf("gtdi-finish-next")>-1 ) {
                                this._moveNextRequest = this.thread; // Store the Thread ID so even if .load is called before archive callback reached, we can catch it.
                            } else if( this.thread.hasLabel("^i") ) {
                                this._archiveRequest = this.thread;
                            }
                            var serverActionType = (this._moveNextRequest && this._main.gmail.page.getActiveViewType()=='cv')? glActionManager.USE_AJAX : this._serverActionType;
                            if( this._main.gmail.page.getActiveViewType()!='cv' ) this._main.gmail.actions.performAction(glActionManager.MARK_AS_READ, this.thread.getRecentId(), serverActionType);
                            this._main.gmail.actions.performAction(glActionManager.ARCHIVE, this.thread.getRecentId(), serverActionType);	// TODO Review use of IDs. It seems archiving is very fussy about last ID, whereas normal labels are not.
                        }
                    */
                    } else if( el.className.indexOf("gtdi-labelling-expander")>-1 ) {	// Clicked expander - toggle
                        this._expanderToggle();
						break;
					} else if( el.className.indexOf("gtdi-label-status")>-1 ) {	// Clicked status label - toggle
						// Stop link being followed:
						giUtil.stopEvent(event);
						// Toggle:
						var labelname = el.getAttribute("gtdLabelName");
						if( giUIObject.hasClassName( el, "gtdi-active" ) ) {
							this._removeLabel(labelname);
						} else {
							this._addLabel(labelname);
						}
						break;
					}
					// Go up tree
					el = el.parentNode;
				}
			}
		} catch(e) {
			giLogger.warn(e, 'gtdLabellingBox._click', this._main.gmail);
		}
	},
	
    /**
     * Toggle the rows - show or hide (except statuses)
     * @function {} ?
     * @param {optional Boolean} on - If nothing, just toggle; otherwise if true, show everything (and false, hide)
     */
    /*
    _expanderToggle: function(on) {
        
        if( typeof on=="undefined" ) {
            // Get current show-status:
            
            if( giUIObject.hasClassName(this._elExpander, 'gtdi-labelling-expander-on') ) {
                on = false;
            } else if( giUIObject.hasClassName(this._elExpander, 'gtdi-labelling-expander-off') ) {
                on = true;
            } else {
                // Retrieve last choice from preferences:
                var on = this._main.prefs.getPref('labelling_box.expanded') || false;
            }
            
        }
        
        //
        if( this._popupTimeout ) {
            clearTimeout( this._popupTimeout );
            this._popupTimeout = null;
        }
        
        // Now set visibility:
        for( t in this._uiRows ) {
            if( t!='statuses' ) {
                this._uiRows[t].el.style.display = on? '' : 'none';
            }
        }
        
        // Update button status:
        if( on ) {
            giUIObject.removeClassName(this._elExpander, 'gtdi-labelling-expander-off');
            giUIObject.addClassName(this._elExpander, 'gtdi-labelling-expander-on');
            this._elExpander.title = giI18N.getString("Labelling.expander.opened");
            this._elExpander.innerHTML = '&minus;';
        } else {
            giUIObject.removeClassName(this._elExpander, 'gtdi-labelling-expander-on');
            giUIObject.addClassName(this._elExpander, 'gtdi-labelling-expander-off');
            this._elExpander.title = giI18N.getString("Labelling.expander.closed");
            this._elExpander.innerHTML = '+';
        }
        
        // Update for future default:
        this._main.prefs.setPref('labelling_box.expanded', on);
        
        // Reset hover listeners:
        giUIObject.removeClassName(this._elExpander, 'gtdi-hover');
        this._eventsContainer.stopObserving(this._elExpander, 'mouseover', this._expanderMouseOver, false);
        this._eventsContainer.observe(this._elExpander, 'mouseover', this._expanderMouseOver, false);   // Will not fire if it's turned on
        this._eventsContainer.stopObserving(this._main.gmail.canvasDocument.body, 'mousemove', this._expanderWindowMouseMove, false);
    },
    
    
    _expanderMouseOver: function(evt) {
        
        try {
            // Check not properly expanded:
            if( !giUIObject.hasClassName(this._elExpander, 'gtdi-labelling-expander-on') ) {
                                
                                
                if( this._popupTimeout ) {
                    clearTimeout( this._popupTimeout );
                    this._popupTimeout = null;
                }
                
                // Stop mouse over listener
                this._eventsContainer.stopObserving(this._elExpander, 'mouseover', this._expanderMouseOver, false);
                
                // Expand it
                for( t in this._uiRows ) {
                    if( t!='statuses' ) {
                        this._uiRows[t].el.style.display = '';
                    }
                }
                
                // Mark us as showing:
                giUIObject.addClassName(this._elExpander, 'gtdi-hover');
                
                // Listen for an exit:
                this._eventsContainer.observe(this._main.gmail.canvasDocument.body, 'mousemove', this._expanderWindowMouseMove, false);
            }
        } catch(e) {
            giLogger.warn(e, 'gtdLabellingBox._expanderMouseOver');
        }
    },
    
    _expanderWindowMouseMove: function(evt) {
        try {
            //if( this._popupTimeout ) return;
            
            // See if outside of labelling box:
            var el = evt.target;
            var i = 10;
            var found = false;
            while( --i>=0 && el.parentNode ) {
                if( el.className.indexOf('gtdi-labelling')>-1 ) {
                    found = true;
                    break;
                }
                el = el.parentNode;
            }
            if( found ) {
                // Keep it on:
                if( this._popupTimeout ) {
                    clearTimeout( this._popupTimeout );
                    this._popupTimeout = null;
                }
            } else {
                // Turn it off:
                if( !this._popupTimeout ) this._popupTimeout = setTimeout(giUtil.bind(this, this._hideTempExpander), 400);
            }
        } catch(e) {
            giLogger.warn(e, 'gtdLabellingBox._expanderWindowMouseMove');
        }
    },
    
    _hideTempExpander: function() {
        this._popupTimeout = null;
        giUIObject.removeClassName(this._elExpander, 'gtdi-hover');
        this._eventsContainer.stopObserving(this._main.gmail.canvasDocument.body, 'mousemove', this._expanderWindowMouseMove, false);
        this._eventsContainer.observe(this._elExpander, 'mouseover', this._expanderMouseOver, false);
        
        for( t in this._uiRows ) {
            if( t!='statuses' ) {
                this._uiRows[t].el.style.display = 'none';
            }
        }
    },
    */
    
    
	/**
	* Respond to a request to add a label (could originate from _click or a broadcast event from gtdLabellingDropdown)
	* If autoUpdate, update associated thread. Broadcast event to any listeners.
	* @function ?
	* @param {} event - An event object (or acceptably, a string) containing the label name.
	*/
	_addLabel: function(event) {
		// Get the label name
		var labelname = (typeof event=="string")? event : event.label;
		// Process it?
		
		if( this.thread && this._autoUpdate ) {
			this._main.gmail.actions.performAction( glActionManager.ADD_LABEL, this.thread, this._serverActionType, labelname );
		}
		
		// Set it as on - the thread should call this again in <1000ms when it confirms it as added. This is just for a fast user response (or if manual).
		this._setLabel(labelname, true);
				
		// Broadcast
		if( this.hasEventListener(gtdLabellingBox.SELECT_LABEL)) this.dispatchEvent({name: gtdLabellingBox.SELECT_LABEL, label:labelname});
	},
	
	/**
	* Respond to a request to remove a label (could originate from _click or a broadcast event from gtdLabellingDropdown)
	* If autoUpdate, update associated thread. Broadcast event to any listeners.
	* @function ?
	* @param {} event - An event object (or acceptably, a string) containing the label name.
	*/	
	_removeLabel: function(event) {
		// Get the label name
		var labelname = (typeof event=="string")? event : event.label;
		
		// Process it?
		if( this.thread && this._autoUpdate ) {
			this._main.gmail.actions.performAction( glActionManager.REMOVE_LABEL, this.thread, this._serverActionType, labelname );
		}
		
		// Set it as off - the thread should call this again in <1000ms when it confirms it as added. This is just for a fast user response (or if manual).
		this._setLabel(labelname, false);
				
		// Broadcast
		if( this.hasEventListener(gtdLabellingBox.UNSELECT_LABEL)) this.dispatchEvent({name: gtdLabellingBox.UNSELECT_LABEL, label:labelname});
	},
	
	/**
	* Respond to a thread having a label added/removed. Update the UI of the box to show the new label selected (and pass to dropdowns)
	* @function ?
	* @param {} event - Contains the labelname and the action {action: const, thread: id}
	*/
	_threadUpdated: function(event) {
        
        
        var statusData = this._main.labelsData.getStatusData();
        if( this.thread && this.thread.hasId(event.thread) ) {	// It affects us
            if( event.label && event.label==statusData.finished ) {
                this._testCmdButtons();
            }
            if( event.action==glActionManager.ADD_LABEL ) {		// Label added to thread
                this._setLabel( event.label, true );
            } else if( event.action==glActionManager.REMOVE_LABEL ) {	// Label removed from thread
                this._setLabel( event.label, false );
            } else if( event.action==glActionManager.ARCHIVE ) {
                this._testCmdButtons();
            }
        }
        
        // Now see if this is the follow up to an archive/archive+movenext request:
        /*
        if( (this._moveNextRequest || this._archiveRequest) && (event.environmentGenerated || event.serverActionType==glActionManager.USE_AJAX_SYNC || event.serverActionType==glActionManager.USE_AJAX) && event.action==glActionManager.ARCHIVE ) { // TODO Not sure why it never fires event.environmentGenerated. Assumed - but not tested - that it is because of the SYNC firing.
            this._testCmdButtons();
            if( this._moveNextRequest && this._moveNextRequest.hasId( event.thread ) ) {
                var threadId = this._moveNextRequest.threadId;
                // Reset the flag:
                this._moveNextRequest = null;
                this._archiveRequest = null;
                // Fire a move next as the Archive is complete
                this.dispatchEvent({name: gtdLabellingBox.USER_ARCHIVE_MOVENEXT_COMPLETE, threadId: threadId});
            } else if( this._archiveRequest && this._archiveRequest.hasId( event.thread ) ) {
                // Clicked the archive or finish button only
                var threadId = this._archiveRequest.threadId;
                this._moveNextRequest = null;
                this._archiveRequest = null;
                this.dispatchEvent({name: gtdLabellingBox.USER_ARCHIVE_COMPLETE, threadId: threadId});
            }
        }
        */
        
	},

	/**
	* Retrieve the container for the command buttons (Archive/Finish)
	* @function {Element} ?
	*/
	getCmdButtons: function() {
		return this._elCmdButtons;
	},
	
	/**
	* Set UI of test buttons based on environment
	* @function {} ?
	*/
	_testCmdButtons: function() {
		if( !this.thread ) {
            return;
        }
		if( !this._btnArchive ) {
            return;	// If no status labels, it will not include this part of the work flow.
        }
		var statusData = this._main.labelsData.getStatusData();
		

        
		var elArchiveInner = this._btnArchive.main.ownerDocument.evaluate(".//*[contains(@class,'gtdi-archive')]/SPAN[@class='gtdi-btn-label']/SPAN", this._btnArchive.main.parentNode, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        var elFinishInner = this._btnFinish.getElementsByTagName("SPAN")[0];
		
		if( this.thread.hasLabel("^i") ) {	// Not archived
			giUIObject.removeClassName(this._btnArchive.main, "gtdi-active");
			if( this._btnArchive.next ) giUIObject.removeClassName(this._btnArchive.next, "gtdi-active");
			this._btnArchive.main.style.color = "#090";
			this._btnArchive.main.style.fontWeight = "bold";
			elArchiveInner.innerHTML = giI18N.getString("Labelling.buttons.archive");
		} else {
			giUIObject.addClassName(this._btnArchive.main, "gtdi-active");
			if( this._btnArchive.next ) giUIObject.addClassName(this._btnArchive.next, "gtdi-active");
			this._btnArchive.main.style.color = "";
			this._btnArchive.main.style.fontWeight = "normal";
			elArchiveInner.innerHTML = giI18N.getString("Labelling.buttons.archived");
		}
		
		if( this.thread.hasLabel(statusData.finished) ) {
			//elFinishInner.style.color = "";
            //elFinishInner.innerHTML = giI18N.getString("Labelling.buttons.finished");
            this._btnFinish.title = "";
		} else {
            // TODO Make it green only if it has an action enabled
			//elFinishInner.style.color = "#090";
            //elFinishInner.innerHTML = giI18N.getString("Labelling.buttons.finish");
            this._btnFinish.title = giI18N.getString("Labelling.buttons.finished.tooltip");
		}
	},
    
    
    //}
},{
    // Static Properties {
    /**
     * Dispatched when a label is selected
     * @event ?
     */
    SELECT_LABEL: 'select_label',
    
    /**
     * Dispatched when a label is unselected
     * @event ?
     */
    UNSELECT_LABEL: 'unselect_label',
    
    
    /**
     * Move to the next thread
     * @event ?
     */
    //USER_ARCHIVE_MOVENEXT_COMPLETE: 'archive_move_next',
    
    /**
     * User clicks Archive/Finish (but not next)
     * @event ?
     */
    //USER_ARCHIVE_COMPLETE: 'archive_thread',
    
    /**
     * The user clicked Archive. May include the moveNext parameter.
     * @event ?
     */
    CLICK_ARCHIVE: 'click_archive',
    
    
    /**
     * Create a Labelling Box Cmd Button
     * @function {Object} ?
     * @param {Document} doc - The document to create the elements for
     * @param {String} id - The id used to identify button
     * @param {String} name - The name
     * @param {String} tooltip - The tooltip for the main button
     * @param {Boolean} incNext - If true, include next
     * @param {String} tooltipNext - The tooltip for the Next segment
     * @return - 3 elements, in format {container:, main:, next: }
     */
    createCmdButton: function(doc, id, name, tooltip, incNext, tooltipNext) {
        var elContainer = doc.createElement("DIV");
        elContainer.className = "gtdi-btn-grp";
        
        var elMain = doc.createElement("DIV");
        elMain.className = "gtdi-btn gtdi-" + id;
        elMain.innerHTML = "<b class='gtdi-btn-t'></b><b class='gtdi-btn-t2'></b><b class='gtdi-btn-m'></b><b class='gtdi-btn-b2'></b><b class='gtdi-btn-b'></b><span class='gtdi-btn-label' title='" + tooltip + "'><span class='' >" + name + "</span></span>";
        elContainer.appendChild( elMain );
        
        var elNext = null;
        if( incNext ) {
            elNext = doc.createElement("DIV");
            elNext.className = "gtdi-btn  gtdi-" + id + "-next";
            elNext.innerHTML = "<b class='gtdi-btn-t'></b><b class='gtdi-btn-t2'></b><b class='gtdi-btn-m'></b><b class='gtdi-btn-b2'></b><b class='gtdi-btn-b'></b><span class='gtdi-btn-label' title='" + tooltipNext + "'><span class='' ><img src='chrome://gtdinbox/skin/gtd/i/gtdi-next.png'/></span></span>";
            elContainer.appendChild( elNext );
        }
        
        return {container: elContainer, main: elMain, next: elNext};
    },

});
gtdLabellingBox.implement(giEventDispatcher); // Add event dispatching support