/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * A combo-box for a type of labels. Belongs to gtdLabellingBox.
 *
 * @class ? @extends giUIObject
 */
var gtdLabellingDropdown = giUIObject.extend({
    // Properties {
    
	/** @property {private gtdLabellingBox} ? The Labelling box that owns this dropdown */
	_labellingBox: null,
	
	/** @property {Element} ? The label container */
	el: null,
	
	/** @property {Element} ? The SPAN title element */
	_elTitle: null,
	
	/** @property {Element} ? The dropdown */
	_elDropdown: null,
	
	/** @property {Element} ? List of recent labels */
	_elRecent: null,
	
	/** @property {Element} ? Title of recent labels */
	_elRecentTitle: null,
	
    /** @property {Element} ? The 'Create Label' link */
	_elCreate: null,
    
	/** @property {string} ? The type of labels this contains (e.g. 'statuses') */
	_type: null,
	
	/** @property {gtdLabelDOM} ? Collection of gtdLabelDOM objects and container el for labels.  */
	_typeLabelDOM: null,
	
   	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,


    //}

	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdLabellingDropdown object to view', 'gtdLabellingDropdown');
		
		this.base(main);
		this._eventsContainer = new giEventsContainer(this);
		
			
		// Create the nodes:
		var doc = main.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.className = "gtdi-btn gtdi-dropdown";
		this.el.innerHTML = "<b class='gtdi-btn-t'></b><b class='gtdi-btn-t2'></b><b class='gtdi-btn-m'></b><b class='gtdi-btn-b2'></b><b class='gtdi-btn-b'></b><span class='gtdi-btn-label'></span>";
		this._elTitle = this.el.getElementsByTagName("SPAN")[0];
		
		
		// Create the dropdown nodes:
		this._elDropdown = doc.createElement("DIV");
		this._elDropdown.className = "gtdi-droppeddown";
		this._elDropdown.style.display = "none";
		//this.el.ownerDocument.body.appendChild( this._elDropdown );
        this.el.appendChild( this._elDropdown );
        
        // Listen for clicks:
        this._eventsContainer.observe( this.el, "click", this._click, false );
	},
	
	/**
	* Load the labelling box
	* @function ?
	* @param {string} type - The type of labels this contains (E.g. 'statuses')
	*/
	setType: function(type) {
		this._type = type;
	
	
		this.refreshTitle();
		
		
		this._elDropdown.innerHTML = "";
		
		var doc = this._main.gmail.canvasDocument;
        
		// Create recent:
		this._elRecentTitle = doc.createElement("DIV");
		this._elRecentTitle.className = 'gtdi-droppeddown-recent-title';
		this._elRecentTitle.innerHTML = giI18N.getString("Labelling.dropdown.recent");
		this._elDropdown.appendChild( this._elRecentTitle );
		
		this._elRecent = doc.createElement("DIV");
		this._elRecent.className = 'gtdi-droppeddown-recent';
		this._elDropdown.appendChild(this._elRecent);
		      
		
		// Create the labels box:
		this._typeLabelDOM = this._main.labelsData.getLabelsEl(this._type);
		this._elDropdown.appendChild( this._typeLabelDOM.el );
		
        // Create 'Create Label':
        this._elCreate = doc.createElement("DIV");
        this._elCreate.className = 'gtdi-droppeddown-create';
        this._elCreate.setAttribute("gtdLabelType", type);
        this._elCreate.innerHTML = "<img src='chrome://gtdinbox/skin/gtd/add.png'/><span>"+giI18N.getString('Labelling.dropdown.create')+"</span>";
        this._elDropdown.appendChild( this._elCreate );
        
        // Hide the dropdown:
        this._hideDropdown();
	},
    
    
    /**
     * Show the main menu
     * @function {} ?
     */
    _showDropdown: function() {
                
        // Render recent
        this._refreshRecent();
    
        // Close all parent nodes and their children
        this._typeLabelDOM.close();            
    
        // Open all selected:
        var labels = this._typeLabelDOM.getSelectedLabelNames();
        for( var i = 0; i < labels.length; i++ ) {
            this._typeLabelDOM.open( labels[i] );
        }
    
        // Decide whether to respond to offline mode:
        if( this._main.gmail.offline.isUnstable() ) {
            this._elCreate.style.display = "none";
        } else {
            this._elCreate.style.display = "";
        }
    
        // Show it and listen for events that will close it
        this._eventsContainer.observe( this.el.ownerDocument.body, "mousedown", this._mousedownWindow, true );
        this._elDropdown.style.display = "";
        
        // Set height:
        // Height: height of popup - offset of dropdown within the popup
        // Why this instead of a full popout? Because gtdLabellingBox relies on this being a child of the labelling box.
        // Only possible correction is to adjust CSS. Or add labelling box at a different level. 
        var popup = this._main.getUIObject('browserPopup');
        if( popup.isActive() ) {
            var dimsPopup = giUIObject.getDimensions(popup.el);
            var offsetInPopup = giUIObject.cumulativeOffset(this._elDropdown, popup.el);
            var padding = giUIObject.getPadding(this._elDropdown);
            height = dimsPopup.height - offsetInPopup[1] - parseInt(padding[0]) - parseInt(padding[2]);
            if( height > 400 ) height = 400;
            this._elDropdown.style.maxHeight = height + "px";
        } else {
            this._elDropdown.style.maxHeight = "";
        }
        
    },
    
    /**
     * Reload the recent menu
     * @function {} ?
     * @param {optional String[]} activeLabels - array of active labels on this thread
     */
    _refreshRecent: function(activeLabels) {
        if( !activeLabels ) activeLabels = this._typeLabelDOM.getSelectedLabelNames();
        
        var doc = this.el.ownerDocument;
        this._elRecent.innerHTML = "";
        var recent = this._main.recent.getLabels(this._type);
        if( recent.length>3 ) recent.length = 3; // Cap it to make it more readable
        recent.sort();  // Put into order - easier to read
        
        if( recent.length==0 ) {
            this._elRecentTitle.style.display = "none";
        } else {
            this._elRecentTitle.style.display = "";
            for( var i = 0; i < recent.length; i++ ) {
                var elRecent = doc.createElement("DIV");
                elRecent.className = "gtdi-droppeddown-recent-item";
                if( activeLabels.indexOf(recent[i][0])>-1 ) {
                    elRecent.className += " gtdi-active";
                }
                elRecent.innerHTML = recent[i][0];
                elRecent.setAttribute("gtdLabelName", recent[i][0]);
                this._elRecent.appendChild( elRecent );
            }
        }
        
    },
    
    /**
     * Hide the main menu
     * @function {} ?
     */
    _hideDropdown: function() {
        
        
		this._eventsContainer.stopObserving( this.el.ownerDocument.body, "mousedown", this._mousedownWindow, true );
        this._elDropdown.style.display = "none";
    },
	
    /**
     * Notification the element or a child was clicked; see if it requires an action
     * @function {} ?
     */
    _click: function(event) {
        try {
            var el = event.target;
            var i = 4;
            while( el.parentNode && --i>=0 ) {
                
                if( giUIObject.hasClassName(el, 'gtdi-dropdown') && this._elDropdown.style.display=="none" ) {
                    this._showDropdown();
                    return;
                }
                if( giUIObject.hasClassName(el, 'gtdi-labeldom-expander') ) {
                    giUtil.stopEvent(event);
                    return;
                }
                if( giUIObject.hasClassName(el, 'gtdi-droppeddown') ) { // Could be scroll bar
                    return;
                }
                
                
                el = el.parentNode;
            }
        } catch(e) {
            giLogger.warn(e, 'gtdLabellingDropdown._click');
        }
    },
    
    /**
     * See if move outside the dropdown, if so, hide it
     * @function {} ?
     */
	_mousedownWindow: function(event) {
        try {
            var found = false;
            var el = event.target;
            var i = 5;
            while( el.parentNode && --i>=0 ) {
                if( el==this._elDropdown ) {
                    found = true;
                    break;
                }
                el = el.parentNode;
            }
            if( !found ) {
                this._hideDropdown();
            }
        } catch(e) {
            giLogger.warn(e, 'gtdLabellingDropdown._mousedownWindow');
        }
    },
    
	
    /**
     * Refresh the title box with the correct title; and reflect #selected-labels
     * @function {} ?
     * @param {optional String[]} activeLabels - array of active labels on this thread
     */
	refreshTitle: function(activeLabels) {
		// Get a count of active labels
        var activeLabelCount = 0;
        if( this._typeLabelDOM ) {
            if( !activeLabels ) activeLabels = this._typeLabelDOM.getSelectedLabelNames();
            activeLabelCount = activeLabels.length;
        }
		
		// Update the title element
		this._elTitle.innerHTML = giI18N.getString("Labelling.dropdown." + this._type + ".title", giI18N.getString("Labels.Types." + this._type, this._type));
		if( activeLabelCount > 0 ) {
            this._elTitle.innerHTML += " (" + activeLabelCount + ")";
            giUIObject.addClassName(this.el, 'gtdi-dropdown-hasSelections');
        } else {
            giUIObject.removeClassName(this.el, 'gtdi-dropdown-hasSelections');
        }
	},
	
	
	
	/**
	* Visually set a label as selected or unselected 
	* @function ?
	* @param {string} labelname - The name of the label to update
	* @param {boolean} isOn - If true, mark selected. Otherwise, mark unselected.
	*/
	setLabel: function(labelname, isOn) {
		this._typeLabelDOM.selected(labelname, isOn);
        
        var activeLabels = this._typeLabelDOM.getSelectedLabelNames();
        this.refreshTitle(activeLabels);
        
        // See if in recent:
        if( this._elRecent.childNodes.length > 0 ) {
            
            // Previously populated elRecent (i.e. shown dropdown), so check this change is registered
            var elRecent = this._elRecent.ownerDocument.evaluate( ".//DIV[@gtdLabelName='"+labelname+"']", this._elRecent, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
            if( elRecent ) {
                if( isOn ) {
                    giUIObject.addClassName( elRecent, 'gtdi-active' );
                } else {
                    giUIObject.removeClassName( elRecent, 'gtdi-active' );
                }
            } else {
                if( isOn ) {
                    // Not in recent, but was added; so refresh
                    this._refreshRecent(activeLabels);
                }
            }
        }
	},
	

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdLabellingDropdown from view', 'gtdLabellingDropdown');

        if( this._popupTimeout ) {
            // Stop the close
            clearTimeout(this._popupTimeout);
            this._popupTimeout = null;
        }
        
        if( this._typeLabelDOM ) this._typeLabelDOM.disconnect();
        
        this._main = null;
        this.thread = null;
        if( this.el && this.el.parentNode ) this.el.parentNode.removeChild( this.el );
        if( this._elDropdown && this._elDropdown.parentNode ) this._elDropdown.parentNode.removeChild(this._elDropdown);
        
        this.el = null;
        this._elTitle = null;
        this._elDropdown = null;
        this._eventsContainer.reset();
        this._eventsContainer = null;
        this.base();
    },
    
    //}
},{
    // Static Properties {
    //}
});
gtdLabellingDropdown.implement(giEventDispatcher); // Add event dispatching support