/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Modify Labels box to break into GTD sections. 
 * Also store hierarchial label data. 
 *

 * @class ? @extends giUIObject
 */
var gtdLabelsBox = giUIObject.extend({
    // Properties {

    /** @property {private gtdShowHider[]} ? Array of ShowHiders in use  */
    _showHiders: null,

    /** @property {private glLabelsBox} ? The glLabelsBox  */
    _glLabelsBox: null,

    

    /** @property {Object} typeBuckets - Collection of LabelData (gtdLabelDOM) objects, grouped by type (e.g. Statuses). */
    typeBuckets: null,
    
    /** @property {Object} typeContainerEls - Collection of Table objects for each type */
    typeContainerEls: null,
    
    
    /** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
    _eventsContainer: null,
    
    /** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
    _eventsContainer2: null,
    
    //}
    

    // Methods {


    /**
    * @constructor ?
    * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
    */
    connect: function(main) {
        giLogger.log('Constructing gtdLabelsBox', 'gtdLabelsBox');
        this.base(main);
        this.typeBuckets = {};
        this.typeContainerEls = {};
        this._showHiders = [];
        this._glLabelsBox = main.gmail.getUIObject('labelsBox');
        this._glLabelsBox.addEventListener(glLabelsBox.LABELS_BOX_MODIFIED, this._arrangeLabelsBox, this);
                
        this._eventsContainer = new giEventsContainer(this);
        this._eventsContainer2 = new giEventsContainer(this);
        this._arrangeLabelsBox();
        giLogger.log('Constructed gtdLabelsBox', 'gtdLabelsBox');
    },
    
    /**
    * Clean up
    * @function ?
    */
    disconnect: function() {
        this._main = null;
        this._eventsContainer.reset();
        this._eventsContainer2.reset();
        for( var i = 0; i < this._showHiders.length; i++ ) this._showHiders[i].reset();
        this.base();
    },
    
    /**
    * Break the Labels box into GTD sections and extract hierarchial label data.
    * @function ?
    */
    _arrangeLabelsBox: function() {
        
        try {
            if( !this._glLabelsBox.getLabelsContainer() ) return;
            
            var doc = this._main.gmail.canvasDocument;
            var typeSettings = this._main.prefs.getPref('labels.types');
            
            giLogger.log('Start arrangeLabelsBox', 'gtdLabelsBox');
            var gtdArranged = this._glLabelsBox.getLabelsContainer().getAttribute("gtdArranged");
            if( gtdArranged ) {	// Already processed or processing
                if( gtdArranged=="1" ) { // processed
                    for( t in this.typeBuckets ) {
                        for( l in this.typeBuckets[t].labelHash ) {
                            var elLeaf = this.typeBuckets[t].getLeaf(l);
                            if( elLeaf.innerHTML.indexOf( typeSettings[t].prefix )==0 ) {
                                elLeaf.innerHTML = elLeaf.innerHTML.replace(/.+?(\s(\(|\[)\d+(\)|\]))?$/, elLeaf.getAttribute("gtdi-leaf") + "$1");
                            }
                        }
                    }
                }
                return;
            }
            this._glLabelsBox.getLabelsContainer().setAttribute("gtdArranged", "-1");	// Indicate processing
            
            
            giLogger.log('Process arrangeLabelsBox', 'gtdLabelsBox');
            
            this._eventsContainer2.reset();
    
                                            
            // Destroy old variables:
            var oldLabelsHash = this._main.labelsData? this._main.labelsData.getLabelsHash(null, true) : {};
            var labelsChange = {'id':(new Date()).valueOf(), 'changed':false};
    
            for( var i = 0; i < this._showHiders.length; i++ ) {
                this._showHiders[i].reset();
            }
            this._showHiders.length = 0;
            
            if( this.typeContainerEls ) {
                for( t in this.typeContainerEls ) {
                    delete this.typeContainerEls[t];
                }
                delete this.typeContainerEls;
            }
            
            if( this.typeBuckets ) {
                for( t in this.typeBuckets ) {
                    this.typeBuckets[t].disconnect();
                }
                delete this.typeBuckets;
            }
            this.typeBuckets = {};
    
    
    
            // Grab the actual original table (contains each label as row)
            this.elTable = this._glLabelsBox.getLabelsTable();
            
            // Create New Structure for Labels Box:
            this.typeContainerEls = {};
            for( t in typeSettings ) {
                this.typeContainerEls[t] = {'elTable':doc.createElement("TABLE"), 'elTBody':doc.createElement("TBODY")};
                    giUIObject.addClassName( this.typeContainerEls[t].elTable, "cf" );
                    giUIObject.addClassName( this.typeContainerEls[t].elTable, "pQ" );
                    this.typeContainerEls[t].elTable.appendChild( this.typeContainerEls[t].elTBody );
                    this.typeContainerEls[t].elTBody.className = "showhider-tbody";
                var sh = new gtdShowHider( this._main, "labelBox-" + t, {doc:doc, elParent:this._glLabelsBox.getLabelsContainer(), elBefore:this.elTable, elContent:this.typeContainerEls[t].elTable, noContentIndent:true, title:giI18N.getString("Labels.Types." + t, t)} );
                    sh.elTitle.style.color = "black";
                    sh.elTitle.style.fontWeight = "bold";
                    sh.labelType = t;
                    //sh.addEventListener( this._glLabelsBox.TOGGLE, this._toggleShowHider, this );
                this._showHiders.push( sh );
            }
            
            
            // Extract labels
            var elRows = this._glLabelsBox.getLabelItems();
            
            
            var fCreatePlaceHolderEl = function(labelname, leafname) {
                var el = doc.createElement("TR");
                el.className = "WnjAod";
                var el2 = doc.createElement("TD");
                el2.innerHTML = "<div style='overflow: hidden; white-space: nowrap;'><div class='p5' gtdi-container='1'><span gtdi-leaf=''>"+(leafname || "/")+"</span></div></div>";
                el2.firstChild.firstChild.firstChild.setAttribute("gtdi-leaf", leafname);
                var el3 = doc.createElement("TD");
                el3.className = "p7";
                el.appendChild( el2 );
                el.appendChild( el3 );
                return el;
            };
            for( var i = 0; i < elRows.snapshotLength; i++ ) {
                
                var elLeaf = elRows.snapshotItem(i);	// A element
                var elRow = elLeaf.parentNode;	// Will be TR element
                while( elRow.parentNode && elRow.nodeName!="TR" ) {
                    elRow = elRow.parentNode;
                }
                
                
                // Get the label name (- the number)
                var labelname = giUtil.decodeEntities(elLeaf.innerHTML.replace(/\s\(\d+\)$/, ""), doc);
                
                
                // Find the type:
                var foundType = "misc";
                for( t in typeSettings ) {
                    if( t!="misc" && gtdLabelsData.testPrefix(labelname, typeSettings[t].prefix) ) {
                        foundType = t;
                        break;
                    }
                }
                
                
                
                // Get the leaf name:
                var hierarchy = labelname.replace(typeSettings[t].prefix, "").split("/");
                var leafname = elLeaf.innerHTML.replace(/.+?(\s\(\d+\))?$/, hierarchy[hierarchy.length-1] + "$1");
                
                // Create a gtdLabelDOM container if needed; use the newly created TBODY to house it
                if( !this.typeBuckets[foundType] ) {
                    this.typeBuckets[foundType] = this._main.getUIObject('labelsTree');
                    this.typeBuckets[foundType].init(foundType, this.typeContainerEls[t].elTBody);
                }
                
                this.typeBuckets[foundType].addLabel( elRow, elLeaf.parentNode, elLeaf, labelname, leafname, fCreatePlaceHolderEl );
                
                // Add Popout Icon
                /*
                if( !this.typeBuckets[foundType].isPlaceHolder(labelname) ) {
                    var elPopout = doc.createElement("DIV");
                    elPopout.innerHTML = "<img src='chrome://gtdinbox/skin/gtd/filter-extend.png' class='gtdi-labeldom-popouticon'/>";
                    elPopout.setAttribute("style", "position:absolute;width:9px;right:0px;top:0px;");	// TODO move to CSS
                    elPopout.title = giI18N.getString("Labels.Popout.Title");
                    
                    elRow.getElementsByTagName("DIV")[0].style.position = "relative";
                    elRow.getElementsByTagName("DIV")[0].firstChild.style.maxWidth = "124px";	// Second level DIV after TD. 136-9. 
                    elRow.getElementsByTagName("DIV")[0].appendChild( elPopout );	// Add to first DIV after TD
                }
                */
                    
            
                    
            }
            // Finalise label addition:
            for( t in this.typeBuckets ) {
                this.typeBuckets[t].finishAddingLabels();
            }
            
            // Rearrange Statuses
            if( this.typeBuckets.statuses ) {
                var defaultStatuses = [];
                for( d in typeSettings.statuses.core ) {
                    defaultStatuses.push( typeSettings.statuses.prefix + typeSettings.statuses.core[d] )
                }
                defaultStatuses.reverse();
                for( var i = 0; i < defaultStatuses.length; i++ ) {
                    if( this.typeBuckets.statuses.labelHash[defaultStatuses[i]] ) {
                        this.typeContainerEls.statuses.elTBody.insertBefore( this.typeBuckets.statuses.labelHash[defaultStatuses[i]], this.typeContainerEls.statuses.elTBody.firstChild );
                    }
                }		
            }
            
            
            // See if need to hide any empty boxes
            for( var i = 0; i < this._showHiders.length; i++ ) {
                var sh = this._showHiders[i];
                if( this.typeContainerEls[sh.labelType].elTBody.innerHTML.length==0 ) {	// Nothing doing
                    sh.visible(false);
                } else {
                    sh.visible(true);
                }
            }
            
            
            
            this._eventsContainer2.observe( this._glLabelsBox.getLabelsContainer(), "mousedown", this.labelsSectionMousedown, true);
            
            
            
            //this._maininboxPage._labelsBoxCount.refresh(true); TODO restore this
            
            
            if( oldLabelsHash ) {
                for( t in this.typeBuckets ) {
                    for( l in this.typeBuckets[t].labelHash ) {
                        if( oldLabelsHash[l] ) {
                            delete oldLabelsHash[l];
                        } else {
                            labelsChange.added = true;
                        }
                    }
                }
        
                if( giUtil.hashLength(oldLabelsHash) > 0 ) {	// Something deleted or renamed
                    labelsChange.removed = true;
                }
            }
            
            if( labelsChange.added && this.hasEventListener(gtdLabelsBox.LABEL_ADDED)) this.dispatchEvent({name: gtdLabelsBox.LABEL_ADDED});
            if( labelsChange.removed && this.hasEventListener(gtdLabelsBox.LABEL_REMOVED)) this.dispatchEvent({name: gtdLabelsBox.LABEL_REMOVED});
                            
            this._glLabelsBox.getLabelsContainer().setAttribute("gtdArranged", "1");	// Indicate processed
            
            giLogger.log('Completed arrangeLabelsBox', 'gtdLabelsBox');
        } catch(e) {
            giLogger.error(e, 'gtdLabelsBox.arrangeLabelsBox');
        }
                            
    },
    
    /**
    * Respond to a click event on the Labels box.
    * @function ?
    */
    labelsSectionMousedown: function(event, typeBucket) {
            
        giLogger.log('Clicked labelsSectionMousedown', 'gtdLabelsBox');
        
        try {
            if( event && event.button!=0 ) return;
    
            var elClicked = event.target;

            if( elClicked.className=="gtdi-labeldom-popouticon" ) {
                var elInspected = elClicked;
                while( elInspected.parentNode ) {
                    elInspected = elInspected.parentNode;
                    if(( elInspected.className == 'WnjAod') || giUIObject.hasClassName(elInspected, 'gtdi-labeldom-placeholder') || giUIObject.hasClassName(elInspected, "gtdi-labeldom-label") ) {
                        var labelName = elInspected.getAttribute("gtdLabelName");
                        
                        var popup = this._main.getUIObject('browserPopup');
                        popup.load(gtdBrowser.createBrowser(this._main, labelName));
        
                        break;
                    }
                }
            }
        } catch(e) {
                giLogger.warn(e, 'gtdLabelsBox.labelsSectionMousedown', this._main.gmail);
        }
    },
    
    
    
},{
    // Static Properties {

    /**
     * Dispatched when new label detected (will not fire on first run)
     * @event ?
     */
    LABEL_ADDED: 'label_added',
    
    /**
     * Dispatched when label detected removed (will not fire on first run)
     * @event ?
     */
    LABEL_REMOVED: 'label_removed',
    //}
    
});

gtdLabelsBox.implement(giEventDispatcher); // Add event dispatching support