/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Labels info for the Settings page
 *
 * @class ? @extends giUIObject
 */


var gtdLabelsInfo = giUIObject.extend({
    // Properties {
    
    
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
    
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
	
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdSetup object to view', 'gtdSetup');
		
		this.base(main);
		this._eventsContainer = new giEventsContainer(this);
        
        var doc = main.gmail.canvasDocument;
		this.el = doc.createElement('DIV');
        this.el.className = 'gtdi';
        
        var elContainer = doc.createElement("DIV");
        this.el.appendChild( elContainer );
        elContainer.id = 'gtdi-gtd-labelsinfo-root';
        elContainer.innerHTML = giFileIO.readFromUrl("chrome://gtdinbox/locale/gtdLabels.htm");
        
        // Attach to view
        var glSettingsView = this._main.gmail.getCurrentViewObject();
        var tabData = glSettingsView.getTab(glSettingsView.LABELS_TAB);
        tabData.elContent.appendChild( this.el );
        
        // Add categories:
        var elCategories = doc.getElementById('gtdi-gtd-labelsinfo-categories');
        var typeSettings = this._main.prefs.getPref('labels.types');
        for( t in typeSettings ) {
            
            // Create and attach container:
            var elC = doc.createElement("DIV");
            elC.className = 'gtdi-gtd-labelsinfo-categories-container';
            elCategories.appendChild( elC );
            
            // Create the name
            var elName = doc.createElement("P");
            elName.className = 'gtdi-gtd-labelsinfo-categories-name';
            elName.innerHTML = giI18N.getString('Labels.Types.'+t);
            elC.appendChild( elName );
            
            // Create and attach prefix:
            var elPrefix = doc.createElement("P");
            elPrefix.className = 'gtdi-gtd-labelsinfo-categories-prefix';
            elPrefix.innerHTML = giI18N.getString('LabelsInfo.categories.prefix') + ' ' + typeSettings[t].prefix;
            elC.appendChild( elPrefix );
            
            // Create and attach description, if known:
            var info = giI18N.getString('LabelsInfo.desc.'+t);
            if( info ) {
                var elInfo = doc.createElement("P");
                elInfo.className = 'gtdi-gtd-labelsinfo-categories-desc';
                elInfo.innerHTML = info;
                elC.appendChild( elInfo )
            }
            
            // If statuses, see if we need Setup button
            if( t=='statuses' ) {
                var sd = this._main.labelsData.getStatusData();
                var statuses = this._main.labelsData.getLabelsHash('statuses');
                if( !(statuses[sd.action] && statuses[sd['waiting_on']] && statuses[sd['waiting_on']] && statuses[sd['some_day']] && statuses[sd.finished]) ) {
                    // Create setup button:
                    var elButton = doc.createElement('DIV');
                    elButton.id = 'gtdi-gtd-labelsinfo-setup';
                    elButton.innerHTML = giI18N.getString('LabelsInfo.install');
                    elC.appendChild( elButton );
                }
            }
        }
        
        
        // Add portion of setup ("Understanding...")
        var elSetup = doc.createElement("DIV");
        elSetup.innerHTML = giFileIO.readFromUrl("chrome://gtdinbox/locale/gtdsetup.htm");
        elSetup.style.display = "none";
        elContainer.appendChild( elSetup );
        var elUnderstand = doc.getElementById('gtdi-gtdsetup-understanding');
        elContainer.appendChild( elUnderstand );
        elContainer.removeChild( elSetup );
        
        
        
        // Attach listener
        this._eventsContainer.observe(this.el, 'click', this._click, false);
		
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		giLogger.log('Disconnecting gtdSetup from view', 'gtdSetup');
		
        
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ) {
			if( this[p] && this[p].nodeName && this[p].nodeType ) {
				this[p] = null;
			}
		}
		
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		
		this.base();
	},
	
	
    /**
     * Notification of click on element
     * @function {}
     */
    _click: function(evt) {
        try {
            var el = evt.target;
            if( el.id=='gtdi-gtd-labelsinfo-setup' ) {
                // Clicked setup:
                this._main.getUIObject('setup');
            }
            if( el.id=='gtdi-gtd-labelsinfo-preferences' ) {
                // Clicked options
                giUtil.stopEvent(evt);
                window.open('chrome://gtdinbox/content/overlay_options.xul');
            }
        } catch(e) {
            giLogger.warn(e, 'gtdLabelsInfo._click');
        }
    },
	
});