/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Loads all the GTD widgets that can be activated in the general view based on
 * user preferences, such as {@link gtdSearchFiltersDropdown}.
 *
 * @class ? @extends giUIObject
 */
var gtdMainView = giUIObject.extend({
    // Properties {
    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
    	try {
	        giLogger.log('Connecting gtdMainView object to view', 'gtdMainView');
	
	        this.base(main);
	
	        if(main.prefs.getPref('components.compose.compose_personal.enabled')) {
	        	main.getUIObject('composePersonal');
	        }
	
	        if(main.prefs.getPref('components.search.search_button.enabled')) {
	        	main.getUIObject('searchButton');
	        }
	
            main.getUIObject('navBar');
	
	
	        giLogger.log('Connected gtdMainView object to view', 'gtdMainView');
        } catch(e) {
            giLogger.error('Error constructing gtdMainView\n' + e.toString(), 'gtdMainView');
        }
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdMainView from view', 'gtdMainView');

        this.base();
    },
    
    /**
     * Whether the UI object can be cached
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return true;
    },
    
    //}
});