/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Tweak the Nav bar
 *
 * @class ? @extends giUIObject
 */
var gtdNavBar = giUIObject.extend({
    // Properties {
    /** @property {glNavBar} ? The abstraction layer for the navigation menu */
    _glNavBar: null,
	
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
	
	/** @property {Element} ? The dashboard button */
	_elDashboard: null,
	
	/** @property {Element} ? The count element within the dashboard button */
	_elDashboardCount: null,

    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting gtdNavBar object to view', 'glNavBar');

        this.base(main);
        this._eventsContainer = new giEventsContainer(this);

		var doc = main.gmail.canvasDocument;
		this._glNavBar = main.gmail.getUIObject("navBar");
		
		// Add the Dashboard:
		
		/*
        if(main.prefs.getPref('components.navigation.dashboard.enabled')) {
			var elSpam = this._glNavBar.getNavItem(glNavBar.SPAM);
			
			if( elSpam.nodeName=="TR" ) {
				// New type of Gmail
				this._elDashboard = doc.createElement("TR");
				this._elDashboard.className = 'Alfa2e';
				var elTD = doc.createElement("TD");
				elTD.className = "nL";
				this._elDashboard.appendChild( elTD );
				var elTD = doc.createElement("TD");
				elTD.className = "n2 n1";
				this._elDashboard.appendChild( elTD );
				elTD.innerHTML = "<div class='n2'><div class='nU'><a class='n0' gtdi-navlink='1' target='_top' href='#'>" + giI18N.getString("Sidebar.Overview") + "</a></div></div>";
			} else {
				// Old type of Gmail
				this._elDashboard = doc.createElement("DIV");
				this._elDashboard.className = 'qi pX';
				this._elDashboard.innerHTML = "<div class='nH'><div class='nH'><div style='height: 4px;' class='pi'><div style='width: 4px;' class='ph p'></div><div style='width: 4px;' class='ph q'></div><div class='l o'></div></div><div class='l m'><div style='margin-left: 4px;' class='l n'><div style='margin: -2px 4px -2px 0px; padding: 0px;' class='k'><div class='diLZtc'><div class='nH'><div class='qk'><span idlink='' class='' id=':q7'><div style='overflow: hidden; white-space: nowrap;'><div idlink='' class='p5' id=':qo'><a gtdi-navlink='1' class='qh' target='_top' href='#'>" + giI18N.getString("Sidebar.Overview") + "</a></div></div></span></div></div></div></div></div></div><div class='dJ'></div><div style='height: 4px;' class='pi'><div style='width: 4px;' class='ph i'></div><div style='width: 4px;' class='ph j'></div><div class='l h'></div></div></div></div>";
			}
			
			if( elSpam.nextSibling && elSpam.nextSibling.nextSibling ) {
				elSpam.parentNode.insertBefore( this._elDashboard, elSpam.nextSibling.nextSibling );
			} else {
				elSpam.parentNode.appendChild( this._elDashboard );
			}
			
			this._elDashboardCount = doc.evaluate('.//a[@gtdi-navlink]', elSpam.parentNode, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
			
    		this._eventsContainer.observe( this._elDashboard, "mousedown", this._clickDashboard, false );
			this._refreshDashboardCount();
			this._main.statusThreadsContainer.addEventListener(gtdThreadContainer.LOADED, this._refreshDashboardCount, this);
			this._main.statusThreadsContainer.addEventListener(gtdThreadContainer.LOADING_UPDATE, this._refreshDashboardCount, this);
			this._main.statusThreadsContainer.addEventListener(gtdThreadContainer.CHANGED, this._refreshDashboardCount, this);
        }
        */
		
        
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdNavBar from view', 'glNavBar');
        this._glNavBar = null;
		
        this._eventsContainer.reset();
        
        if( this._elDashboard && this._elDashboard.parentNode ) this._elDashboard.parentNode.removeChild( this._elDashboard );
        this._elDashboard = null;

        this.base();
    },

	
	_refreshDashboardCount: function() {
		if( this._main.statusThreadsContainer.threads.length==0 ) {
			this._elDashboardCount.innerHTML = giI18N.getString("Sidebar.Overview");
			this._elDashboard.style.fontWeight = 'normal';
		} else {
			this._elDashboardCount.innerHTML = giI18N.getString("Sidebar.Overview") + " (" + this._main.statusThreadsContainer.threads.length + ")";
			this._elDashboard.style.fontWeight = 'bold';
		}
	},
    
    /**
     * Whether the UI object can be cached
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return true;
    },
    
    //}
});
