/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * A button next to the standard Gmail search button that loads the async results
 *
 * @class ? @extends giUIObject
 */
var gtdSearchButton = giUIObject.extend({
    // Properties {
    /**
     * The search button container
     * @property {Element} ?
     */
    el: null,
    
	
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting gtdSearchButton object to view', 'gtdSearchButton');

        this.base(main);
        this._eventsContainer = new giEventsContainer(this);

        var searchBox = main.gmail.getUIObject('searchBox');

        // Build Node {
        this.el = giUIObject.createGmailButton("gmail-header-button", giI18N.getString("Search.Button"), null, searchBox.searchBox.ownerDocument);
        //}

        this._eventsContainer.observe(this.el, "click", this._click, false);
		this._eventsContainer.observe(this.el, "keydown", this._key, false);

        // Insert into document {
        searchBox.searchMailButton.parentNode.insertBefore(this.el, searchBox.searchMailButton.nextSibling);
        //}
    },

	/**
	* Respond to a key press. If enter/space, go to click functionality
	* @function ?
	*/
	_key: function(event) {
		if( event.keyCode==13 || event.keyCode==32 ) {
			this._click(event);
		}
	},

	/** Respond to a search button click -> extract current query and open in async popup
	* @function ?
	*/
	_click: function(event) {
		try {
			var searchBox = this._main.gmail.getUIObject('searchBox');
					
            var popup = this._main.getUIObject('browserPopup');
            popup.load(gtdBrowser.createBrowser(this._main, searchBox.searchBox.value)); // Let it figure out the text itself. A label, a view, a contact or the overview.
            
            
		} catch(e) {
			giLogger.warn(e, 'gtdSearchButton._click', this._main.gmail);
		}
	},

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdSearchFiltersDropdown from view', 'gtdSearchFiltersDropdown');

        if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
        this.el = null;
        this._eventsContainer.reset();
	
        this.base();
    },
    
    /**
     * Whether the UI object can be cached
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return true;
    },
    
    //}
});