/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * The settings view abstraction layer.
 *
 * @class ? @extends giUIObject
 */
var gtdSettingsView = giUIObject.extend({
    // Properties {
	
    
    /**
     * @property {gtdLabelsInfo}
     */
    _labelsInfo: null,
    
    //}

    // Methods {
    /**
     * Connects the element to the given glGmail object's page. Called automatically
     * by {@link glGmail.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
        giLogger.log('Connecting gtdSettingsView object to view', 'gtdSettingsView');

        this.base(main);
        
        var settingsView = this._main.gmail.getCurrentViewObject();
        settingsView.addEventListener(glSettingsView.SELECT_TAB, this._selectTab, this);
        
        // Get the default tab:
        var tabData = settingsView.getTab();
        this._selectTab(tabData);
        
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        giLogger.log('Disconnecting gtdSettingsView from view', 'gtdSettingsView');
        
        if( this._labelsInfo ) {
            this._labelsInfo.disconnect();
            this._labelsInfo = null;
        }
        
        this.base();
    },
    
    /**
     * Notification that a tab was selected. Fired by glSettingsView
     * @function {} ?
     */
    _selectTab: function(evt) {
        
        switch( evt.tabName ) {
            case glSettingsView.LABELS_TAB:
                // Ensure we have integrated labels view
                if( !this._labelsInfo ) {
                    this._labelsInfo = this._main.getUIObject('labelsInfo');
                }
                break;
        }
        
    },
    

    //}
});
