/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * The setup routine for GTD
 *
 * @class ? @extends giUIObject
 */


var gtdSetup = giUIObject.extend({
    // Properties {
    
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
	
	/** @property {private Element} - The el that contains the threads-viewer and thread-viewer */
	_elContainer: null,
	
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdSetup object to view', 'gtdSetup');
		
		this.base(main);
		this._eventsContainer = new giEventsContainer(this);
		
		// Create container
		var doc = main.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.className = "gtdi gtdi-popup gtdi-gtdsetup";
		this.el.style.position = "absolute";
		
		this._elContainer = doc.createElement("DIV");
		this._elContainer.id = "gtdi-browser-container";
		this.el.appendChild( this._elContainer );
		
		// Add the close:
		var elClose = doc.createElement("A");
		elClose.className = "gtdi-close";
		elClose.title = giI18N.getString("Browser.Popup.Close");
		elClose.innerHTML = giI18N.getString("Browser.Popup.Close");
		this._elContainer.appendChild( elClose );
		
        
        
		// Attach to view:
		this._main.gmail.canvasDocument.body.appendChild(this.el);
        
        // Add content
        var elContent = doc.createElement("DIV");
        elContent.className = 'gtdi-gtdsetup-container';
        elContent.innerHTML = giFileIO.readFromUrl("chrome://gtdinbox/locale/gtdsetup.htm");
        this._elContainer.appendChild( elContent );
        
        var elChecks = doc.getElementById('gtdi-gtdsetup-labels');
        var sd = this._main.labelsData.getStatusData();
        var statuses = this._main.labelsData.getLabelsHash('statuses');
        if( !statuses[sd['action']] ) this._createCheckbox(elChecks, sd['action']);
        if( !statuses[sd['waiting_on']] ) this._createCheckbox(elChecks, sd['waiting_on']);
        if( !statuses[sd['some_day']] ) this._createCheckbox(elChecks, sd['some_day']);
        if( !statuses[sd['finished']] ) this._createCheckbox(elChecks, sd['finished']);
        
		
		// Add Event
		this._eventsContainer.observe(this._elContainer, "click", this._click, false);
        
        // Ensure new label creation is caught:
        this._main.gmail.environment.addEventListener(glEnvironment.CREATE_LABEL, this._createLabelEvent, this);
        
        // Centre popup:
        var popupDims = giUIObject.getDimensions(this.el);
        var viewport = giUIObject.viewport(this.el.ownerDocument);
                        
        x = (viewport[0]/2)-(popupDims.width/2);
        y = (viewport[1]/2)-(popupDims.height/2) + this.el.ownerDocument.defaultView.pageYOffset;
        
        var xy = giUIObject.boundsCheckXY(this._elContainer, x,y,true,true);
        x = xy[0];
        y = xy[1];
        this.el.style.left = x + 'px';
        this.el.style.top = y + 'px';
    
        this._main.gmail.lightbox.show();
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		giLogger.log('Disconnecting gtdSetup from view', 'gtdSetup');
		
        this._main.gmail.lightbox.hide();
        
        this._main.gmail.environment.removeEventListener(glEnvironment.CREATE_LABEL, this._createLabelEvent, this);
        
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ) {
			if( this[p] && this[p].nodeName && this[p].nodeType ) {
				this[p] = null;
			}
		}
		
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		
		this.base();
	},
	
	
    /**
     * @function {Element} ?
     * @param {Element} elContainer - The container to attach it to
     * @param {String} label - The label name
     * @return Return the checkbox
     */
    _createCheckbox: function(elContainer, label) {
        var doc = elContainer.ownerDocument;
        var elCheckContainer = doc.createElement('DIV');
        var elCheck = doc.createElement('INPUT');
        elCheck.setAttribute('type', 'checkbox');
        elCheck.setAttribute('gtdLabelName', label);
        elCheck.checked = true;
        elCheckContainer.appendChild( elCheck );
        elCheckContainer.appendChild( doc.createTextNode(label) );
        elContainer.appendChild( elCheckContainer );
        
        return elCheckContainer;
    },
	
	
	/**
	* Notification that the element was clicked
	* Hide the popup
	* @function ?
	*/
	_click: function(evt) {
		try {
            var el = evt.target;
            
            if( el.id=='gtdi-gtdsetup-install' ) {
                // Find which items are checked
                var els = this.el.ownerDocument.getElementById('gtdi-gtdsetup-labels').getElementsByTagName('INPUT');
                this._labels = {};
                var foundOne = false;
                for( var i = 0; i < els.length; i++ ) {
                    if( els[i].checked ) {
                        foundOne = true;
                        this._labels[ els[i].getAttribute('gtdLabelName') ] = false;
                    }
                }
                if( !foundOne ) return;
                
                
                // Set the spinner
                el.innerHTML += "&nbsp;<img src='chrome://gtdinbox/skin/indicator_small.gif'/>";
                el.disabled = true;
                
                // Now create the labels; it will call _createLabelEvent on success
                for( l in this._labels ) {
                    this._main.gmail.actions.performAction(glActionManager.CREATE_LABEL, null, glActionManager.USE_AJAX, l);
                }
            }
            
            if( el.id=='gtdi-gtdsetup-hide' ) {
                // Disable it and close
                this._main.prefs.setPref('components.navigation.gtd_setup.disabled', true);
                this._main.getUIObject('boxSetup').disconnect();
                this.disconnect();
            }
            
            if( el.className=='gtdi-close' ) {
                this.disconnect();
            }
		} catch(e) {
			giLogger.warn(e, 'gtdSetup._clickClose', this._main.gmail);
		}
	},	
	
    /**
     * Notification a label was created, if one of ours, check it off.
     * Once all complete, close.
     * @function {} ?
     */
	_createLabelEvent: function(evt) {
        
        // Test if in this._labels, if so, set true
        if( typeof this._labels[evt.label]!="undefined" ) this._labels[evt.label] = true;
        
        // Test if all complete
        var fail = false;
        for( l in this._labels ) {
            if( !this._labels[l] ) {
                fail = true;
                break;
            }
        }
        if( !fail ) {
            // Success - refresh and hide
            this._main.gmail.environment.requestRefresh();
            this.disconnect();
        }
    },
    
	
});
//gtdExplorerPopup.implement(giEventDispatcher); // Add event dispatching support