/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */



/**
 * A self-contained UI for a thread
 *
 * @class ? @extends giUIObject
 */
var gtdThread = giUIObject.extend({
    // Properties {
    
    
	
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
    	   	
    
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
	
	_elSubject: null,
	
	_elMessages: null,
	
	/** @property {private glThread} ? The associated thread */
	thread: null,
	
	
	/** @property {private gtdLabellingBox} ? The labelling box */
	_labellingBox: null,
	
	
    //}


	// Methods {
	/**
	* @function ?
	* @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdThread object to view', 'gtdThread');
		
		
		this._eventsContainer = new giEventsContainer(this);
		this.base(main);
		
		// Create container
		var doc = main.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.id = "gtdi-thread";
        this.el.setAttribute("gtdi-resizable", "1");
		
		// Subject and link:
		var elH1 = doc.createElement("H1");
		elH1.className = 'gtdi-thread-subject';
		this.el.appendChild( elH1 );
		
		this._elSubject = doc.createElement("SPAN");
		elH1.appendChild( this._elSubject );
		
		var elLink = doc.createElement("IMG");
        elLink.src = 'chrome://gtdinbox/skin/gtd/goto.png';
        elLink.className = 'gtdi-open-in-gmail';
        elLink.title = giI18N.getString('Browser.Popup.openInGmail');
		elH1.appendChild( elLink );
		
		
		// Labelling Box
		this._labellingBox = main.getUIObject('labellingBox');
		this.el.appendChild( this._labellingBox.el );
		
		
		// Messages container
		this._elMessages = doc.createElement("DIV");
		this._elMessages.id = "gtdi-messages";
		this.el.appendChild( this._elMessages );
		
		
		// Event handlers:
		this._eventsContainer.observe(this.el, "click", this._click, false);
		
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		giLogger.log('Disconnecting gtdThread from view', 'gtdThread');
		
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ) {
			if( this[p] && this[p].nodeName && this[p].nodeType ) {
				this[p] = null;
			}
		}
		
		
		if( this.thread ) this.thread.removeEventListener(glThread.LOADED_DETAILS, this._threadLoaded, this);
		this.thread = null;
		
		
		this._labellingBox.disconnect();
		this._labellingBox = null;
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		
		this.base();
	},

	/**
	* Notification our thread object has been updated
	*/
	_threadLoaded: function(event) {
	    this.load(this.thread);
	},

	/**
	* Render a thread
	* @function ?
	* @param {glThead} thread - The thread to render
	*/
	load: function(thread) {
		
		if( this.thread ) {
			this.thread.removeEventListener(glThread.LOADED_DETAILS, this._threadLoaded, this);
		}
                
		
		this.thread = thread;
		this.thread.addEventListener(glThread.LOADED_DETAILS, this._threadLoaded, this);
		
		// Load basic details:
		this._elSubject.innerHTML = thread.subjectSnippet;
		this._labellingBox.load(thread, true);
		
		// Load the messages
		var doc = this.el.ownerDocument;
		this._elMessages.innerHTML = "";

        var threadDetails = this.thread.getThreadDetails(); // Will start it loading details if not yet loaded
        if( this.thread.isLoading() ) return;   // Not ready yet
                
		var messages = this.thread.getThreadDetails().messages;
		for( var i = 0; i < messages.length; i++ ) {
			var elMessage = this._createMessageEl(messages[i]);
			if( i<(messages.length-1) ) elMessage.className += " gtdi-closed";
			else elMessage.className += " gtdi-final";
			
			this._elMessages.appendChild( elMessage );
		}
	},
	
	/**
	* Create an element for a given message
	* @function {Element} ?
	* @param {glThreadMessage} message
	* @return The message element
	*/
	_createMessageEl: function(message) {
		// TODO recycle message elements
		
		var doc = this.el.ownerDocument;
		
		var elMessage = doc.createElement("DIV");
		elMessage.className = "gtdi-message";
		
		// Message Head
		var elMessageHead = doc.createElement("DIV");
		elMessageHead.className = 'gtdi-messagehead';
		elMessage.appendChild( elMessageHead );
		
		var elUl = doc.createElement("UL");
		elMessageHead.appendChild( elUl );
		
		var elMessageFrom = doc.createElement("LI");
		elMessageFrom.className = 'gtdi-messagefrom';
		elMessageFrom.innerHTML = "<span email='" + message.sender.email + "'>" + message.sender.name + "</span>";
		elUl.appendChild( elMessageFrom );
		
		var elMessageSnippet = doc.createElement("LI");
		elMessageSnippet.className = 'gtdi-messagesnippet';
		elMessageSnippet.innerHTML = "<span>" + message.message + "</span>";
		elUl.appendChild( elMessageSnippet );
		
		var elMessageDate = doc.createElement("LI");
		elMessageDate.className = 'gtdi-messagedatetime';
		elMessageDate.innerHTML = "<span>" + message.date + "</span>";
		elUl.appendChild( elMessageDate );
		
		// Message Body
		var elMessageBody = doc.createElement("DIV");
		elMessageBody.className = "gtdi-messagebody";
		elMessage.appendChild( elMessageBody );
		
		elMessageContent = doc.createElement("DIV");
		elMessageContent.className = "gtdi-messagecontent";
		elMessageContent.innerHTML = message.message;
		elMessageBody.appendChild( elMessageContent );
		
		return elMessage;
	},
	
	
	/**
	* Respond to a click of the messages
	* @function ?
	*/
	_click: function(event) {
		try {
			var el = event.target;
			
			if( el.className=="gtdi-open-in-gmail" ) {
				// Load the current thread ID			
				this.dispatchEvent({name:gtdThread.CLICK_OPEN, thread: this.thread, newWindow: event.shiftKey});
			} else {
				
				// Scan up tree to find if a header was clicked
				while( el.parentNode && !giUIObject.hasClassName(el,"gtdi-message") ) el = el.parentNode;
				if( giUIObject.hasClassName(el, "gtdi-message") ) {
					// Clicked header, toggle
					if( giUIObject.hasClassName(el, 'gtdi-closed') ) {
						giUIObject.removeClassName(el, 'gtdi-closed');
					} else {
						giUIObject.addClassName(el, 'gtdi-closed');
					}
				}
				
			}
			
		} catch(e) {
			giLogger.warn(e, "gtdThread._click", this._main.gmail);
		}
	},
	
	/**
	* Set method for updating server. Ensure future created objects can use this info; and existing objects are updated.
	* @function ?
	* @param {String} type - The type. A static variable of glActionManager
	*/
	setServerActionType: function(type) {
		this._labellingBox.setServerActionType(type);
	},

    getLabellingBox: function() {
        return this._labellingBox;
    },

	

},{
	CLICK_OPEN: 'clickOpen',
});
gtdThread.implement(giEventDispatcher); // Add event dispatching support