/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Hold and control the gtdExplorerPopup for the thread lists; used by gtdEnvironment and gtdThreadListCommands.
 *
 * TODO Is this the best kind of class (and in the best location) conceptually for this role?
 *
 * @class ? @extends giUIObject
 */
var gtdThreadListPreviewer = giUIObject.extend({
    // Properties {
            
    
    /**
     * The element used to launch the preview
     * @property {private Element}
     */
    _elThreadListItem: null,
	
    
	
    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
     */
    connect: function(main) {
        this.base(main);
    },
    
    
    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        this.base();
    },
    
    
    
    /**
    * Correctly select an item in the thread list and load the Preview box
    * @function ?
    * @paramset 1
    * @param {Element} elRow - The index in the thread list to use
    * @param {Event} positioningObject - The event from which xy coordinates for the popup are coordinated
    * @paramset 2
    * @param {Element} elRow - The index in the thread list to use
    * @param {Element} positioningObject - The element from which xy coordinates for the popup are coordinated
    */
    load: function(elRow, positioningObject, threadListAlreadyChecked) {
        var doc = this._main.gmail.canvasDocument;
                
        // Store for later use
        this._elThreadListItem = elRow;
        
        // Retrieve the gmail threadListView functionality
        var gmailThreadListView = this._main.gmail.getCurrentViewObject();
	
        /*
        if( !threadListAlreadyChecked ) {
            
            // Uncheck all checkboxes
            gmailThreadListView.uncheckRows();
            
            // Check the deisred offset
            gmailThreadListView.checkRow(offset);
        }
        */
    
        // Uncheck all checkboxes
        gmailThreadListView.uncheckRows();
        
        
        var thread = gmailThreadListView.getThread( elRow );
        
        
        // Create a Browser:
        this._currentBrowser = gtdBrowser.createBrowser(this._main, thread, gtdBrowser.TYPES.THREAD);
        this._currentBrowser.setThreads( gmailThreadListView.getThreads(elRow) );
        
        // Get the popup
        var popup = this._main.getUIObject('browserPopup');
        popup.load(this._currentBrowser);
        popup.setServerActionType(glActionManager.USE_AJAX_SYNC);
        popup.addEventListener(gtdBrowserPopup.HIDE, this._closeThreadPopup, this);
        
        
        // Listen for Preview movement
        this._currentBrowser.addEventListener(gtdBrowserThread.MOVE_NEXT, this._movePreview, this);
        this._currentBrowser.addEventListener(gtdBrowserThread.MOVE_PREVIOUS, this._movePreview, this);
        /*
        this._currentBrowser.addEventListener(gtdBrowserThread.USER_ARCHIVE_COMPLETE, this._threadArchived, this);
        this._currentBrowser.addEventListener(gtdBrowserThread.USER_ARCHIVE_MOVENEXT_COMPLETE, this._threadArchived, this);
        */
        
        // Listen for thread list updates
        gmailThreadListView.getThreadListCache().addEventListener(glThreadListCache.THREAD_LIST_CHANGED, this._threadListCacheChanged, this);
        
        // TODO Listen for thread updates; and when changed, ensure that the popup issues an Instant environment refresh on close. Required to stop glActionManager.performAction from trying to perform one (or at least to clear out all requests -> that's a better way)
        this._main.gmail.actions.addEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);
        this._hadThreadChange = false;
        
        this._selectThreadListRow();
    },
    
    
    /**
     * Notification that the thread was archived (but not moved next)
     * If inbox view, close it and move next
     * @function ?
     */
    /*
    _threadArchived: function(evt) {
        var inInbox = (this._main.gmail.doc.location.hash.indexOf("inbox")>-1) // TODO this will also accept -in:inbox, erroneously.
        
        
        var popup = this._main.getUIObject('browserPopup');
        switch(evt.name) {
            case gtdBrowserThread.USER_ARCHIVE_COMPLETE:
                if( inInbox ) {
                    popup.hide();
                }
                break
        }
    },
    */

    /**
    * Notification that the thread popup has closed.
    * Remove listeners
    * @function ?
    */
    _closeThreadPopup: function() {
        
        // Retrieve the gmail threadListView functionality
        var gmailThreadListView = this._main.gmail.getCurrentViewObject();
        // Uncheck current row
        gmailThreadListView.uncheckRows();
        
        var popup = this._main.getUIObject('browserPopup');
        popup.removeEventListener(gtdBrowserPopup.HIDE, this._closeThreadPopup, this);
        
        this._currentBrowser.removeEventListener(gtdBrowserThread.MOVE_NEXT, this._movePreview, this);
        this._currentBrowser.removeEventListener(gtdBrowserThread.MOVE_PREVIOUS, this._movePreview, this);
        /*
        this._currentBrowser.removeEventListener(gtdBrowserThread.USER_ARCHIVE_COMPLETE, this._threadArchived, this);
        this._currentBrowser.removeEventListener(gtdBrowserThread.USER_ARCHIVE_MOVENEXT_COMPLETE, this._threadArchived, this);
        */
        
        this._main.gmail.actions.removeEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);
        
        gmailThreadListView.getThreadListCache().removeEventListener(glThreadListCache.THREAD_LIST_CHANGED, this._threadListCacheChanged, this);
        
        if( this._hadThreadChange ) {
            this._main.gmail.environment.requestRefresh(true);
        }
        
        this._elThreadListItem = null;
        this._currentBrowser = null;
    },

    /**
     * Notification that the user moved the Preview to a different thread; for which we need to update the selected row.
     * @function {} ?
     * @param {Event} evt
     */
    _threadChanged: function(evt) {
        this._hadThreadChange = true;
    },

    /**
     * Using the currently viewed thread, find its associated row in the Gmail threadlist, and select it.
     * @function {} ?
     */
    _selectThreadListRow: function() {
        
        // Why use text matching rather than index matching?
        // Because if archive, it can take 5 seconds to refresh screen.
        // So, in threadListCache, the index is zero (after archive), but #0 in the rows is still the one that was archived.
        //  So, it continues to select the already archived item.
        // Either archive would necessitate instant refresh (CPU intensive), or we use the text match method.
        
        // Retrieve the gmail threadListView functionality
        var gmailThreadListView = this._main.gmail.getCurrentViewObject();
        // Uncheck current row
        gmailThreadListView.uncheckRows();
        
        // Get the visible thread:
        var thread = this._currentBrowser.getThread();
        var subject = giUtil.stripTags(thread.subjectSnippet);
        var date = giUtil.stripTags(thread.dateSnippet);
        var contact = giUtil.stripTags(thread.contactsSnippet);
        
        var elTable = gmailThreadListView.getThreadListTables()[0];
        var elTrs = elTable.getElementsByTagName("TR");
        var idxs = [];
        for( var i = 0; i < elTrs.length; i++ ) {
            var inner = elTrs[i].innerHTML;
            if( inner.indexOf(subject)>-1 && inner.indexOf(date)>-1 && inner.indexOf(contact)>-1 ) {
                idxs.push(i);
            }
        }
        if( idxs.length>0 ) {
            // Matched it
            if( idxs.length==1 ) {
                gmailThreadListView.checkRow(idxs[0]);
            } else {
                // Get the closest one to the apparent thread list view:
                
                var idxData = null;
                var threadsInView = gmailThreadListView.getThreads();
                for( var i = 0; i < threadsInView.length; i++ ) {
                    if( thread.hasId(threadsInView[i].threadId) ) {
                        // i is the index of the thread:
                        idxData = i;
                        break;
                    }
                }
                
                if( idxData!==null ) {
                    var minDiff = 100;
                    var currentIdx = null;
                    for( var i = 0; i < idxs.length; i++ ) {
                        var testDiff = idxs[i]-idxData;
                        if( testDiff<0 ) testDiff *= -1;
                        if( testDiff < minDiff ) {
                            minDiff = testDiff;
                            currentIdx = idxs[i];
                        }
                    }
                    
                    if( currentIdx ) {
                        gmailThreadListView.checkRow(currentIdx);
                    }
                }
                
            }
        }
        
        /*
        var threadsInView = gmailThreadListView.getThreads();
        for( var i = 0; i < threadsInView.length; i++ ) {
            if( thread.hasId(threadsInView[i].threadId) ) {
                // i is the index of the thread:
                gmailThreadListView.checkRow(i);
                break;
            }
        }
        */
        
    },

    /**
     * Notification that the user moved the Preview to a different thread; for which we need to update the selected row.
     * @function {} ?
     * @param {Event} evt
     */
    _movePreview: function(evt) {
        
        
        this._selectThreadListRow();
        
        /*
        if( this._inhibitMoveThreadId && evt.name==gtdBrowserThread.MOVE_NEXT && this._inhibitMoveThreadId==evt.oldThreadId ) {
            // We hit an edge case, and it was requested that this thread was not moved
            
            // Go back to the earlier thread
            var popup = this._main.getUIObject("browserPopup");
            var threads = this._currentBrowser.getThreads();
            for( var i = 0; i < threads.length; i++ ) {
                if( threads[i].threadId==evt.oldThreadId ) {
                    this._currentBrowser.load( gtdBrowser.TYPES.THREAD, threads[i] );
                    this._currentBrowser.setThreads( threads );
                    break;
                }
            }
            
            this._inhibitMoveThreadId = null;
            // Inhibit checkbox updating:
            return;
        }
        */
        
        
    },
    
    
    /**
     * We need to use the latest set of threads, while maintaining our position.
     * Note THREAD_LIST_CACHE_CHANGED should always fire before MOVE_NEXT (and they're both generated by THREAD_ARCHIVE)
     * @function {} ?
     */
    _threadListCacheChanged: function(evt) {
        
        
        this._selectThreadListRow();
    
        /*
        // Test it is relevant to this view:
        if( evt.key!=glThreadListCache.locationKey( this._main.gmail.doc.location.hash ) ) {
            return;
        }
                
        var gmailThreadListView = this._main.gmail.getCurrentViewObject();
        var popup = this._main.getUIObject("browserPopup");
        
        // Save the threadId prior to reloading the threads
        var threadId = this._currentBrowser.getThread().threadId;
        
        // Get the current threads from the popup
        var oldThreads = this._currentBrowser.getThreads();
        
        // Get the latest threads from the thread cache
        var newThreads = gmailThreadListView.getThreads(this._elThreadListItem);
        
        
        
        // Clear out previous inhibits:
        this._inhibitMoveThreadId = null;
        
        // If threadId no longer found (as in, it has been archived/deleted in inbox), find the first thread that still exists that was before it  (as we wish to maintain the same position, but when MOVE_NEXT fires - as it will after an archive - it will move it forward one)
        // Otherwise, use the same threadId
        if( newThreads.length==0 ) {
            // Close it
            popup.hide();
            return;
        }
        var found = false;
        var newIndex = 0;
        for( var i = 0; i < newThreads.length; i++ ) {
            if( newThreads[i].threadId==threadId ) {
                // Our thread is still here
                found = true;
                this._currentBrowser.load(gtdBrowser.TYPES.THREAD, newThreads[i]);
                this._currentBrowser.setThreads(newThreads);
                newIndex = i;
            }
        }
        if( !found ) {
            // Need to find the first thread id, before the chosen thread id, in the old threads that still exists in the new ones
            for( var i = 0; i < oldThreads.length; i++ ) {
                if( oldThreads[i].threadId==threadId ) {
                    // Match, now work backwards
                    if( i==0 ) {
                        // TODO It was at the start. This is an edge case that will fail with the move next.
                        this._currentBrowser.load(gtdBrowser.TYPES.THREAD, newThreads[0]);
                        this._currentBrowser.setThreads(newThreads);
                        this._inhibitMoveThreadId = newThreads[0].threadId;   // The thread that will be moved from
                        //newIndex = 0;
                    } else {
                        var found2 = false;
                        for( var j = i; j >= 0; j-- ) {
                            var testThreadId = oldThreads[j].threadId;
                            for( var k = 0; k < newThreads.length; k++ ) {
                                if( newThreads[k].threadId==testThreadId ) {
                                    // We found the desired thread still exists in the new threads, use it
                                    found2 = true;
                                    this._currentBrowser.load(gtdBrowser.TYPES.THREAD, newThreads[k]);
                                    this._currentBrowser.setThreads(newThreads);
                                    break;
                                    //newIndex = k;
                                }
                            }
                            if( found2 ) break;
                        }
                        if( !found2 ) {
                            // TODO Second edge case. Presume all previous threads were deleted; leaving us at 0 in newThreads; which will fail on the move next.
                            // TODO We need some way to flag to the threadViewer, such that on its MOVE_NEXT, it inhibits actually moving next.
                            this._currentBrowser.load(gtdBrowser.TYPES.THREAD, newThreads[0]);
                            this._currentBrowser.setThreads(newThreads);
                            this._inhibitMoveThreadId = newThreads[0].threadId;   // Old thread id
                            //newIndex = 0;
                        }
                    }
                }
            }
        }
        */
        
    },
    
    /**
     * Whether the UI object can be cached
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return true;
    },
    
    
    //}
});