/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * The welcome routine for GTD
 *
 * TODO This is another popup (gtdSetup, gtdExplorerPopup). Should we subclass or otherwise start doing more reuse?
 *
 * @class ? @extends giUIObject
 */


var gtdWelcome = giUIObject.extend({
    // Properties {
    
	/** @property {private giEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
	
	/** @property {private Element} - The el that contains the threads-viewer and thread-viewer */
	_elContainer: null,
	
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdWelcome object to view', 'gtdWelcome');
		
		this.base(main);
		this._eventsContainer = new giEventsContainer(this);
		
		// Create container
		var doc = main.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.className = "gtdi gtdi-popup gtdi-gtdwelcome";
		this.el.style.position = "absolute";
		
		this._elContainer = doc.createElement("DIV");
		this._elContainer.id = "gtdi-browser-container";
		this.el.appendChild( this._elContainer );
		
		// Add the close:
		var elClose = doc.createElement("A");
		elClose.className = "gtdi-close";
		elClose.title = giI18N.getString("Browser.Popup.Close");
		elClose.innerHTML = giI18N.getString("Browser.Popup.Close");
		this._elContainer.appendChild( elClose );
		
		// Attach to view:
		this._main.gmail.canvasDocument.body.appendChild(this.el);
        
        // Add content
        var elContent = doc.createElement("DIV");
        elContent.className = 'gtdi-gtdwelcome-container';
        elContent.innerHTML = giFileIO.readFromUrl("chrome://gtdinbox/locale/gtdwelcome.htm");
        this._elContainer.appendChild( elContent );
        
        // Add News Box:
        var box = giPageManager.getPluginInstance(boxMain, this._main.gmail);
        if( box ) {
            var elSidepanel = doc.getElementById('gtdi-gtdwelcome-sidepanel');
            var boxNews = box.getUIObject('news');
            if( !boxNews.isInitiated() ) boxNews.refresh();
            elSidepanel.appendChild( boxNews.el );
        }
		
		// Add Event
		this._eventsContainer.observe(this._elContainer, "click", this._click, false);
        
    
        this._main.gmail.lightbox.show();
        
        // Centre popup:
        var popupDims = giUIObject.getDimensions(this.el);
        var viewport = giUIObject.viewport(this.el.ownerDocument);
                                
        x = (viewport[0]/2)-(popupDims.width/2);
        y = (viewport[1]/2)-(popupDims.height/2) + this.el.ownerDocument.defaultView.pageYOffset;
        
        
        var xy = giUIObject.boundsCheckXY(this._elContainer, x,y,true,true);
        x = xy[0];
        y = xy[1];
        this.el.style.left = x + 'px';
        this.el.style.top = y + 'px';
    
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		giLogger.log('Disconnecting gtdWelcome from view', 'gtdWelcome');
		
        this._main.gmail.lightbox.hide();
                
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ) {
			if( this[p] && this[p].nodeName && this[p].nodeType ) {
				this[p] = null;
			}
		}
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		this.base();
	},
	
	
	
	
	/**
	* Notification that the element was clicked
	* Hide the popup
	* @function ?
	*/
	_click: function(evt) {
		try {
            var el = evt.target;
                        
            if( el.className=='gtdi-close' ) {
                this.disconnect();
            }
		} catch(e) {
			giLogger.warn(e, 'gtdWelcome._clickClose', this._main.gmail);
		}
	},	
	
    
	
	
});