/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * A list of threads, possibly partitioned by project.
 *
 * TODO Reduce code by moving the loading box into its own class. 
 *
 * @class ? @extends giUIObject
 */
var gtdThreadListDownloadProgress = giUIObject.extend({
    // Properties {
    
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,

	/** @property {private Object} ? Elements for the loading boxes. Format: {loading:{el:, elCurrent:, elTotal:}, partialComplete:{el:, elCurrent:, elTotal:}} */
	_loadbox: null,

    /** @property {private gtdThreadContainer} ? The thread container */
	_threadContainer: null,
    
    _eventsContainer: null,
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdThreadListDownloadProgress object to view', 'gtdThreadListDownloadProgress');
		
		this.base(main);
		this._eventsContainer = new giEventsContainer(this);
		
		var doc = main.gmail.canvasDocument;
        this.el = doc.createElement("DIV");
		this.el.id = "gtdi-threadlist-downloadprogress";
        
		
		// Create the loading boxes
		this._loadbox = {loading:{}, partialComplete:{}};
		
		this._loadbox.loading.el = doc.createElement("DIV");
		this._loadbox.loading.el.style.display = "none";
		this._loadbox.loading.el.className = "gtdi-loadingbox";
		this._loadbox.loading.el.innerHTML = "<img src='chrome://gtdinbox/skin/indicator_small.gif'/><div>" + giI18N.getString("ThreadList.Loadbox.loading").replace("%1", "<span></span>").replace("%2", "<span></span>") + "</div>";
		this._loadbox.loading.elCurrent = this._loadbox.loading.el.getElementsByTagName("SPAN")[0];
		this._loadbox.loading.elTotal = this._loadbox.loading.el.getElementsByTagName("SPAN")[1];
		this.el.appendChild( this._loadbox.loading.el );
		
		this._loadbox.partialComplete.el = doc.createElement("DIV");
		this._loadbox.partialComplete.el.className = "gtdi-loadingbox";
		this._loadbox.partialComplete.el.style.display = "none";
		this._loadbox.partialComplete.el.innerHTML = giI18N.getString("ThreadList.Loadbox.partialComplete").replace("%1", "<span></span>").replace("%2", "<span></span>");
		this._loadbox.partialComplete.elCurrent = this._loadbox.partialComplete.el.getElementsByTagName("SPAN")[0];
		this._loadbox.partialComplete.elTotal = this._loadbox.partialComplete.el.getElementsByTagName("SPAN")[1];
		this._loadbox.partialComplete.elDownload = doc.createElement("A");
		this._loadbox.partialComplete.elDownload.className = "gtdi-downloadall";
		this._loadbox.partialComplete.elDownload.href = "#";
		this._loadbox.partialComplete.elDownload.innerHTML = giI18N.getString("ThreadList.Loadbox.partialComplete.all");
		this._loadbox.partialComplete.el.appendChild( this._loadbox.partialComplete.elDownload );
		this.el.appendChild( this._loadbox.partialComplete.el );
		
        this._eventsContainer.observe(this.el, 'click', this._click, false);
	},

	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		giLogger.log('Disconnecting gtdThreadListDownloadProgress from view', 'gtdThreadListDownloadProgress');
		
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		this._unhookThreadContainer();
        
        this._eventsContainer.reset();
        this._eventsContainer = null;
		
		this.base();
	},
	
    _unhookThreadContainer: function() {
        if( this._threadContainer ) {
			this._threadContainer.removeEventListener(gtdThreadContainer.NEW_SEARCH, this._newSearch, this);
			this._threadContainer.removeEventListener(gtdThreadContainer.LOADED, this._loadComplete, this);
			this._threadContainer.removeEventListener(gtdThreadContainer.LOADING_UPDATE, this._loadingUpdate, this);
			this._threadContainer = null;
		}
    },
    
    /**
     * Attach us to a thread container
     * @function {} ?
     * @param {gtdThreadContainer} threadContainer
     */
    setThreadContainer: function(threadContainer) {
        this._unhookThreadContainer();
        
        this._threadContainer = threadContainer;
        if( this._threadContainer ) {
            this._threadContainer.addEventListener(gtdThreadContainer.NEW_SEARCH, this._newSearch, this);
			this._threadContainer.addEventListener(gtdThreadContainer.LOADED, this._loadComplete, this);
			this._threadContainer.addEventListener(gtdThreadContainer.LOADING_UPDATE, this._loadingUpdate, this);
            
            // Set initial state:
            this._refresh();
        }
    },
	
    _refresh: function() {
        
        
        if( this._threadContainer.isSearching() ) {
            // In progress
            this._loadbox.partialComplete.el.style.display = "none";
            var upto = this._threadContainer.threads.length+25;
            var total = this._threadContainer.getNumberThreads(gtdThreadContainer.COUNT_TOTAL_DESIRED);
            if( this._threadContainer.getNumberThreads(gtdThreadContainer.COUNT_TOTAL_CALCULATED) < total ) total = this._threadContainer.getNumberThreads(gtdThreadContainer.COUNT_TOTAL_CALCULATED);
            if( upto > total ) upto = total;
            this._loadbox.loading.elCurrent.innerHTML = this._threadContainer.threads.length + "-" + upto;
            this._loadbox.loading.elTotal.innerHTML = total;
            this._loadbox.loading.el.style.display = "";
        } else {
            // Stopped searching, is it complete?
            if( this._threadContainer.isComplete() ) {
                // It is complete - hide
                this.el.style.display = "none";
            } else {
                // It is partially complete
                this._loadbox.partialComplete.elCurrent.innerHTML = this._threadContainer.threads.length;
                this._loadbox.partialComplete.elTotal.innerHTML = this._threadContainer.getNumberThreads(gtdThreadContainer.COUNT_TOTAL_CALCULATED);
                this._loadbox.partialComplete.el.style.display = "";
                this._loadbox.loading.el.style.display = "none";
                this.el.style.display = "";
            }
        }
        
        
    },
	
	/**
	* Called when the gtdThreadContainer commences a new search
	* @function ?
	*/
	_newSearch: function(event) {
		this._refresh();
	},
	
	/**
	* Called when the gtdThreadContainer has new threads during a search. 
	* @function ?
	*/
	_loadingUpdate: function(event) {
		
		this._refresh();
	},
	
	/**
	* Called when the gtdThreadContainer is loaded with a new search. 
	* @function ?
	*/
	_loadComplete: function(event) {
		
		this._refresh();
	},
	
    /**
     * Notification of a click event
     * @function {} ?
     */
    _click: function(evt) {
        try {
            var el = evt.target;
            if( el.className=="gtdi-downloadall" ) {	// Download All loading link
                this._threadContainer.search();	// download everything in this query
            }
        } catch(e) {
            giLogger.warn(e, 'gtdThreadListDownloadProgress._click');
        }
    },

    //}
});
