/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Logic for grouping threads and groups
 *
 * @class ? @extends giBase
 */
var gtdThreadListGroupBy = giBase.extend({
    // Properties {
    
    /**
     * @property {gtdMainPagePlugin}
     */
    _gtd: null,
    
    /**
     * The ID of this GroupBy
     * @property {String}
     */
    id: null,
    
    /**
     * The localised name of this GroupBy
     * @property {String}
     */
    displayName: null,
    
    /**
     * The full path for the image src
     * @property {String}
     */
    iconSrc: null,
    
    //}


	// Methods {
	
    /**
     * The constructor
     * @function {} ?
     */
    constructor: function(gtd){
        this._gtd = gtd;
    },
    
    /**
     * Return an array of group ids for this thread.
     * @function {String[][]} ?
     * @param {glThread} thread -
     * @return Format: [[group id, group name]]
     */
    groupThread: function(thread) {
        throw new Error("Must override groupThread");
    },
    
    /**
     * A sort function compatible with Javascript's inbuilt Array.sort
     * Compare two threads
     * @function {int} ?
     * @param {gtdThreadListThread} a
     * @param {gtdThreadListThread} b
     * @return -1 if a<b in order, 0 is a==b, and 1 if a>b in order
     */
    sortThreads: function(a, b) {
        throw new Error("Must override sortThreads");
    },
    
    /**
     * A sort function compatible with Javascript's inbuilt Array.sort
     * Compare two groups
     * Default to an alphabetical sort.
     * @function {int} ?
     * @param {gtdThreadListGroup} a
     * @param {gtdThreadListGroup} b
     * @return -1 if a<b in order, 0 is a==b, and 1 if b<a in order
     */
    sortGroups: function(a, b) {
        a = a.id.toLowerCase();
        b = b.id.toLowerCase();
        if(a==b) {
            return 0;
        } else if( a < b ) {
            return -1;
        } else {
            return 1;
        }
    },
    
    /**
     * Return the browser type and item for the given group.
     * If not applicable (e.g. a misc), return null.
     * @function {String[]} ?
     * @param {gtdThreadListGroup} group - The group to evaluate
     * @return An array in the format: [gtdBrowser TYPE, item], or null
     */
    getBrowserTypeAndItem: function(group) {
        return null;
    },
    
    // }

});
