/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Logic for grouping threads and groups
 *
 * @class ? @extends gtdThreadListGroupBy
 */
var gtdThreadListGroupByContact = gtdThreadListGroupBy.extend({
    // Properties {
    
    //}

	// Methods {
	
    /**
     * The constructor
     * @function {} ?
     */
    constructor: function(gtd){
        this.base(gtd);
        
        this.id = "contact";
        this.displayName = giI18N.getString("ThreadList.GroupBy."+this.id);
        this.iconSrc = "chrome://gtdinbox/skin/gtd/i/gtdi-groupby-contact.png";
        
    },
    
    /**
     * Return an array of group ids for this thread.
     * @function {String[][]} ?
     * @param {glThread} thread -
     * @return Format: [[group id, group name]]
     */
    groupThread: function(thread) {
        // TODO Include the contact name. The _addGroup routine would need to take the first 'displayName' for the group when instantiated. Everything else would be waste.
        var ids = [];
        for( var i = 0; i < thread.knownContacts.length; i++ ) {
            if( giUtil.testEmailAddress(thread.knownContacts[i].name) || !thread.knownContacts[i].name ) {
                ids.push([thread.knownContacts[i].email]);
            } else {
                ids.push([thread.knownContacts[i].email, thread.knownContacts[i].name+' ('+thread.knownContacts[i].email+')' ]);
            }
        }
        return ids;
    },
    
    /**
     * A sort function compatible with Javascript's inbuilt Array.sort
     * Compare two threads
     * @function {int} ?
     * @param {gtdThreadListThread} a
     * @param {gtdThreadListThread} b
     * @return -1 if a<b in order, 0 is a==b, and 1 if a>b in order
     */
    sortThreads: null,
    
    /**
     * Return the browser type and item for the given group.
     * If not applicable (e.g. a misc), return null.
     * @function {String[]} ?
     * @param {gtdThreadListGroup} group - The group to evaluate
     * @return An array in the format: [gtdBrowser TYPE, item], or null
     */
    getBrowserTypeAndItem: function(group) {
        return [gtdBrowser.TYPES.CONTACT, group.id];
    },
    
    // }

});
