/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * Logic for grouping threads and groups
 *
 * @class ? @extends gtdThreadListGroupBy
 */
var gtdThreadListGroupByProject = gtdThreadListGroupBy.extend({
    // Properties {
    
    //}

	// Methods {
	
    /**
     * The constructor
     * @function {} ?
     */
    constructor: function(gtd){
        this.base(gtd);
        
        this.id = "project";
        this.displayName = giI18N.getString("ThreadList.GroupBy."+this.id);
        this.iconSrc = "chrome://gtdinbox/skin/gtd/i/gtdi-groupby-project.png";
        
        this._projectPrefix = this._gtd.prefs.getPref("labels.types.projects.prefix");
    },
    
    /**
     * Return an array of group ids for this thread.
     * @function {String[][]} ?
     * @param {glThread} thread -
     * @return Format: [[group id, <optional group name>]]
     */
    groupThread: function(thread) {
        var ids = [];
        for( var i = 0; i < thread.labels.length; i++ ) {
            if( gtdLabelsData.testPrefix(thread.labels[i], this._projectPrefix) ) {
                ids.push([thread.labels[i]]);
            }
        }
        if( ids.length==0 ) ids.push(["misc",giI18N.getString("ThreadList.GroupBy.misc")]);
        return ids;
    },
    
    /**
     * A sort function compatible with Javascript's inbuilt Array.sort
     * Compare two threads
     * @function {int} ?
     * @param {gtdThreadListThread} a
     * @param {gtdThreadListThread} b
     * @return -1 if a<b in order, 0 is a==b, and 1 if a>b in order
     */
    sortThreads: null,
    
    /**
     * Return the browser type and item for the given group.
     * If not applicable (e.g. a misc), return null.
     * @function {String[]} ?
     * @param {gtdThreadListGroup} group - The group to evaluate
     * @return An array in the format: [gtdBrowser TYPE, item], or null
     */
    getBrowserTypeAndItem: function(group) {
        if( group.id=="misc" ) return null;
        return [gtdBrowser.TYPES.PROJECTS, group.id];
    },
    
    // }

});
