/*
 * ***** BEGIN LICENSE BLOCK *****
 * GTDInbox source code. Copyright (2009) of The Inbox Foundry Ltd.
 * No reuse without express permission.
 * If you're interested in utilising or working on this code, please contact Andy Mitchell (andy@inboxfoundry.com).
 * ***** END LICENSE BLOCK ***** */


/**
 * A line item in the thread list. Represents a thread.
 *
 * @class ? @extends giUIObject
 */
var gtdThreadListThread = giUIObject.extend({
    // Properties {
    
    
    
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
    
	/** @property {private Element} ? */
	_elCheckbox: null,
	
	/** @property {private Element} ? */
	_elSubject: null,
	
	/** @property {private Element} ? */
	_elSnippet: null,
	
	/** @property {private Element} ? */
	_elPeople: null,
	
	/** @property {private Element} ? */
	_elAttachment: null,
	
	/** @property {private Element} ? */
	_elDate: null,
	
	/** @property {private Element} ? */
	_elLabels: null,
		
	/** @property {private glThread} ? */
	thread: null,
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} main - The gtdMainPagePlugin object for the page
	*/
	connect: function(main) {
		giLogger.log('Connecting gtdThreadListThread object to view', 'gtdThreadListThread');
		
		this.base(main);
		
		// Create container
		var doc = main.gmail.canvasDocument;
		this.el = doc.createElement("TABLE");
        this.el.setAttribute("width", "100%");
        this.el.className = "gtdi-threadlist-thread";
        
        
        
        var elTr = doc.createElement("TR");
        this.el.appendChild( elTr );
        
        // First row, first col:
        var elTd = doc.createElement("TD");
        elTd.className = "gtdi-threadlist-thread-colfirst"
        elTd.setAttribute("width", "120");
        elTr.appendChild( elTd );
        
        this._elCheckbox = doc.createElement("INPUT");
        this._elCheckbox.setAttribute("type", "checkbox");
        this._elCheckbox.className = "gtdi-selectthread";
        elTd.appendChild( this._elCheckbox );
        
        this._elPeople = doc.createElement("SPAN");
        this._elPeople.className = "gtdi-people";
        elTd.appendChild( this._elPeople );
        
        
        // First row, middle col:
        var elTd = doc.createElement("TD");
        elTd.className = "gtdi-threadlist-thread-colmiddle";
        elTr.appendChild( elTd );
        
        var elSubjectContainer = doc.createElement("DIV");
        elSubjectContainer.className = "gtdi-threadlist-thread-subjectcontainer";
        elTd.appendChild( elSubjectContainer );
        
        this._elSubject = doc.createElement("SPAN");
        this._elSubject.className = "gtdi-subjectline";
        elSubjectContainer.appendChild( this._elSubject );
        
        this._elSnippet = doc.createElement("SPAN");
        this._elSnippet.className = "gtdi-snippet";
        elSubjectContainer.appendChild( this._elSnippet );
        
        
        // First row, final col:
        var elTd = doc.createElement("TD");
        elTd.className = "gtdi-threadlist-thread-collast";
        elTr.appendChild( elTd );
        
        
        this._elAttachment = doc.createElement("IMG");
        this._elAttachment.src = "chrome://gtdinbox/skin/gtd/i/gtdi-attachment.png";
        this._elAttachment.title = giI18N.getString("ThreadList.ThreadAttachment.title");
        elTd.appendChild( this._elAttachment );
        
        this._elDate = doc.createElement("SPAN");
        this._elDate.className = "gtdi-date";
        elTd.appendChild( this._elDate );
        
        var elPopout = doc.createElement('IMG');
        elPopout.src = 'chrome://gtdinbox/skin/gtd/goto.png';
        elPopout.className = 'gtdi-open-in-gmail';
        elPopout.title = giI18N.getString('Browser.Popup.openInGmail');
        elTd.appendChild( elPopout );
        
        
        // Second row:
        var elTr = doc.createElement("TR");
        this.el.appendChild( elTr );
        
        // Second row, first col:
        var elTd = doc.createElement("TD");
        elTd.className = "gtdi-threadlist-thread-colfirst";
        elTr.appendChild( elTd );
        
        // Second row, two cols:
        // First row, final col:
        var elTd = doc.createElement("TD");
        elTd.setAttribute("colspan", "2");
        elTd.className = "gtdi-labels";
        elTr.appendChild( elTd );
        this._elLabels = elTd;
        
        		
	},

	/**
	* Associate a thread with this UI element and update with the latest thread data
	* @function ?
	* @param {glThread} thread - The thread to render
	*/
	load: function(thread) {
		this.el.setAttribute("gtdi-thread-id", thread.threadId);
		this.thread = thread;
		this.refresh();
	},
	
    
	/**
	* Rebuild the data elements from the thread. Preserve checkbox status.
	* @function ?
	*/
	refresh: function() {
		// Basic data:
		this._elSubject.title = giUtil.decodeEntities(this.thread.subjectSnippet);
		this._elSubject.innerHTML = this._elSubject.title;
        var snippet = giUtil.decodeEntities(giUtil.stripTags(this.thread.bodySnippet));
        this._elSnippet.title = snippet;
		this._elSnippet.innerHTML = snippet? ("- "+snippet) : "";
		this._elPeople.innerHTML = this.thread.contactsSnippet? this.thread.contactsSnippet.replace("<b>","").replace("</b>","") : "";
		
        // Set date:
        // Cannot use timestampJS as it refers to date of last activity (e.g. add label), not last message
        var dateSnippet = giUtil.stripTags(this.thread.dateSnippet);
        this._elDate.innerHTML = dateSnippet;
        if( /\d+\/\d+\/\d+/.test(dateSnippet) ) {
            // It's in full date format, 06/12/2008; which Gmail only uses for old items
            this._elDate.style.color = "rgb(153,46,40)";
        } else if( /\d+\s\w+/i.test(dateSnippet) ) {
            // It's in 4 Jul format. Append year and Javascript can parse
            var now = new Date();
            var d = new Date(dateSnippet + " " + now.getFullYear());
            var age = now.valueOf() - d.valueOf();
            if( age < 172800000 ) { // 48 hrs
                this._elDate.style.color = "rgb(0,0,0)"; // Current
            } else if( age < 604800000 ) { // 1 week
                this._elDate.style.color = "rgb(193, 125, 0)"; // Middle
            } else {
                this._elDate.style.color = "rgb(153,46,40)"; // Old
            }
        } else {
            this._elDate.style.color = "rgb(0,0,0)"; // Current
        }
        
		
		
		// Attachment
		if( this.thread.attachmentsCSV ) {
			this._elAttachment.style.display = "";
			this._elAttachment.title = this.thread.attachmentsCSV;
		} else {
			this._elAttachment.style.display = "none";
		}
		
        // Read
        if( this.thread.isRead ) {
            giUIObject.removeClassName(this.el, 'gtdi-unread');
        } else {
            giUIObject.addClassName(this.el, 'gtdi-unread');
        }
		
		// Add labels:
		var doc = this._main.gmail.canvasDocument;
        var labels = this.thread.labels;
        this._elLabels.innerHTML = "";
        var statusEls = [];
        var projectEls = [];
        var otherEls = [];
        var typeSettings = null;
        for( var i = 0; i < labels.length; i++ ) {
            var labelname = labels[i];
            if( labelname.substr(0,1)!="^" ) {	// Avoid internal
                
                if( !typeSettings ) typeSettings = this._main.prefs.getPref('labels.types'); // TODO Can we cache this and periodically refresh?
                
                var elLabel = doc.createElement("DIV");
                elLabel.className = "gtdi-label";
                elLabel.setAttribute("gtdLabelName", labelname);
                elLabel.innerHTML = "<span class='gtdi-labelname'>" + labelname + "</span>"; // <span class='gtdi-labelremove'>x</span>
                
                
                if( gtdLabelsData.testPrefix( labelname, typeSettings.statuses.prefix ) ) {
                    statusEls.push(elLabel);
                } else if( gtdLabelsData.testPrefix( labelname, typeSettings.projects.prefix ) ) {
                    projectEls.push(elLabel);
                } else {
                    otherEls.push(elLabel);
                }
            }
        }
        
        for( var i = 0; i < statusEls.length; i++ ) this._elLabels.appendChild( statusEls[i] );
        for( var i = 0; i < projectEls.length; i++ ) this._elLabels.appendChild( projectEls[i] );
        for( var i = 0; i < otherEls.length; i++ ) this._elLabels.appendChild( otherEls[i] );
        
        if( this._elLabels.childNodes.length==0 ) {
            this._elLabels.innerHTML = "&nbsp;";
        }
        
	},

	/**
	* Find out if this thread has been ticked/selected
	* @function ?
	* @return {boolean} - True if ticked/selected
	*/
	isChecked: function() {
		return this._elCheckbox.checked;
	},


	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function(full) {
		giLogger.log('Disconnecting gtdThreadListThread from view', 'gtdThreadListThread');
		
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
        
        // Non-destructively clean up:
        this._elLabels.innerHTML = "";
		this.thread = null;
        
        if( full ) {
            for( p in this ) {
                if( this.hasOwnProperty(p) ) {
                    this[p] = null;
                }
            }
        } else {
            // Put back in pool:
            gtdThreadListThread.objectPool.push(this);
        }
        
	},

}, {
    
    /**
     * Create a gtdThreadListThread object.
     * Try to reuse existing ones where possible.
     * @function {gtdThreadListThread} ?
     * @param {gtdMainPagePlugin} gtd - The gtd object for the page
     * @param {glThread} thread - The thread object to build it around
     * @return A populated gtdThreadList object
     */
    create: function(gtd, thread) {
        var tlThread = null;
        if( gtdThreadListThread.objectPool.length>0 ) {
            tlThread = gtdThreadListThread.objectPool[0];
            tlThread._elCheckbox.checked = false;
            gtdThreadListThread.objectPool.splice(0, 1);
        } else {
            tlThread = gtd.getUIObject('threadListThread');
        }
        tlThread.load(thread);
        return tlThread;
    },
    
    /**
     * Used to completely clean up the pool of gtdThreadListThread objects.
     * Should be called on page close.
     * @function {} ?
     */
    unload: function() {
        for( var i = 0; i < gtdThreadListThread.objectPool.length; i++ ) {
            gtdThreadListThread.objectPool[i].disconnect(true);
        }
        gtdThreadListThread.objectPool.length = 0;
    },
    
    /**
     * Array of previously instantiated gtdThreadListThread objects.
     * @property {gtdThreadListThread[]}
     */
    objectPool: [],
    
});
