/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */

/**
 * The sidebar list of accounts.
 * @class
 */
var gipAccounts = giBase.extend({
	// Properties {
	
    /**
     * @property {Element} ?
     */
    el: null,	
        
    /**
     * @property {giEventsContainer} ?
     */
    _eventsContainer: null,
        
	// }
	
	/**
	* @constructor ?
	*/
	constructor: function() {
		this.el = document.getElementById('gtdi-accounts');
        this._eventsContainer = new giEventsContainer(this);
        
        
        // Add listener:
        this._eventsContainer.observe(this.el, 'select', this._select, false);
        
	},
    
    /**
     * Refresh the list with user accounts
     * @function {} ?
     */
    load: function() {
        var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
        var accountsData = prefManager.getCharPref('extensions.gtdinbox.account_prefs');
        var json = Components.classes["@mozilla.org/dom/json;1"].createInstance(Components.interfaces.nsIJSON);
        var accounts = json.decode(accountsData);
        
        // Clear out old
        while( this.el.firstChild ) {
            this.el.removeChild( this.el.firstChild );
        }
        
        giLogger.log(accountsData);
        
        // Add new elements:
        for( account in accounts ) {
            giLogger.log('account: ' + account);
            var elLi = document.createElement("listitem");
            elLi.value = account;
            var elLiLabel = document.createElement("label");
            elLiLabel.setAttribute('value', account);
            elLi.appendChild( elLiLabel );
            this.el.appendChild( elLi );
        }
        
        giLogger.log(this.el.childNodes.length);
        giLogger.log('itemCount #1: ' + this.el.itemCount);
        giLogger.log('itemCount #2: ' + this.el.getAttribute('itemCount'));
        this.el = document.getElementById('gtdi-accounts');
        giLogger.log('itemCount #3: ' + this.el.itemCount);
        
        // Select the first
        if( this.el.itemCount>0 ) {
            this.el.selectedIndex = 0;
        }
        
    },
    
    /**
     * Fired when a list item is selected
     * @function {} ?
     */
    _select: function(evt) {
        try {
            giLogger.log('select account');
            // Fire select event:
            if( this.el.selectedItem ) {
                giLogger.log('known account: ' + this.el.selectedItem.value);
                this.dispatchEvent({name: gipAccounts.SELECT_ACCOUNT, account: this.el.selectedItem.value})
            }
        } catch(e) {
            giLogger.warn(e, 'gipAccounts._select');
        }
    },
	
    unload: function() {
        this._eventsContainer.reset();
    },
	
},
{
    SELECT_ACCOUNT: 'select_account',
});
gipAccounts.implement(giEventDispatcher); // Add event dispatching support
