/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com.
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */


var gipPluginGtd = gipPluginBase.extend({
	// Properties {
		
    /**
     * @property {Element}
     */
    _listTypes: null,
    
    /**
     * @property {Element}
     */
    _listStatusesCore: null,
    
    
    /**
     * @property {Element}
     */
    _buttonAddType: null,
    
    
    /**
     * @property {Element}
     */
    _buttonRemoveType: null,
    
        
	// }
	
	/**
     * Take all the preferences in the page and load them
     * @function {} ?
     * @param {gipPluginContainer} pluginContainer - The plugin container for this account/plugin
	*/
	connect: function(pluginContainer) {
        giLogger.log('connect pluginGtd');
		this.base(pluginContainer);
        
        this._listTypes = document.getElementById('gtd-labels-categories');
        this._listStatusesCore = document.getElementById('gtd-labels-defaults-status');
        this._buttonAddType = document.getElementById('gtd-labels-add-category');
        this._buttonRemoveType = document.getElementById('gtd-labels-remove-category');
        
        // Initialise list types:
        while( this._listTypes.firstChild ) this._listTypes.removeChild( this._listTypes.firstChild );
        var types = this._pluginContainer.prefManager.getPref('labels.types');
        for( t in types ) {
            this._createListItem(this._listTypes, t, giI18N.getString('gtd.labels.types.'+t, t), types[t].prefix);
        }
        
        // Initialise list core:
        while( this._listStatusesCore.firstChild ) this._listStatusesCore.removeChild( this._listStatusesCore.firstChild );
        var statusesCore = this._pluginContainer.prefManager.getPref('labels.types.statuses.core');
        for( c in statusesCore ) {
            if( c!='today' ) {
                this._createListItem(this._listStatusesCore, c, giI18N.getString('gtd.labels.types.statuses.core.'+c), statusesCore[c]);
            }
        }
        
        // Add listeners:
        this._eventsContainer.observe(this._listTypes, 'select', this._selectList, false);
        this._eventsContainer.observe(this._listStatusesCore, 'select', this._selectList, false);
        this._eventsContainer.observe(this._buttonAddType, 'command', this._clickTypeButton, false);
        this._eventsContainer.observe(this._buttonRemoveType, 'command', this._clickTypeButton, false);
        
        // Select defaults:
        if( this._listTypes.itemCount>0 ) this._listTypes.selectedIndex = 0;
        if( this._listStatusesCore.itemCount>0 ) this._listStatusesCore.selectedIndex = 0;
        
	},
	
    disconnect: function() {
        
    },
    
    /**
     * Create a two column list item for the given list
     * @function {} ?
     * @param {Element} elList - the list element
     * @param {String} liValue - The value of the ListItem element
     * @param {String} label1 - The label of the first listcell
     * @param {String} label2 - The label of the second listcell
     */
    _createListItem: function(elList, liValue, label1, label2) {
        var elLi = document.createElement('listitem');
        elLi.value = liValue;
        var elListCell1 = document.createElement('listcell');
        elListCell1.setAttribute('label', label1);
        elLi.appendChild( elListCell1 );
        var elListCell2 = document.createElement('listcell');
        elListCell2.setAttribute('label', label2);
        elLi.appendChild( elListCell2 );
        
        elList.appendChild( elLi );
    },
    
    /**
     * Fires when an element changes status
     * Correctly handle events related to our listboxes; and then pass to the base for more generic preference-element handling
     * @function {}?
     */
    _save: function(evt) {
        try {
            
            // Listen for textbox changes and update lists
            var el = evt.target;
            if( el.nodeName=='textbox' ) {
                switch(el.id) {
                    case 'gtd-labels-categories-prefix':
                        // Update the associated list item node:
                        var elLi = this._listTypes.selectedItem;
                        elLi.getElementsByTagName('listcell')[1].setAttribute('label', el.value);
                        break;
                    case 'gtd-labels-defaults-status-text':
                        // Update the associated list item node:
                        var elLi = this._listStatusesCore.selectedItem;
                        giLogger.log('*2 Update list item, currently: ' + elLi.innerHTML);
                        elLi.getElementsByTagName('listcell')[1].setAttribute('label', el.value);
                        // Update preview
                        var elPreview = document.getElementById('gtd-labels-defaults-status-text-label-preview');
                        elPreview.value = this._pluginContainer.prefManager.getPref('labels.types.statuses.prefix')+el.value;
                        break;
                }
            }
            
            this.base(evt);
            
        } catch(e) {
            giLogger.warn(e, 'gipPluginGtd._save');
        }
    },
    
    /**
     * The list was changed, update the text boxes if one of our lists
     * @function {} ?
     */
    _selectList: function(evt) {
        
        try {
            
                        
            
            // Detect a change to the list (TODO may need to move this out and add own 'select' handler):
            var el = evt.target;
            if( el.nodeName=='listbox' && el.selectedItem ) {
                if( el==this._listTypes ) {
                    // Extract the item:
                    var elLi = el.selectedItem;
                    
                    // Update textbox
                    var elTextbox = document.getElementById('gtd-labels-categories-prefix');
                    elTextbox.setAttribute('gtdi-preference', 'labels.types.'+elLi.value +  ".prefix");
                    elTextbox.value = elLi.getElementsByTagName('listcell')[1].getAttribute('label');
                    return;
                }else if( el==this._listStatusesCore ) {
                    // Extract the item:
                    var elLi = el.selectedItem;
                    
                    
                    // Update textbox
                    var elTextbox = document.getElementById('gtd-labels-defaults-status-text');
                    elTextbox.setAttribute('gtdi-preference', 'labels.types.statuses.core.'+elLi.value);
                    elTextbox.value = elLi.getElementsByTagName('listcell')[1].getAttribute('label');
                    
                    // Update preview
                    var elPreview = document.getElementById('gtd-labels-defaults-status-text-label-preview');
                    elPreview.value = this._pluginContainer.prefManager.getPref('labels.types.statuses.prefix')+elTextbox.value;
                    return;
                }
            }
            
        } catch(e) {
            giLogger.warn(e, 'gipPluginGtd._selectList');
        }
    },
    
    /**
     * Notification that the Type add/remove button was clicked. Update preferences/listboxes
     * @function {} ?
     */
	_clickTypeButton: function(evt) {
        try {
            var el = evt.target;
            if( el==this._buttonAddType ) {
                // Get the name/prefix and add it to the list
                
                var type = prompt(giI18N.getString("gtd.labels.types.button.add.getType"));
                if( !type ) return;
                
                var prefix = prompt(giI18N.getString("gtd.labels.types.button.add.getPrefix").replace("%1", type));
                if( !prefix ) return;
                
                // Wrangling around the all important statuses:
                if( type.toLowerCase()=="statuses" ) type = type.toLowerCase();
                
                if( this._pluginContainer.prefManager.setPref('labels.types.'+type) ) {
                    alert(giI18N.getString("gtd.labels.types.button.add.already_exists").replace("%1", type));
                    return;
                }
                
                // Add preference:
                this._pluginContainer.prefManager.setPref('labels.types.'+type, {prefix:prefix}, true);
                
                // Add list item:
                this._createListItem(this._listTypes, type, giI18N.getString('gtd.labels.types.'+type, type), prefix);
                
            }else if( el==this._buttonRemoveType ) {
                // Confirm they want to remove the current item, and remove it from the list
                
                var elLi = this._listTypes.selectedItem;
                var type = elLi.value;
                
                if( type.toLowerCase()=="statuses" ) {
                    alert(giI18N.getString('gtd.labels.types.button.remove.cannot_remove_statuses'));
                    return;
                }
                
                if( !confirm(giI18N.getString('gtd.labels.types.button.remove.confirm').replace('%1', type)) ) {
                    return;
                }
                
                // Clear the preference:
                this._pluginContainer.prefManager.setPref('labels.types.'+type, null, true);
                
                // Remove the list item
                this._listTypes.removeChild( elLi );
                
                // Select the first item:
                if( this._listTypes.itemCount>0 ) this._listTypes.selectedIndex = 0;
                
            }
        } catch(e) {
            giLogger.warn(e, 'gipPluginGtd._clickTypeButton');
        }
    },
    
    
});