/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Originally developed for GTDInbox for Gmail. http://www.gtdinbox.com
 * Find out more about developing with this codebase at http://www.gtdinbox.com/development.htm
 * Anyone who utilises this code is required to include this license notice in all covered code.
 * 
 * ***** END LICENSE BLOCK ***** */


var giI18N = giBase.extend({

	// Properties {
    
    	/** @property {private Object} ? Collection of String Bundles  */
	_stringBundles: null,
	
	// }
	
	// Methods {
    

    /**
    * @constructor ?
    */
    constructor: function() {
        giLogger.log('giI18N object created', 'giI18N');
        this._stringBundles = {};
    },
    
    /**
    * Add a string bundle that can be accessed by getString
    * @function ?
    * @param {string} bundleName - The name of the .properties file that contains the keys/values.
    */
    addStringBundle: function(bundleName) {
        if( this._stringBundles[bundleName] ) return;
        
        if( document.getElementById("gl" + bundleName) ) {	// Pull it out the environment
            this._stringBundles[bundleName] = document.getElementById("gl" + bundleName);
        } else {	// Add to the environment then store a connection to it
            //var sbs = document.createElement("stringbundleset");
            //document.body.appendChild( sbs );
            var sbs = document.getElementById("stringbundleset");
            this._stringBundles[bundleName] = document.createElement("stringbundle");
            this._stringBundles[bundleName].id = "gl" + bundleName;
            this._stringBundles[bundleName].src = "chrome://gtdinbox/locale/" + bundleName + ".properties";
            sbs.appendChild( this._stringBundles[bundleName] );
        }
    },
    
    /**
    * Retrieve the value associated with the key from any known String Bundle
    * @function ?
    * @param {string} key - The name of value to return.
    * @param {string} failsafe - Optional. The value to return if no key match.
    * @param {string} bundleName - Optional. The String Bundle key to use.
    */
    getString: function(key, failsafe, bundleName) {
        if( bundleName ) {
            if( this._stringBundles[bundleName] ) {
                try {
                    return this._stringBundles[bundleName].getString(key);
                } catch(e) {
                    return failsafe || "";
                }
            } else {
                return failsafe || "";
            }
        } else {
            for( sb in this._stringBundles ) {
                try {
                    return this._stringBundles[sb].getString(key);
                } catch(e) {}
            }
            return failsafe || "";
        }
    },
    
    // }
	

});

// Initialise:
var giI18N = new giI18N();